@@ -170,27 +170,27 @@ end;
 
 
 gpick.converters['color_web_hex'] = {
-	human_readable = 'Web: hex code',
+	human_readable = _("Web: hex code"),
 	serialize = gpick.serialize_web_hex,
 	deserialize = gpick.deserialize_web_hex
 }
 gpick.converters['color_web_hex_3_digit'] = {
-	human_readable = 'Web: hex code (3 digits)',
+	human_readable = _("Web: hex code (3 digits)"),
 	serialize = gpick.serialize_web_hex_3_digit,
 	deserialize = gpick.deserialize_web_hex_3_digit
 }
 gpick.converters['color_web_hex_no_hash'] = {
-	human_readable = 'Web: hex code (no hash)',
+	human_readable = _("Web: hex code (no hash symbol)"),
 	serialize = gpick.serialize_web_hex_no_hash,
 	deserialize = gpick.deserialize_web_hex_no_hash
 }
 gpick.converters['color_css_hsl'] = {
-	human_readable = 'CSS: hue saturation lightness',
+	human_readable = _("CSS: hue saturation lightness"),
 	serialize = gpick.serialize_css_hsl,
 	deserialize = nil
 }
 gpick.converters['color_css_rgb'] = {
-	human_readable = 'CSS: red green blue',
+	human_readable = _("CSS: red green blue"),
 	serialize = gpick.serialize_css_rgb,
 	deserialize = gpick.deserialize_css_rgb
 }
@@ -236,12 +236,6 @@ gpick.converters['color_csv'] = {
 	deserialize = nil
 };
 
-gpick.converters['color_csv'] = {
-	human_readable = 'CSV',
-	serialize = gpick.serialize_color_csv,
-	deserialize = nil
-};
-
 gpick.color_serialize = function(converter, color_object)
 	return gpick.converters[converter].serialize(color_object)
 end
@@ -250,3 +244,25 @@ gpick.color_deserialize = function(converter, text, color_object)
 	return gpick.converters[converter].deserialize(text, color_object)
 end
 
+gpick.component_to_text = function(component_type, color)
+	if component_type == 'rgb' then
+		return {round(color:red()*255) .. '', round(color:green()*255) .. '', round(color:blue()*255) .. ''}
+	end
+	if component_type == 'hsl' then
+		return {round(color:hue()*360) .. '', round(color:saturation()*100) .. '', round(color:lightness()*100) .. ''}
+	end
+	if component_type == 'hsv' then
+		return {round(color:hue()*360) .. '', round(color:saturation()*100) .. '', round(color:value()*100) .. ''}
+	end
+	if component_type == 'cmyk' then
+		return {round(color:cyan()*255) .. '', round(color:magenta()*255) .. '', round(color:yellow()*255) .. '', round(color:key_black()*255) .. ''}
+	end
+	if component_type == 'lab' then
+		return {round(color:lab_lightness()) .. '', round(color:lab_a()) .. '', round(color:lab_b()) .. ''}
+	end
+	if component_type == 'lch' then
+		return {round(color:lch_lightness()) .. '', round(color:lch_chroma()) .. '', round(color:lch_hue()) .. ''}
+	end
+	return {}
+end
+
