@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2011, Albertas Vyšniauskas
+ * Copyright (c) 2009-2012, Albertas Vyšniauskas
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -20,6 +20,7 @@
 #include "DragDrop.h"
 
 #include "GlobalStateStruct.h"
+#include "ToolColorNaming.h"
 #include "uiUtilities.h"
 #include "ColorList.h"
 #include "MathUtil.h"
@@ -30,6 +31,7 @@
 #include "CopyPaste.h"
 #include "Converter.h"
 #include "DynvHelpers.h"
+#include "Internationalisation.h"
 
 #include "gtk/LayoutPreview.h"
 #include "layout/Layout.h"
@@ -38,6 +40,7 @@
 #include "uiApp.h"
 
 #include <gdk/gdkkeysyms.h>
+#include <boost/format.hpp>
 
 #include <math.h>
 #include <string.h>
@@ -65,6 +68,25 @@ typedef struct BrightnessDarknessArgs{
 	GlobalState* gs;
 }BrightnessDarknessArgs;
 
+class BrightnessDarknessColorNameAssigner: public ToolColorNameAssigner {
+	protected:
+		stringstream m_stream;
+		const char *m_ident;
+	public:
+		BrightnessDarknessColorNameAssigner(GlobalState *gs):ToolColorNameAssigner(gs){
+		}
+
+		void assign(struct ColorObject *color_object, Color *color, const char *ident){
+			m_ident = ident;
+			ToolColorNameAssigner::assign(color_object, color);
+		}
+
+		virtual std::string getToolSpecificName(struct ColorObject *color_object, Color *color){
+			m_stream.str("");
+			m_stream << color_names_get(m_gs->color_names, color, false) << " " << _("brightness darkness") << " " << m_ident;
+			return m_stream.str();
+		}
+};
 
 static void calc(BrightnessDarknessArgs *args, bool preview, bool save_settings){
 	double brightness = gtk_range_2d_get_x(GTK_RANGE_2D(args->brightness_darkness));
@@ -80,14 +102,14 @@ static void calc(BrightnessDarknessArgs *args, bool preview, bool save_settings)
 	color_rgb_to_hsl(&color, &hsl_orig);
 
 	Box* box;
-	char tmp[4];
+	string name;
 	for (int i = 1; i <= 4; i++){
 		color_copy(&hsl_orig, &hsl);
 		hsl.hsl.lightness = mix_float(hsl.hsl.lightness, mix_float(hsl.hsl.lightness, 1, brightness), i / 4.0); //clamp_float(hsl.hsl.lightness + brightness / 8.0 * i, 0, 1);
         color_hsl_to_rgb(&hsl, &r);
 
-		snprintf(tmp, sizeof(tmp), "b%d", i);
-		box = args->layout_system->GetNamedBox(tmp);
+		name = boost::str(boost::format("b%d") % i);
+		box = args->layout_system->GetNamedBox(name.c_str());
 		if (box && box->style){
 			color_copy(&r, &box->style->color);
 		}
@@ -97,8 +119,8 @@ static void calc(BrightnessDarknessArgs *args, bool preview, bool save_settings)
 		hsl.hsl.lightness = mix_float(hsl.hsl.lightness, mix_float(hsl.hsl.lightness, 0, darkness), i / 4.0); //clamp_float(hsl.hsl.lightness - darkness / 8.0 * i, 0, 1);
         color_hsl_to_rgb(&hsl, &r);
 
-		snprintf(tmp, sizeof(tmp), "c%d", i);
-		box = args->layout_system->GetNamedBox(tmp);
+		name = boost::str(boost::format("c%d") % i);
+		box = args->layout_system->GetNamedBox(name.c_str());
 		if (box && box->style){
 			color_copy(&r, &box->style->color);
 		}
@@ -114,12 +136,16 @@ static void update(GtkWidget *widget, BrightnessDarknessArgs *args ){
 }
 
 static int source_get_color(BrightnessDarknessArgs *args, struct ColorObject** color){
+	Style* style = 0;
 	Color c;
 	if (gtk_layout_preview_get_current_color(GTK_LAYOUT_PREVIEW(args->layout_view), &c) == 0){
+		if (gtk_layout_preview_get_current_style(GTK_LAYOUT_PREVIEW(args->layout_view), &style) != 0){
+			return -1;
+		}
 		*color = color_list_new_color_object(args->gs->colors, &c);
 
-		string name = color_names_get(args->gs->color_names, &c, dynv_get_bool_wd(args->gs->params, "gpick.color_names.imprecision_postfix", true));
-		dynv_set_string((*color)->params, "name", name.c_str());
+		BrightnessDarknessColorNameAssigner name_assigner(args->gs);
+		name_assigner.assign(*color, &c, style->human_name.c_str());
 		return 0;
 	}
 	return -1;
@@ -191,20 +217,18 @@ static void add_to_palette_cb(GtkWidget *widget,  gpointer item) {
 
 	struct ColorObject *color_object;
 	if (source_get_color(args, &color_object)==0){
-		dynv_set_string(color_object->params, "name", "");
 		color_list_add_color_object(args->gs->colors, color_object, 1);
 		color_object_release(color_object);
 	}
 }
 
 static void add_all_to_palette_cb(GtkWidget *widget, BrightnessDarknessArgs *args) {
-
 	struct ColorObject *color_object;
+	BrightnessDarknessColorNameAssigner name_assigner(args->gs);
 
 	for (list<Style*>::iterator i = args->layout_system->styles.begin(); i != args->layout_system->styles.end(); i++){
-
 		color_object = color_list_new_color_object(args->gs->colors, &(*i)->color);
-		dynv_set_string(color_object->params, "name", (*i)->ident_name.c_str());
+		name_assigner.assign(color_object, &(*i)->color, (*i)->human_name.c_str());
 		color_list_add_color_object(args->gs->colors, color_object, 1);
 		color_object_release(color_object);
 	}
@@ -233,18 +257,18 @@ static gboolean button_press_cb (GtkWidget *widget, GdkEventButton *event, Brigh
 		bool selection_avail = gtk_layout_preview_is_selected(GTK_LAYOUT_PREVIEW(args->layout_view));
 		bool edit_avail = gtk_layout_preview_is_editable(GTK_LAYOUT_PREVIEW(args->layout_view));
 
-	    item = gtk_menu_item_new_with_image ("_Add to palette", gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
+	    item = gtk_menu_item_new_with_image (_("_Add to palette"), gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
 	    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	    g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (add_to_palette_cb), args);
 		if (!selection_avail) gtk_widget_set_sensitive(item, false);
 
-	    item = gtk_menu_item_new_with_image ("_Add all to palette", gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
+	    item = gtk_menu_item_new_with_image (_("A_dd all to palette"), gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
 	    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	    g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (add_all_to_palette_cb), args);
 
 	    gtk_menu_shell_append (GTK_MENU_SHELL (menu), gtk_separator_menu_item_new ());
 
-	    item = gtk_menu_item_new_with_mnemonic ("_Copy to clipboard");
+	    item = gtk_menu_item_new_with_mnemonic (_("_Copy to clipboard"));
 	    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
 		if (selection_avail){
@@ -260,12 +284,12 @@ static gboolean button_press_cb (GtkWidget *widget, GdkEventButton *event, Brigh
 
 			gtk_menu_shell_append (GTK_MENU_SHELL (menu), gtk_separator_menu_item_new ());
 
-			item = gtk_menu_item_new_with_image ("_Edit...", gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU));
+			item = gtk_menu_item_new_with_image (_("_Edit..."), gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU));
 			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 			g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (edit_cb), args);
 			if (!selection_avail) gtk_widget_set_sensitive(item, false);
 
-			item = gtk_menu_item_new_with_image ("_Paste", gtk_image_new_from_stock(GTK_STOCK_PASTE, GTK_ICON_SIZE_MENU));
+			item = gtk_menu_item_new_with_image (_("_Paste"), gtk_image_new_from_stock(GTK_STOCK_PASTE, GTK_ICON_SIZE_MENU));
 			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 			g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (paste_cb), args);
 			if (!selection_avail) gtk_widget_set_sensitive(item, false);
@@ -311,6 +335,10 @@ static int set_rgb_color(BrightnessDarknessArgs *args, struct ColorObject* color
 }
 
 static int source_activate(BrightnessDarknessArgs *args){
+
+	transformation::Chain *chain = static_cast<transformation::Chain*>(dynv_get_pointer_wdc(args->gs->params, "TransformationChain", 0));
+	gtk_layout_preview_set_transformation_chain(GTK_LAYOUT_PREVIEW(args->layout_view), chain);
+
 	gtk_statusbar_push(GTK_STATUSBAR(args->statusbar), gtk_statusbar_get_context_id(GTK_STATUSBAR(args->statusbar), "empty"), "");
 	return 0;
 }
@@ -318,7 +346,7 @@ static int source_activate(BrightnessDarknessArgs *args){
 static int source_deactivate(BrightnessDarknessArgs *args){
 	dynv_set_color(args->params, "color", &args->color);
 	calc(args, true, true);
-    return 0;
+	return 0;
 }
 
 static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struct dynvSystem *dynv_namespace){
@@ -354,7 +382,7 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 
 	args->brightness_darkness = widget = gtk_range_2d_new();
 	gtk_range_2d_set_values(GTK_RANGE_2D(widget), dynv_get_float_wd(dynv_namespace, "brightness", 0.5), dynv_get_float_wd(dynv_namespace, "darkness", 0.5));
-	gtk_range_2d_set_axis(GTK_RANGE_2D(widget), "Brightness", "Darkness");
+	gtk_range_2d_set_axis(GTK_RANGE_2D(widget), _("Brightness"), _("Darkness"));
 	g_signal_connect(G_OBJECT(widget), "values_changed", G_CALLBACK(update), args);
     gtk_box_pack_start(GTK_BOX(hbox), widget, false, false, 0);
 
@@ -385,7 +413,7 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 
 	color_copy(color, &args->color);
 	gtk_layout_preview_set_color_named(GTK_LAYOUT_PREVIEW(args->layout_view), color, "main");
-    calc(args, true, false);
+	calc(args, true, false);
 
 	gtk_widget_show_all(hbox);
 
@@ -399,10 +427,11 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 }
 
 int brightness_darkness_source_register(ColorSourceManager *csm){
-    ColorSource *color_source = new ColorSource;
-	color_source_init(color_source, "brightness_darkness", "Brightness Darkness");
+	ColorSource *color_source = new ColorSource;
+	color_source_init(color_source, "brightness_darkness", _("Brightness Darkness"));
 	color_source->implement = (ColorSource* (*)(ColorSource *source, GlobalState *gs, struct dynvSystem *dynv_namespace))source_implement;
-    color_source_manager_add_source(csm, color_source);
+	color_source->default_accelerator = GDK_d;
+	color_source_manager_add_source(csm, color_source);
 	return 0;
 }
 
