@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2010, Albertas Vyšniauskas
+ * Copyright (c) 2009-2012, Albertas Vyšniauskas
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -23,8 +23,41 @@
 #include <iostream>
 using namespace std;
 
-void
-color_rgb_to_hsv(Color* a, Color* b)
+// Constant used for lab->xyz transform. Should be calculated with maximum accuracy possible.
+#define EPSILON (216.0 / 24389.0)
+
+static vector3 references[][2] = {
+	{{{{109.850, 100.000,  35.585}}}, {{{111.144, 100.000,  35.200}}}},
+	{{{{ 98.074, 100.000, 118.232}}}, {{{ 97.285, 100.000, 116.145}}}},
+	{{{{ 96.422, 100.000,  82.521}}}, {{{ 96.720, 100.000,  81.427}}}},
+	{{{{ 95.682, 100.000,  92.149}}}, {{{ 95.799, 100.000,  90.926}}}},
+	{{{{ 95.047, 100.000, 108.883}}}, {{{ 94.811, 100.000, 107.304}}}},
+	{{{{ 94.972, 100.000, 122.638}}}, {{{ 94.416, 100.000, 120.641}}}},
+	{{{{ 99.187, 100.000,  67.395}}}, {{{103.280, 100.000,  69.026}}}},
+	{{{{ 95.044, 100.000, 108.755}}}, {{{ 95.792, 100.000, 107.687}}}},
+	{{{{100.966, 100.000,  64.370}}}, {{{103.866, 100.000,  65.627}}}},
+};
+
+static matrix3x3 sRGB_transformation;
+static matrix3x3 sRGB_transformation_inverted;
+
+static matrix3x3 d65_d50_adaptation_matrix;
+static matrix3x3 d50_d65_adaptation_matrix;
+
+
+void color_init()
+{
+	// constants used below are sRGB working space red, green and blue primaries for D65 reference white
+	color_get_working_space_matrix(0.6400, 0.3300, 0.3000, 0.6000, 0.1500, 0.0600, color_get_reference(REFERENCE_ILLUMINANT_D65, REFERENCE_OBSERVER_2), &sRGB_transformation);
+	matrix3x3_inverse(&sRGB_transformation, &sRGB_transformation_inverted);
+
+	color_get_chromatic_adaptation_matrix(color_get_reference(REFERENCE_ILLUMINANT_D65, REFERENCE_OBSERVER_2), color_get_reference(REFERENCE_ILLUMINANT_D50, REFERENCE_OBSERVER_2), &d65_d50_adaptation_matrix);
+	color_get_chromatic_adaptation_matrix(color_get_reference(REFERENCE_ILLUMINANT_D50, REFERENCE_OBSERVER_2), color_get_reference(REFERENCE_ILLUMINANT_D65, REFERENCE_OBSERVER_2), &d50_d65_adaptation_matrix);
+
+}
+
+
+void color_rgb_to_hsv(const Color* a, Color* b)
 {
 	float min, max, delta;
 
@@ -60,8 +93,7 @@ color_rgb_to_hsv(Color* a, Color* b)
 }
 
 
-void
-color_hsv_to_rgb(Color* a, Color* b)
+void color_hsv_to_rgb(const Color* a, Color* b)
 {
 	float h,v, f, x, y, z;
 	int i;
@@ -109,8 +141,7 @@ color_hsv_to_rgb(Color* a, Color* b)
 	}
 }
 
-void
-color_rgb_to_xyz(Color* a, Color* b, matrix3x3* transformation)
+void color_rgb_to_xyz(const Color* a, Color* b, const matrix3x3* transformation)
 {
 	float R=a->rgb.red, G=a->rgb.green, B=a->rgb.blue;
 
@@ -144,8 +175,7 @@ color_rgb_to_xyz(Color* a, Color* b, matrix3x3* transformation)
 	b->xyz.z = rgb.z;
 }
 
-void
-color_xyz_to_rgb(Color* a, Color* b, matrix3x3* transformation_inverted)
+void color_xyz_to_rgb(const Color* a, Color* b, const matrix3x3* transformation_inverted)
 {
 	vector3 rgb;
 	float R,G,B;
@@ -156,17 +186,17 @@ color_xyz_to_rgb(Color* a, Color* b, matrix3x3* transformation_inverted)
 	B=rgb.z;
 
 	if (R>0.0031308){
-		R=1.055*(pow(R,1/2.4))-0.055;
+		R=1.055*(pow(R,1/2.4f))-0.055;
 	}else{
 		R=12.92*R;
 	}
 	if (G>0.0031308){
-		G=1.055*(pow(G,1/2.4))-0.055;
+		G=1.055*(pow(G,1/2.4f))-0.055;
 	}else{
 		G=12.92*G;
 	}
 	if(B>0.0031308){
-		B=1.055*(pow(B,1/2.4))-0.055;
+		B=1.055*(pow(B,1/2.4f))-0.055;
 	}else{
 		B=12.92*B;
 	}
@@ -178,76 +208,61 @@ color_xyz_to_rgb(Color* a, Color* b, matrix3x3* transformation_inverted)
 
 
 
-void
-color_rgb_to_lab(Color* a, Color* b, vector3* reference_white, matrix3x3* transformation)
+void color_rgb_to_lab(const Color* a, Color* b, const vector3* reference_white, const matrix3x3* transformation, const matrix3x3* adaptation_matrix)
 {
 	Color c;
 	color_rgb_to_xyz(a, &c, transformation);
+	color_xyz_chromatic_adaptation(&c, &c, adaptation_matrix);
 	color_xyz_to_lab(&c, b, reference_white);
 }
 
-void
-color_lab_to_rgb(Color* a, Color* b, vector3* reference_white, matrix3x3* transformation_inverted)
+void color_lab_to_rgb(const Color* a, Color* b, const vector3* reference_white, const matrix3x3* transformation_inverted, const matrix3x3* adaptation_matrix_inverted)
 {
 	Color c;
 	color_lab_to_xyz(a, &c, reference_white);
+	color_xyz_chromatic_adaptation(&c, &c, adaptation_matrix_inverted);
 	color_xyz_to_rgb(&c, b, transformation_inverted);
 }
 
-void
-color_copy(Color* a, Color* b)
+void color_copy(const Color* a, Color* b)
 {
-	b->m.m1=a->m.m1;
-	b->m.m2=a->m.m2;
-	b->m.m3=a->m.m3;
-	b->m.m4=a->m.m4;
+	b->m.m1 = a->m.m1;
+	b->m.m2 = a->m.m2;
+	b->m.m3 = a->m.m3;
+	b->m.m4 = a->m.m4;
+}
+
+void color_add(Color* a, const Color* b)
+{
+	a->m.m1 += b->m.m1;
+	a->m.m2 += b->m.m2;
+	a->m.m3 += b->m.m3;
+	a->m.m4 += b->m.m4;
+}
+
+void color_multiply(Color* a, float b)
+{
+	a->m.m1 *= b;
+	a->m.m2 *= b;
+	a->m.m3 *= b;
+	a->m.m4 *= b;
+}
+
+void color_zero(Color* a)
+{
+	a->m.m1 = 0;
+	a->m.m2 = 0;
+	a->m.m3 = 0;
+	a->m.m4 = 0;
 }
 
 void
-color_add(Color* a, Color* b)
-{
-	a->m.m1+=b->m.m1;
-	a->m.m2+=b->m.m2;
-	a->m.m3+=b->m.m3;
-	a->m.m4+=b->m.m4;
-}
-
-void
-color_multiply(Color* a, float b)
-{
-	a->m.m1*=b;
-	a->m.m2*=b;
-	a->m.m3*=b;
-	a->m.m4*=b;
-}
-
-void
-color_zero(Color* a)
-{
-	a->m.m1=0;
-	a->m.m2=0;
-	a->m.m3=0;
-	a->m.m4=0;
-}
-
-void
-color_get_contrasting(Color* a, Color* b)
+color_get_contrasting(const Color* a, Color* b)
 {
 	Color t;
+	color_rgb_to_lab(a, &t, color_get_reference(REFERENCE_ILLUMINANT_D50, REFERENCE_OBSERVER_2), &sRGB_transformation, &d65_d50_adaptation_matrix);
 
-	static vector3 d65={
-		{{95.047, 100.000, 108.883}}
-	};
-    static matrix3x3 transformation={{		//sRGB transformation matrix
-    	{0.4124564,  0.3575761,  0.1804375},
-		{0.2126729,  0.7151522,  0.0721750},
-		{0.0193339,  0.1191920,  0.9503041},
-    }};
-
-	color_rgb_to_lab(a, &t, &d65, &transformation);
-
-
-	if (t.lab.L>3){
+	if (t.lab.L > 50){
 		t.hsv.value=0;
 	}else{
 		t.hsv.value=1;
@@ -259,21 +274,23 @@ color_get_contrasting(Color* a, Color* b)
 	color_hsv_to_rgb(&t, b);
 }
 
-void
-color_set(Color* a, float value)
+void color_set(Color* a, float value)
 {
 	a->rgb.red = a->rgb.green = a->rgb.blue = value;
 }
 
-Color* color_new() {
+Color* color_new()
+{
 	return new Color;
 }
 
-void color_destroy(Color *a) {
+void color_destroy(Color *a)
+{
 	delete a;
 }
 
-void color_rgb_to_hsl(Color* a, Color* b) {
+void color_rgb_to_hsl(const Color* a, Color* b)
+{
 	float min, max, delta;
 
 	min = min_float_3(a->rgb.red, a->rgb.green, a->rgb.blue);
@@ -310,7 +327,8 @@ void color_rgb_to_hsl(Color* a, Color* b) {
 }
 
 
-void color_hsl_to_rgb(Color* a, Color* b) {
+void color_hsl_to_rgb(const Color* a, Color* b)
+{
 	if (a->hsl.saturation == 0) {
 		b->rgb.red = b->rgb.green = b->rgb.blue = a->hsl.lightness;
 	} else {
@@ -361,57 +379,94 @@ void color_hsl_to_rgb(Color* a, Color* b) {
 	}
 }
 
-void color_lab_to_lch(Color* a, Color* b) {
-	float H;
-	H = atan2(a->lab.b, a->lab.a);
+void color_lab_to_lch(const Color* a, Color* b)
+{
+	double H;
+	if (a->lab.a == 0 && a->lab.b == 0){
+		H = 0;
+	}else{
+		H = atan2(a->lab.b, a->lab.a);
+	}
 
-	if (H > 0)
-		H /= PI;
-	else
-		H = (2*PI - abs_float(H))/PI;
+	H *= 180.0 / PI;
+
+	if (H < 0) H += 360;
+	if (H >= 360) H -= 360;
 
 	b->lch.L = a->lab.L;
-	b->lch.C = sqrt(a->lab.b * a->lab.b + a->lab.a * a->lab.a);
+	b->lch.C = sqrt(a->lab.a * a->lab.a + a->lab.b * a->lab.b);
 	b->lch.h = H;
 }
 
-void color_rgb_to_lch(Color* a, Color* b){
+void color_lch_to_lab(const Color* a, Color* b)
+{
+	b->lab.L = a->lch.L;
+	b->lab.a = a->lch.C * cos(a->lch.h * PI / 180.0);
+	b->lab.b = a->lch.C * sin(a->lch.h * PI / 180.0);
+}
+
+void color_rgb_to_lch_d50(const Color* a, Color* b)
+{
 	Color c;
-
-	static vector3 d65={
-		{{95.047, 100.000, 108.883}}
-	};
-    static matrix3x3 transformation={{		//sRGB transformation matrix
-		{0.4124564,  0.3575761,  0.1804375},
-		{0.2126729,  0.7151522,  0.0721750},
-		{0.0193339,  0.1191920,  0.9503041}
-    }};
-
-	color_rgb_to_lab(a, &c, &d65, &transformation);
+	color_rgb_to_lab(a, &c, color_get_reference(REFERENCE_ILLUMINANT_D50, REFERENCE_OBSERVER_2), &sRGB_transformation, &d65_d50_adaptation_matrix);
 	color_lab_to_lch(&c, b);
 }
 
-void color_xyz_to_lab(Color* a, Color* b, vector3* reference_white){
+void color_lch_to_rgb_d50(const Color* a, Color* b)
+{
+	Color c;
+	color_lch_to_lab(a, &c);
+	color_lab_to_rgb(&c, b, color_get_reference(REFERENCE_ILLUMINANT_D50, REFERENCE_OBSERVER_2), &sRGB_transformation_inverted, &d50_d65_adaptation_matrix);
+}
+
+void color_rgb_to_lch(const Color* a, Color* b, const vector3* reference_white, const matrix3x3* transformation, const matrix3x3* adaptation_matrix)
+{
+	Color c;
+	color_rgb_to_lab(a, &c, reference_white, transformation, adaptation_matrix);
+	color_lab_to_lch(&c, b);
+}
+
+void color_lch_to_rgb(const Color* a, Color* b, const vector3* reference_white, const matrix3x3* transformation_inverted, const matrix3x3* adaptation_matrix_inverted)
+{
+	Color c;
+	color_lch_to_lab(a, &c);
+	color_lab_to_rgb(&c, b, reference_white, transformation_inverted, adaptation_matrix_inverted);
+}
+
+void color_rgb_to_lab_d50(const Color* a, Color* b)
+{
+	color_rgb_to_lab(a, b, color_get_reference(REFERENCE_ILLUMINANT_D50, REFERENCE_OBSERVER_2), &sRGB_transformation, &d65_d50_adaptation_matrix);
+}
+
+void color_lab_to_rgb_d50(const Color* a, Color* b)
+{
+	color_lab_to_rgb(a, b, color_get_reference(REFERENCE_ILLUMINANT_D50, REFERENCE_OBSERVER_2), &sRGB_transformation_inverted, &d50_d65_adaptation_matrix);
+}
+
+#define Kk (24389.0 / 27.0)
+
+void color_xyz_to_lab(const Color* a, Color* b, const vector3* reference_white)
+{
 	float X,Y,Z;
 
 	X = a->xyz.x / reference_white->x; //95.047f;
 	Y = a->xyz.y / reference_white->y; //100.000f;
 	Z = a->xyz.z / reference_white->z; //108.883f;
 
-	if (X>0.008856){
-		X=pow(X,1.0/3.0);
+	if (X>EPSILON){
+		X=pow(X,1.0f/3.0f);
 	}else{
-		X=(7.787*X)+(16.0/116.0);
+		X=(Kk*X+16.0f)/116.0f;
 	}
-	if (Y>0.008856){
-		Y=pow(Y,1.0/3.0);
+	if (Y>EPSILON){
+		Y=pow(Y,1.0f/3.0f);
 	}else{
-		Y=(7.787*Y)+(16.0/116.0);
+		Y=(Kk*Y+16.0f)/116.0f;
 	}
-	if (Z>0.008856){
-		Z=pow(Z,1.0/3.0);
+	if (Z>EPSILON){
+		Z=pow(Z,1.0f/3.0f);
 	}else{
-		Z=(7.787*Z)+(16.0/116.0);
+		Z=(Kk*Z+16.0f)/116.0f;
 	}
 
 	b->lab.L=(116*Y)-16;
@@ -420,29 +475,29 @@ void color_xyz_to_lab(Color* a, Color* b, vector3* reference_white){
 
 }
 
-void color_lab_to_xyz(Color* a, Color* b, vector3* reference_white) {
+void color_lab_to_xyz(const Color* a, Color* b, const vector3* reference_white)
+{
 	float x, y, z;
 
 	float fy = (a->lab.L + 16) / 116;
 	float fx = a->lab.a / 500 + fy;
 	float fz = fy - a->lab.b / 200;
 
-	float e=0.008856;
-	float K=903.3;
+	const double K=(24389.0 / 27.0);
 
-	if (pow(fx, 3)>e){
+	if (pow(fx, 3)>EPSILON){
 		x=pow(fx, 3);
 	}else{
 		x=(116*fx-16)/K;
 	}
 
-	if (a->lab.L > K*e){
+	if (a->lab.L > K*EPSILON){
 		y=pow((a->lab.L+16)/116, 3);
 	}else{
 		y=a->lab.L/K;
 	}
 
-	if (pow(fz, 3)>e){
+	if (pow(fz, 3)>EPSILON){
 		z=pow(fz, 3);
 	}else{
 		z=(116*fz-16)/K;
@@ -453,7 +508,8 @@ void color_lab_to_xyz(Color* a, Color* b, vector3* reference_white) {
 	b->xyz.z = z * reference_white->z; //108.883f;
 }
 
-void color_get_working_space_matrix(float xr, float yr, float xg, float yg, float xb, float yb, vector3* reference_white, matrix3x3* result){
+void color_get_working_space_matrix(float xr, float yr, float xg, float yg, float xb, float yb, const vector3* reference_white, matrix3x3* result)
+{
 	float Xr,Yr,Zr;
 	float Xg,Yg,Zg;
 	float Xb,Yb,Zb;
@@ -489,7 +545,8 @@ void color_get_working_space_matrix(float xr, float yr, float xg, float yg, floa
 	result->m[0][2]=Xb*v.z;	result->m[1][2]=Yb*v.z;	result->m[2][2]=Zb*v.z;
 }
 
-void color_get_chromatic_adaptation_matrix(vector3* source_reference_white, vector3* destination_reference_white, matrix3x3* result){
+void color_get_chromatic_adaptation_matrix(const vector3* source_reference_white, const vector3* destination_reference_white, matrix3x3* result)
+{
 
 	matrix3x3 Ma;
 	//Bradford matrix
@@ -517,7 +574,8 @@ void color_get_chromatic_adaptation_matrix(vector3* source_reference_white, vect
 	matrix3x3_multiply(&M, &Ma_inv, result);
 }
 
-void color_xyz_chromatic_adaptation(Color* a, Color* result, matrix3x3* adaptation ){
+void color_xyz_chromatic_adaptation(const Color* a, Color* result, const matrix3x3* adaptation)
+{
 	vector3 x;
 	x.x=a->xyz.x;
 	x.y=a->xyz.y;
@@ -528,19 +586,22 @@ void color_xyz_chromatic_adaptation(Color* a, Color* result, matrix3x3* adaptati
 	result->xyz.z=x.z;
 }
 
-void color_rgb_to_cmy(Color* a, Color* b){
+void color_rgb_to_cmy(const Color* a, Color* b)
+{
 	b->cmy.c = 1 - a->rgb.red;
 	b->cmy.m = 1 - a->rgb.green;
 	b->cmy.y = 1 - a->rgb.blue;
 }
 
-void color_cmy_to_rgb(Color* a, Color* b){
+void color_cmy_to_rgb(const Color* a, Color* b)
+{
 	b->rgb.red = 1 - a->cmy.c;
 	b->rgb.green = 1 - a->cmy.m;
 	b->rgb.blue = 1 - a->cmy.y;
 }
 
-void color_cmy_to_cmyk(Color* a, Color* b){
+void color_cmy_to_cmyk(const Color* a, Color* b)
+{
 	float k = 1;
 
 	if (a->cmy.c < k) k = a->cmy.c;
@@ -557,33 +618,38 @@ void color_cmy_to_cmyk(Color* a, Color* b){
 	b->cmyk.k = k;
 }
 
-void color_cmyk_to_cmy(Color* a, Color* b){
+void color_cmyk_to_cmy(const Color* a, Color* b)
+{
 	b->cmy.c = (a->cmyk.c * (1 - a->cmyk.k) + a->cmyk.k);
 	b->cmy.m = (a->cmyk.m * (1 - a->cmyk.k) + a->cmyk.k);
 	b->cmy.y = (a->cmyk.y * (1 - a->cmyk.k) + a->cmyk.k);
 }
 
-void color_rgb_to_cmyk(Color* a, Color* b){
+void color_rgb_to_cmyk(const Color* a, Color* b)
+{
 	Color c;
 	color_rgb_to_cmy(a, &c);
 	color_cmy_to_cmyk(&c, b);
 }
 
-void color_cmyk_to_rgb(Color* a, Color* b){
+void color_cmyk_to_rgb(const Color* a, Color* b)
+{
 	Color c;
 	color_cmyk_to_cmy(a, &c);
 	color_cmy_to_rgb(&c, b);
 }
 
 
-void color_rgb_normalize(Color* a){
+void color_rgb_normalize(Color* a)
+{
 	a->rgb.red = clamp_float(a->rgb.red, 0, 1);
 	a->rgb.green = clamp_float(a->rgb.green, 0, 1);
 	a->rgb.blue = clamp_float(a->rgb.blue, 0, 1);
 }
 
-void color_hsl_to_hsv(Color *a, Color *b){
-    float l = a->hsl.lightness * 2.0;
+void color_hsl_to_hsv(const Color *a, Color *b)
+{
+	float l = a->hsl.lightness * 2.0;
 	float s = a->hsl.saturation * ((l <= 1.0) ? (l) : (2.0 - l));
 
 	b->hsv.hue = a->hsl.hue;
@@ -596,7 +662,8 @@ void color_hsl_to_hsv(Color *a, Color *b){
 	}
 }
 
-void color_hsv_to_hsl(Color *a, Color *b){
+void color_hsv_to_hsl(const Color *a, Color *b)
+{
 	float l = (2.0 - a->hsv.saturation) * a->hsv.value;
 	float s = (a->hsv.saturation * a->hsv.value) / ((l <= 1.0) ? (l) : (2 - l));
     if (l == 0) s = 0;
@@ -606,3 +673,93 @@ void color_hsv_to_hsl(Color *a, Color *b){
 	b->hsl.lightness = l / 2.0;
 }
 
+void color_rgb_get_linear(const Color* a, Color* b)
+{
+	b->rgb.red = pow(a->rgb.red, 1.0f / 2.1f);
+	b->rgb.green = pow(a->rgb.green, 1.0f / 2.0f);
+	b->rgb.blue = pow(a->rgb.blue, 1.0f / 2.1f);
+}
+
+void color_linear_get_rgb(const Color* a, Color* b)
+{
+	b->rgb.red = pow(a->rgb.red, 2.1f);
+	b->rgb.green = pow(a->rgb.green, 2.0f);
+	b->rgb.blue = pow(a->rgb.blue, 2.1f);
+}
+
+const matrix3x3* color_get_sRGB_transformation_matrix()
+{
+	return &sRGB_transformation;
+}
+
+const matrix3x3* color_get_inverted_sRGB_transformation_matrix()
+{
+	return &sRGB_transformation_inverted;
+}
+
+const matrix3x3* color_get_d65_d50_adaptation_matrix()
+{
+	return &d65_d50_adaptation_matrix;
+}
+
+const matrix3x3* color_get_d50_d65_adaptation_matrix()
+{
+	return &d50_d65_adaptation_matrix;
+}
+
+const vector3* color_get_reference(ReferenceIlluminant illuminant, ReferenceObserver observer)
+{
+	return &references[illuminant][observer];
+}
+
+const ReferenceIlluminant color_get_illuminant(const char *illuminant)
+{
+	const struct{
+		const char *label;
+		ReferenceIlluminant illuminant;
+	}illuminants[] = {
+		{"A", REFERENCE_ILLUMINANT_A},
+		{"C", REFERENCE_ILLUMINANT_C},
+		{"D50", REFERENCE_ILLUMINANT_D50},
+		{"D55", REFERENCE_ILLUMINANT_D55},
+		{"D65", REFERENCE_ILLUMINANT_D65},
+		{"D75", REFERENCE_ILLUMINANT_D75},
+		{"F2", REFERENCE_ILLUMINANT_F2},
+		{"F7", REFERENCE_ILLUMINANT_F7},
+		{"F11", REFERENCE_ILLUMINANT_F11},
+		{0, REFERENCE_ILLUMINANT_D50},
+	};
+	for (int i = 0; illuminants[i].label; i++){
+		if (string(illuminants[i].label).compare(illuminant) == 0){
+			return illuminants[i].illuminant;
+		}
+	}
+	return REFERENCE_ILLUMINANT_D50;
+};
+
+const ReferenceObserver color_get_observer(const char *observer)
+{
+	const struct{
+		const char *label;
+		ReferenceObserver observer;
+	}observers[] = {
+		{"2", REFERENCE_OBSERVER_2},
+		{"10", REFERENCE_OBSERVER_10},
+		{0, REFERENCE_OBSERVER_2},
+	};
+	for (int i = 0; observers[i].label; i++){
+		if (string(observers[i].label).compare(observer) == 0){
+			return observers[i].observer;
+		}
+	}
+	return REFERENCE_OBSERVER_2;
+}
+
+bool color_is_rgb_out_of_gamut(const Color* a)
+{
+	if (a->rgb.red < 0 || a->rgb.red > 1) return true;
+	if (a->rgb.green < 0 || a->rgb.green > 1) return true;
+	if (a->rgb.blue < 0 || a->rgb.blue > 1) return true;
+	return false;
+}
+
