@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2011, Albertas Vyšniauskas
+ * Copyright (c) 2009-2012, Albertas Vyšniauskas
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -20,6 +20,7 @@
 #include "DragDrop.h"
 
 #include "GlobalStateStruct.h"
+#include "ToolColorNaming.h"
 #include "uiUtilities.h"
 #include "ColorList.h"
 #include "MathUtil.h"
@@ -31,10 +32,12 @@
 #include "CopyPaste.h"
 #include "Converter.h"
 #include "DynvHelpers.h"
+#include "Internationalisation.h"
 
 #include "uiApp.h"
 
 #include <gdk/gdkkeysyms.h>
+#include <boost/format.hpp>
 
 #include <math.h>
 #include <string.h>
@@ -59,14 +62,14 @@ typedef struct ColorMixerType{
 	int mode_id;
 }ColorMixerType;
 
-static const ColorMixerType color_mixer_types[] = {
-	{"Normal", "normal", MODE_ID_NORMAL},
-	{"Multiply", "multiply", MODE_ID_MULTIPLY},
-	{"Add", "add", MODE_ID_ADD},
-	{"Difference", "difference", MODE_ID_DIFFERENCE},
-	{"Hue", "hue", MODE_ID_HUE},
-	{"Saturation", "saturation", MODE_ID_SATURATION},
-	{"Lightness", "lightness", MODE_ID_LIGHTNESS},
+const ColorMixerType color_mixer_types[] = {
+	{N_("Normal"), "normal", MODE_ID_NORMAL},
+	{N_("Multiply"), "multiply", MODE_ID_MULTIPLY},
+	{N_("Add"), "add", MODE_ID_ADD},
+	{N_("Difference"), "difference", MODE_ID_DIFFERENCE},
+	{N_("Hue"), "hue", MODE_ID_HUE},
+	{N_("Saturation"), "saturation", MODE_ID_SATURATION},
+	{N_("Lightness"), "lightness", MODE_ID_LIGHTNESS},
 };
 
 typedef struct ColorMixerArgs{
@@ -82,7 +85,7 @@ typedef struct ColorMixerArgs{
 	GtkWidget *color_previews;
 
 	const ColorMixerType *mixer_type;
-  struct{
+	struct{
 		GtkWidget *input;
 		GtkWidget *output;
 	}color[MAX_COLOR_LINES];
@@ -93,10 +96,31 @@ typedef struct ColorMixerArgs{
 	GlobalState* gs;
 }ColorMixerArgs;
 
+class ColorMixerColorNameAssigner: public ToolColorNameAssigner {
+	protected:
+		stringstream m_stream;
+		const char *m_ident;
+	public:
+		ColorMixerColorNameAssigner(GlobalState *gs):ToolColorNameAssigner(gs){
+		}
+
+		void assign(struct ColorObject *color_object, Color *color, const char *ident){
+			m_ident = ident;
+			ToolColorNameAssigner::assign(color_object, color);
+		}
+
+		virtual std::string getToolSpecificName(struct ColorObject *color_object, Color *color){
+			m_stream.str("");
+			m_stream << color_names_get(m_gs->color_names, color, false) << " " << _("color mixer") << " " << m_ident;
+			return m_stream.str();
+		}
+};
 
 static int set_rgb_color(ColorMixerArgs *args, struct ColorObject* color, uint32_t color_index);
 static int set_rgb_color_by_widget(ColorMixerArgs *args, struct ColorObject* color, GtkWidget* color_widget);
 
+
+
 static void calc(ColorMixerArgs *args, bool preview, bool save_settings){
 
 	double opacity = gtk_range_get_value(GTK_RANGE(args->opacity));
@@ -112,24 +136,24 @@ static void calc(ColorMixerArgs *args, bool preview, bool save_settings){
 
 		switch (args->mixer_type->mode_id){
 		case MODE_ID_NORMAL:
-      r.rgb.red = color2.rgb.red;
-      r.rgb.green = color2.rgb.green;
-      r.rgb.blue = color2.rgb.blue;
+			r.rgb.red = color2.rgb.red;
+			r.rgb.green = color2.rgb.green;
+			r.rgb.blue = color2.rgb.blue;
 			break;
 		case MODE_ID_MULTIPLY:
-      r.rgb.red = color.rgb.red * color2.rgb.red;
-      r.rgb.green = color.rgb.green * color2.rgb.green;
-      r.rgb.blue = color.rgb.blue * color2.rgb.blue;
+			r.rgb.red = color.rgb.red * color2.rgb.red;
+			r.rgb.green = color.rgb.green * color2.rgb.green;
+			r.rgb.blue = color.rgb.blue * color2.rgb.blue;
 			break;
 		case MODE_ID_ADD:
-      r.rgb.red = clamp_float(color.rgb.red + color2.rgb.red, 0, 1);
-      r.rgb.green = clamp_float(color.rgb.green + color2.rgb.green, 0, 1);
-      r.rgb.blue = clamp_float(color.rgb.blue + color2.rgb.blue, 0, 1);
+			r.rgb.red = clamp_float(color.rgb.red + color2.rgb.red, 0, 1);
+			r.rgb.green = clamp_float(color.rgb.green + color2.rgb.green, 0, 1);
+			r.rgb.blue = clamp_float(color.rgb.blue + color2.rgb.blue, 0, 1);
 			break;
 		case MODE_ID_DIFFERENCE:
-      r.rgb.red = fabs(color.rgb.red - color2.rgb.red);
-      r.rgb.green = fabs(color.rgb.green - color2.rgb.green);
-      r.rgb.blue = fabs(color.rgb.blue - color2.rgb.blue);
+			r.rgb.red = fabs(color.rgb.red - color2.rgb.red);
+			r.rgb.green = fabs(color.rgb.green - color2.rgb.green);
+			r.rgb.blue = fabs(color.rgb.blue - color2.rgb.blue);
 			break;
 		case MODE_ID_HUE:
 			color_rgb_to_hsv(&color, &hsv1);
@@ -198,42 +222,67 @@ static void on_color_edit(GtkWidget *widget,  gpointer item) {
 	color_object_release(color_object);
 }
 
+static boost::format format_ignore_arg_errors(const std::string &f_string) {
+	boost::format fmter(f_string);
+	fmter.exceptions(boost::io::all_error_bits ^ (boost::io::too_many_args_bit | boost::io::too_few_args_bit));
+	return fmter;
+}
 
-static void on_color_add_to_palette(GtkWidget *widget,  gpointer item) {
-	ColorMixerArgs* args = (ColorMixerArgs*)item;
+static string identify_color_widget(GtkWidget *widget, ColorMixerArgs *args)
+{
+	if (args->secondary_color == widget){
+		return _("secondary");
+	}else for (int i = 0; i < MAX_COLOR_LINES; ++i){
+		if (args->color[i].input == widget){
+			try{
+				return (format_ignore_arg_errors(_("primary %d")) % (i + 1)).str();
+			}catch(const boost::io::format_error &e){
+				return (format_ignore_arg_errors("primary %d") % (i + 1)).str();
+			}
+		}else if (args->color[i].output == widget){
+			try{
+				return (format_ignore_arg_errors(_("result %d")) % (i + 1)).str();
+			}catch(const boost::io::format_error &e){
+				return (format_ignore_arg_errors("result %d") % (i + 1)).str();
+			}
+		}
+	}
+	return "unknown";
+}
+
+static void add_color_to_palette(GtkWidget *color_widget, ColorMixerColorNameAssigner &name_assigner, ColorMixerArgs *args)
+{
 	Color c;
-
-	gtk_color_get_color(GTK_COLOR(g_object_get_data(G_OBJECT(widget), "color_widget")), &c);
-
-	struct ColorObject *color_object = color_list_new_color_object(args->gs->colors, &c);
-	string name = color_names_get(args->gs->color_names, &c, dynv_get_bool_wd(args->gs->params, "gpick.color_names.imprecision_postfix", true));
-	dynv_set_string(color_object->params, "name", name.c_str());
+	struct ColorObject *color_object;
+	string widget_ident;
+	gtk_color_get_color(GTK_COLOR(color_widget), &c);
+	color_object = color_list_new_color_object(args->gs->colors, &c);
+	widget_ident = identify_color_widget(color_widget, args);
+	name_assigner.assign(color_object, &c, widget_ident.c_str());
 	color_list_add_color_object(args->gs->colors, color_object, 1);
 	color_object_release(color_object);
 }
 
+static void on_color_add_to_palette(GtkWidget *widget, gpointer item) {
+	ColorMixerArgs* args = (ColorMixerArgs*)item;
+    GtkWidget *color_widget = GTK_WIDGET(g_object_get_data(G_OBJECT(widget), "color_widget"));
+	ColorMixerColorNameAssigner name_assigner(args->gs);
+	add_color_to_palette(color_widget, name_assigner, args);
+}
+
 static void on_color_add_all_to_palette(GtkWidget *widget,  gpointer item) {
 	ColorMixerArgs* args = (ColorMixerArgs*)item;
-	Color c;
+	ColorMixerColorNameAssigner name_assigner(args->gs);
 
 	for (int i = 0; i < MAX_COLOR_LINES; ++i){
-		gtk_color_get_color(GTK_COLOR(args->color[i].input), &c);
-
-		struct ColorObject *color_object = color_list_new_color_object(args->gs->colors, &c);
-		string name = color_names_get(args->gs->color_names, &c, dynv_get_bool_wd(args->gs->params, "gpick.color_names.imprecision_postfix", true));
-		dynv_set_string(color_object->params, "name", name.c_str());
-		color_list_add_color_object(args->gs->colors, color_object, 1);
-		color_object_release(color_object);
-
-		gtk_color_get_color(GTK_COLOR(args->color[i].output), &c);
-
-		color_object = color_list_new_color_object(args->gs->colors, &c);
-		name = color_names_get(args->gs->color_names, &c, dynv_get_bool_wd(args->gs->params, "gpick.color_names.imprecision_postfix", true));
-		dynv_set_string(color_object->params, "name", name.c_str());
-		color_list_add_color_object(args->gs->colors, color_object, 1);
-		color_object_release(color_object);
+		add_color_to_palette(args->color[i].input, name_assigner, args);
 	}
 
+	add_color_to_palette(args->secondary_color, name_assigner, args);
+
+	for (int i = 0; i < MAX_COLOR_LINES; ++i){
+		add_color_to_palette(args->color[i].output, name_assigner, args);
+	}
 }
 
 static gboolean color_focus_in_cb(GtkWidget *widget, GdkEventFocus *event, ColorMixerArgs *args){
@@ -242,17 +291,10 @@ static gboolean color_focus_in_cb(GtkWidget *widget, GdkEventFocus *event, Color
 }
 
 
-static void on_color_activate(GtkWidget *widget,  gpointer item) {
+static void on_color_activate(GtkWidget *widget, gpointer item) {
 	ColorMixerArgs* args=(ColorMixerArgs*)item;
-	Color c;
-
-	gtk_color_get_color(GTK_COLOR(widget), &c);
-
-	struct ColorObject *color_object=color_list_new_color_object(args->gs->colors, &c);
-	string name = color_names_get(args->gs->color_names, &c, dynv_get_bool_wd(args->gs->params, "gpick.color_names.imprecision_postfix", true));
-	dynv_set_string(color_object->params, "name", name.c_str());
-	color_list_add_color_object(args->gs->colors, color_object, 1);
-	color_object_release(color_object);
+	ColorMixerColorNameAssigner name_assigner(args->gs);
+	add_color_to_palette(widget, name_assigner, args);
 }
 
 static void type_toggled_cb(GtkWidget *widget, ColorMixerArgs *args) {
@@ -265,7 +307,7 @@ static void type_toggled_cb(GtkWidget *widget, ColorMixerArgs *args) {
 
 		Color c;
 		gtk_color_get_color(GTK_COLOR(color_widget), &c);
-		gtk_color_set_color(GTK_COLOR(args->secondary_color), &c, args->mixer_type->name);
+		gtk_color_set_color(GTK_COLOR(args->secondary_color), &c, _(args->mixer_type->name));
 		update(0, args);
 	}
 }
@@ -276,19 +318,19 @@ static void color_show_menu(GtkWidget* widget, ColorMixerArgs* args, GdkEventBut
 
 	menu = gtk_menu_new ();
 
-	item = gtk_menu_item_new_with_image ("_Add to palette", gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
+	item = gtk_menu_item_new_with_image (_("_Add to palette"), gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (on_color_add_to_palette), args);
 	g_object_set_data(G_OBJECT(item), "color_widget", widget);
 
-	item = gtk_menu_item_new_with_image ("_Add all to palette", gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
+	item = gtk_menu_item_new_with_image (_("A_dd all to palette"), gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (on_color_add_all_to_palette), args);
 	g_object_set_data(G_OBJECT(item), "color_widget", widget);
 
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), gtk_separator_menu_item_new ());
 
-	item = gtk_menu_item_new_with_mnemonic ("_Copy to clipboard");
+	item = gtk_menu_item_new_with_mnemonic (_("_Copy to clipboard"));
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
 	Color c;
@@ -315,7 +357,7 @@ static void color_show_menu(GtkWidget* widget, ColorMixerArgs* args, GdkEventBut
 		if (secondary_color){
 			GSList *group = 0;
 			for (uint32_t i = 0; i < sizeof(color_mixer_types) / sizeof(ColorMixerType); i++){
-				item = gtk_radio_menu_item_new_with_label(group, color_mixer_types[i].name);
+				item = gtk_radio_menu_item_new_with_label(group, _(color_mixer_types[i].name));
 				group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(item));
 				if (args->mixer_type == &color_mixer_types[i]){
 					gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), true);
@@ -328,12 +370,12 @@ static void color_show_menu(GtkWidget* widget, ColorMixerArgs* args, GdkEventBut
 			gtk_menu_shell_append (GTK_MENU_SHELL (menu), gtk_separator_menu_item_new ());
 		}
 
-		item = gtk_menu_item_new_with_image ("_Edit...", gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU));
+		item = gtk_menu_item_new_with_image (_("_Edit..."), gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 		g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (on_color_edit), args);
 		g_object_set_data(G_OBJECT(item), "color_widget", widget);
 
-		item = gtk_menu_item_new_with_image ("_Paste", gtk_image_new_from_stock(GTK_STOCK_PASTE, GTK_ICON_SIZE_MENU));
+		item = gtk_menu_item_new_with_image (_("_Paste"), gtk_image_new_from_stock(GTK_STOCK_PASTE, GTK_ICON_SIZE_MENU));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 		g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (on_color_paste), args);
 		g_object_set_data(G_OBJECT(item), "color_widget", widget);
@@ -441,16 +483,18 @@ static int source_destroy(ColorMixerArgs *args){
 }
 
 static int source_get_color(ColorMixerArgs *args, ColorObject** color){
+	ColorMixerColorNameAssigner name_assigner(args->gs);
 	Color c;
+	string widget_ident;
 	if (args->last_focused_color){
 		gtk_color_get_color(GTK_COLOR(args->last_focused_color), &c);
+		widget_ident = identify_color_widget(args->last_focused_color, args);
 	}else{
 		gtk_color_get_color(GTK_COLOR(args->color[0].input), &c);
+		widget_ident = identify_color_widget(args->color[0].input, args);
 	}
 	*color = color_list_new_color_object(args->gs->colors, &c);
-
-	string name = color_names_get(args->gs->color_names, &c, dynv_get_bool_wd(args->gs->params, "gpick.color_names.imprecision_postfix", true));
-	dynv_set_string((*color)->params, "name", name.c_str());
+	name_assigner.assign(*color, &c, widget_ident.c_str());
 	return 0;
 }
 
@@ -463,7 +507,7 @@ static int set_rgb_color_by_widget(ColorMixerArgs *args, struct ColorObject* col
 		if (args->color[i].input == color_widget){
 			set_rgb_color(args, color_object, i);
 			return 0;
-		}else	if (args->color[i].output == color_widget){
+		}else if (args->color[i].output == color_widget){
 			set_rgb_color(args, color_object, i);
 			return 0;
 		}
@@ -471,11 +515,12 @@ static int set_rgb_color_by_widget(ColorMixerArgs *args, struct ColorObject* col
 	return -1;
 }
 
+
 static int set_rgb_color(ColorMixerArgs *args, struct ColorObject* color, uint32_t color_index){
 	Color c;
 	color_object_get_color(color, &c);
 	if (color_index == (uint32_t)-1){
-		gtk_color_set_color(GTK_COLOR(args->secondary_color), &c, args->mixer_type->name);
+		gtk_color_set_color(GTK_COLOR(args->secondary_color), &c, _(args->mixer_type->name));
 	}else{
 		gtk_color_set_color(GTK_COLOR(args->color[color_index].input), &c, "");
 	}
@@ -493,6 +538,15 @@ static int source_set_color(ColorMixerArgs *args, struct ColorObject* color){
 }
 
 static int source_activate(ColorMixerArgs *args){
+
+	transformation::Chain *chain = static_cast<transformation::Chain*>(dynv_get_pointer_wdc(args->gs->params, "TransformationChain", 0));
+	gtk_color_set_transformation_chain(GTK_COLOR(args->secondary_color), chain);
+
+	for (int i = 0; i < MAX_COLOR_LINES; ++i){
+		gtk_color_set_transformation_chain(GTK_COLOR(args->color[i].input), chain);
+		gtk_color_set_transformation_chain(GTK_COLOR(args->color[i].output), chain);
+	}
+
 	gtk_statusbar_push(GTK_STATUSBAR(args->statusbar), gtk_statusbar_get_context_id(GTK_STATUSBAR(args->statusbar), "empty"), "");
 	return 0;
 }
@@ -576,9 +630,9 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 	dragdrop_widget_attach(widget, DragDropFlags(DRAGDROP_SOURCE | DRAGDROP_DESTINATION), &dd);
 
 
-	for (int i = 0; i < MAX_COLOR_LINES; ++i){
+	for (intptr_t i = 0; i < MAX_COLOR_LINES; ++i){
 
-		for (int j = 0; j < 2; ++j){
+		for (intptr_t j = 0; j < 2; ++j){
 
 			widget = gtk_color_new();
 			gtk_color_set_rounded(GTK_COLOR(widget), true);
@@ -636,7 +690,7 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 		sprintf(tmp, "color%d", i);
 		gtk_color_set_color(GTK_COLOR(args->color[i].input), dynv_get_color_wdc(args->params, tmp, &c), "");
 	}
-	gtk_color_set_color(GTK_COLOR(args->secondary_color), dynv_get_color_wdc(args->params, "secondary_color", &c), args->mixer_type->name);
+	gtk_color_set_color(GTK_COLOR(args->secondary_color), dynv_get_color_wdc(args->params, "secondary_color", &c), _(args->mixer_type->name));
 
 
 	hbox2 = gtk_hbox_new(FALSE, 0);
@@ -647,7 +701,7 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 	gtk_box_pack_start(GTK_BOX(hbox2), table, true, true, 0);
 	table_y = 0;
 
-	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new("Opacity:",0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,0,0);
+	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new(_("Opacity:"),0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,0,0);
 	args->opacity = gtk_hscale_new_with_range(1, 100, 1);
 	gtk_range_set_value(GTK_RANGE(args->opacity), dynv_get_float_wd(args->params, "opacity", 50));
 	g_signal_connect(G_OBJECT(args->opacity), "value-changed", G_CALLBACK (update), args);
@@ -675,8 +729,9 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 
 int color_mixer_source_register(ColorSourceManager *csm){
 	ColorSource *color_source = new ColorSource;
-	color_source_init(color_source, "color_mixer", "Color mixer");
+	color_source_init(color_source, "color_mixer", _("Color mixer"));
 	color_source->implement = (ColorSource* (*)(ColorSource *source, GlobalState *gs, struct dynvSystem *dynv_namespace))source_implement;
+	color_source->default_accelerator = GDK_m;
 	color_source_manager_add_source(csm, color_source);
 	return 0;
 }
