@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2011, Albertas Vyšniauskas
+ * Copyright (c) 2009-2012, Albertas Vyšniauskas
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -25,6 +25,7 @@
 #include "FloatingPicker.h"
 #include "ColorRYB.h"
 #include "ColorWheelType.h"
+#include "ColorSpaceType.h"
 
 #include "gtk/Swatch.h"
 #include "gtk/Zoomed.h"
@@ -35,10 +36,15 @@
 #include "uiUtilities.h"
 #include "uiColorInput.h"
 #include "uiConverter.h"
+#include "Internationalisation.h"
 
 #include <gdk/gdkkeysyms.h>
 
 #include <math.h>
+#ifdef _MSC_VER
+#define M_PI 3.14159265359
+#endif
+#include <string.h>
 #include <sstream>
 #include <iostream>
 #include <iomanip>
@@ -57,6 +63,7 @@ typedef struct ColorPickerArgs{
 	GtkWidget* expanderCMYK;
 	GtkWidget* expanderXYZ;
 	GtkWidget* expanderLAB;
+	GtkWidget* expanderLCH;
 
 	GtkWidget* expanderInfo;
 	GtkWidget* expanderMain;
@@ -71,6 +78,7 @@ typedef struct ColorPickerArgs{
 	GtkWidget* rgb_control;
 	GtkWidget* cmyk_control;
 	GtkWidget* lab_control;
+	GtkWidget* lch_control;
 
 	GtkWidget* color_name;
 	GtkWidget* statusbar;
@@ -88,7 +96,26 @@ typedef struct ColorPickerArgs{
 
 }ColorPickerArgs;
 
+struct ColorCompItem{
+	GtkWidget *widget;
+	GtkColorComponentComp component;
+	int component_id;
+};
+
 static int source_set_color(ColorPickerArgs *args, ColorObject* color);
+static int source_set_nth_color(ColorPickerArgs *args, uint32_t color_n, ColorObject* color);
+static int source_get_nth_color(ColorPickerArgs *args, uint32_t color_n, ColorObject** color);
+
+static void updateMainColorNow(ColorPickerArgs* args){
+	if (!dynv_get_bool_wd(args->params, "zoomed_enabled", true)){
+		Color c;
+		gtk_swatch_get_active_color(GTK_SWATCH(args->swatch_display), &c);
+		gchar* text = main_get_color_text(args->gs, &c, COLOR_TEXT_TYPE_DISPLAY);
+		gtk_color_set_color(GTK_COLOR(args->color_code), &c, text);
+		if (text) g_free(text);
+		gtk_swatch_set_main_color(GTK_SWATCH(args->swatch_display), &c);
+	}
+}
 
 static gboolean updateMainColor( gpointer data ){
 	ColorPickerArgs* args=(ColorPickerArgs*)data;
@@ -112,8 +139,12 @@ static gboolean updateMainColor( gpointer data ){
 	sampler_get_screen_rect(args->gs->sampler, pointer, window_size, &sampler_rect);
 	screen_reader_add_rect(args->gs->screen_reader, screen, sampler_rect);
 
-	gtk_zoomed_get_screen_rect(GTK_ZOOMED(args->zoomed_display), pointer, window_size, &zoomed_rect);
-	screen_reader_add_rect(args->gs->screen_reader, screen, zoomed_rect);
+	bool zoomed_enabled = dynv_get_bool_wd(args->params, "zoomed_enabled", true);
+
+	if (zoomed_enabled){
+		gtk_zoomed_get_screen_rect(GTK_ZOOMED(args->zoomed_display), pointer, window_size, &zoomed_rect);
+		screen_reader_add_rect(args->gs->screen_reader, screen, zoomed_rect);
+	}
 
 	screen_reader_update_pixbuf(args->gs->screen_reader, &final_rect);
 
@@ -124,15 +155,16 @@ static gboolean updateMainColor( gpointer data ){
 	sampler_get_color_sample(args->gs->sampler, pointer, window_size, offset, &c);
 
 	gchar* text = main_get_color_text(args->gs, &c, COLOR_TEXT_TYPE_DISPLAY);
-
 	gtk_color_set_color(GTK_COLOR(args->color_code), &c, text);
 	if (text) g_free(text);
 
 	gtk_swatch_set_main_color(GTK_SWATCH(args->swatch_display), &c);
 
 
-	offset = Vec2<int>(zoomed_rect.getX()-final_rect.getX(), zoomed_rect.getY()-final_rect.getY());
-	gtk_zoomed_update(GTK_ZOOMED(args->zoomed_display), pointer, window_size, offset, screen_reader_get_pixbuf(args->gs->screen_reader));
+	if (zoomed_enabled){
+		offset = Vec2<int>(zoomed_rect.getX()-final_rect.getX(), zoomed_rect.getY()-final_rect.getY());
+		gtk_zoomed_update(GTK_ZOOMED(args->zoomed_display), pointer, window_size, offset, screen_reader_get_pixbuf(args->gs->screen_reader));
+	}
 
 	return TRUE;
 }
@@ -142,16 +174,40 @@ static gboolean updateMainColorTimer(ColorPickerArgs* args){
 	return true;
 }
 
+static void updateComponentText(ColorPickerArgs *args, GtkColorComponent *component, const char *type){
+	Color transformed_color;
+	gtk_color_component_get_transformed_color(component, &transformed_color);
+	lua_State* L = static_cast<lua_State*>(dynv_get_pointer_wdc(args->gs->params, "lua_State", 0));
+	list<string> str = color_space_color_to_text(type, &transformed_color, L);
+	int j = 0;
+	const char *text[4];
+	memset(text, 0, sizeof(text));
+	for (list<string>::iterator i = str.begin(); i != str.end(); i++){
+		text[j] = (*i).c_str();
+		j++;
+		if (j > 3) break;
+	}
+	gtk_color_component_set_text(component, text);
+}
 
-static void updateDiplays(ColorPickerArgs *args, GtkWidget *except_widget){
+static void updateDisplays(ColorPickerArgs *args, GtkWidget *except_widget){
+	updateMainColorNow(args);
 	Color c, c2;
 	gtk_swatch_get_active_color(GTK_SWATCH(args->swatch_display),&c);
 
 	if (except_widget != args->hsl_control) gtk_color_component_set_color(GTK_COLOR_COMPONENT(args->hsl_control), &c);
 	if (except_widget != args->hsv_control) gtk_color_component_set_color(GTK_COLOR_COMPONENT(args->hsv_control), &c);
-	if (except_widget != args->rgb_control) gtk_color_component_set_color(GTK_COLOR_COMPONENT(args->rgb_control), &c);
+	if (except_widget != args->rgb_control)	gtk_color_component_set_color(GTK_COLOR_COMPONENT(args->rgb_control), &c);
 	if (except_widget != args->cmyk_control) gtk_color_component_set_color(GTK_COLOR_COMPONENT(args->cmyk_control), &c);
 	if (except_widget != args->lab_control) gtk_color_component_set_color(GTK_COLOR_COMPONENT(args->lab_control), &c);
+	if (except_widget != args->lch_control) gtk_color_component_set_color(GTK_COLOR_COMPONENT(args->lch_control), &c);
+
+	updateComponentText(args, GTK_COLOR_COMPONENT(args->hsl_control), "hsl");
+	updateComponentText(args, GTK_COLOR_COMPONENT(args->hsv_control), "hsv");
+	updateComponentText(args, GTK_COLOR_COMPONENT(args->rgb_control), "rgb");
+	updateComponentText(args, GTK_COLOR_COMPONENT(args->cmyk_control), "cmyk");
+	updateComponentText(args, GTK_COLOR_COMPONENT(args->lab_control), "lab");
+	updateComponentText(args, GTK_COLOR_COMPONENT(args->lch_control), "lch");
 
 	string color_name = color_names_get(args->gs->color_names, &c, true);
 	gtk_entry_set_text(GTK_ENTRY(args->color_name), color_name.c_str());
@@ -162,16 +218,9 @@ static void updateDiplays(ColorPickerArgs *args, GtkWidget *except_widget){
 	stringstream ss;
 	ss.setf(ios::fixed, ios::floatfield);
 
-	matrix3x3 adaptation_matrix, working_space_matrix;
-	vector3 d50, d65;
-	vector3_set(&d50, 96.442, 100.000,  82.821);
-	vector3_set(&d65, 95.047, 100.000, 108.883);
-	color_get_chromatic_adaptation_matrix(&d50, &d65, &adaptation_matrix);
-	color_get_working_space_matrix(0.6400, 0.3300, 0.3000, 0.6000, 0.1500, 0.0600, &d65, &working_space_matrix);
-
 	Color c_lab, c2_lab;
-    color_rgb_to_lab(&c, &c_lab, &d50, &working_space_matrix);
-    color_rgb_to_lab(&c2, &c2_lab, &d50, &working_space_matrix);
+	color_rgb_to_lab_d50(&c, &c_lab);
+	color_rgb_to_lab_d50(&c2, &c2_lab);
 
 	const ColorWheelType *wheel = &color_wheel_types_get()[0];
 	Color hsl1, hsl2;
@@ -183,7 +232,7 @@ static void updateDiplays(ColorPickerArgs *args, GtkWidget *except_widget){
 	wheel->rgbhue_to_hue(hsl2.hsl.hue, &hue2);
 
 	double complementary = std::abs(hue1 - hue2);
-    complementary -= std::floor(complementary);
+	complementary -= std::floor(complementary);
 	complementary *= std::sin(hsl1.hsl.lightness * M_PI) * std::sin(hsl2.hsl.lightness * M_PI);
 	complementary *= std::sin(hsl1.hsl.saturation * M_PI / 2) * std::sin(hsl2.hsl.saturation * M_PI / 2);
 
@@ -194,13 +243,13 @@ static void updateDiplays(ColorPickerArgs *args, GtkWidget *except_widget){
 
 
 static void on_swatch_active_color_changed( GtkWidget *widget, gint32 new_active_color, gpointer data ){
-	ColorPickerArgs* args=(ColorPickerArgs*)data;
-	updateDiplays(args, widget);
+	ColorPickerArgs* args = (ColorPickerArgs*)data;
+	updateDisplays(args, widget);
 }
 
 static void on_swatch_color_changed( GtkWidget *widget, gpointer data ){
 	ColorPickerArgs* args=(ColorPickerArgs*)data;
-	updateDiplays(args, widget);
+	updateDisplays(args, widget);
 }
 
 
@@ -315,17 +364,17 @@ static gboolean swatch_button_press_cb(GtkWidget *widget, GdkEventButton *event,
 		Color c;
 		gtk_swatch_get_active_color(GTK_SWATCH(args->swatch_display), &c);
 
-	    item = gtk_menu_item_new_with_image ("_Add to palette", gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
+	    item = gtk_menu_item_new_with_image (_("_Add to palette"), gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
 	    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	    g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (on_swatch_menu_add_to_palette), args);
 
-	    item = gtk_menu_item_new_with_image ("_Add all to palette", gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
+	    item = gtk_menu_item_new_with_image (_("A_dd all to palette"), gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
 	    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	    g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (on_swatch_menu_add_all_to_palette), args);
 
 	    gtk_menu_shell_append (GTK_MENU_SHELL(menu), gtk_separator_menu_item_new());
 
-	    item = gtk_menu_item_new_with_mnemonic ("_Copy to clipboard");
+	    item = gtk_menu_item_new_with_mnemonic (_("_Copy to clipboard"));
 	    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
 	    struct ColorObject* color_object;
@@ -335,11 +384,11 @@ static gboolean swatch_button_press_cb(GtkWidget *widget, GdkEventButton *event,
 
 		gtk_menu_shell_append (GTK_MENU_SHELL(menu), gtk_separator_menu_item_new());
 
-		item = gtk_menu_item_new_with_image ("_Edit...", gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU));
+		item = gtk_menu_item_new_with_image (_("_Edit..."), gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 		g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (on_swatch_color_edit), args);
 
-		item = gtk_menu_item_new_with_image ("_Paste", gtk_image_new_from_stock(GTK_STOCK_PASTE, GTK_ICON_SIZE_MENU));
+		item = gtk_menu_item_new_with_image (_("_Paste"), gtk_image_new_from_stock(GTK_STOCK_PASTE, GTK_ICON_SIZE_MENU));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 		g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (paste_cb), args);
 
@@ -372,7 +421,7 @@ static gboolean on_swatch_focus_change(GtkWidget *widget, GdkEventFocus *event,
 
 
 	if (event->in){
-		gtk_statusbar_push(GTK_STATUSBAR(args->statusbar), gtk_statusbar_get_context_id(GTK_STATUSBAR(args->statusbar), "swatch_focused"), "Press Spacebar to sample color under mouse pointer");
+		gtk_statusbar_push(GTK_STATUSBAR(args->statusbar), gtk_statusbar_get_context_id(GTK_STATUSBAR(args->statusbar), "swatch_focused"), _("Press Spacebar to sample color under mouse pointer"));
 	}else{
 		gtk_statusbar_pop(GTK_STATUSBAR(args->statusbar), gtk_statusbar_get_context_id(GTK_STATUSBAR(args->statusbar), "swatch_focused"));
 	}
@@ -380,8 +429,6 @@ static gboolean on_swatch_focus_change(GtkWidget *widget, GdkEventFocus *event,
 }
 
 
-
-
 static gboolean on_key_up (GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
 	ColorPickerArgs* args=(ColorPickerArgs*)data;
@@ -390,6 +437,19 @@ static gboolean on_key_up (GtkWidget *widget, GdkEventKey *event, gpointer data)
 
 	switch(event->keyval)
 	{
+		case GDK_m:
+			{
+				int x, y;
+				gdk_display_get_pointer(gdk_display_get_default(), NULL, &x, &y, NULL);
+				math::Vec2<int> position(x, y);
+				if ((event->state&modifiers)==GDK_CONTROL_MASK){
+					gtk_zoomed_set_mark(GTK_ZOOMED(args->zoomed_display), 1, position);
+				}else{
+					gtk_zoomed_set_mark(GTK_ZOOMED(args->zoomed_display), 0, position);
+				}
+			}
+			break;
+
 		case GDK_c:
 			if ((event->state&modifiers)==GDK_CONTROL_MASK){
 
@@ -414,49 +474,49 @@ static gboolean on_key_up (GtkWidget *widget, GdkEventKey *event, gpointer data)
 
 		case GDK_1:
 			gtk_swatch_set_active_index(GTK_SWATCH(args->swatch_display), 1);
-			updateDiplays(args, widget);
+			updateDisplays(args, widget);
 			return TRUE;
 			break;
 
 		case GDK_2:
 			gtk_swatch_set_active_index(GTK_SWATCH(args->swatch_display), 2);
-			updateDiplays(args, widget);
+			updateDisplays(args, widget);
 			return TRUE;
 			break;
 
 		case GDK_3:
 			gtk_swatch_set_active_index(GTK_SWATCH(args->swatch_display), 3);
-			updateDiplays(args, widget);
+			updateDisplays(args, widget);
 			return TRUE;
 			break;
 
 		case GDK_4:
 			gtk_swatch_set_active_index(GTK_SWATCH(args->swatch_display), 4);
-			updateDiplays(args, widget);
+			updateDisplays(args, widget);
 			return TRUE;
 			break;
 
 		case GDK_5:
 			gtk_swatch_set_active_index(GTK_SWATCH(args->swatch_display), 5);
-			updateDiplays(args, widget);
+			updateDisplays(args, widget);
 			return TRUE;
 			break;
 
 		case GDK_6:
 			gtk_swatch_set_active_index(GTK_SWATCH(args->swatch_display), 6);
-			updateDiplays(args, widget);
+			updateDisplays(args, widget);
 			return TRUE;
 			break;
 
 		case GDK_Right:
 			gtk_swatch_move_active(GTK_SWATCH(args->swatch_display),1);
-			updateDiplays(args, widget);
+			updateDisplays(args, widget);
 			return TRUE;
 			break;
 
 		case GDK_Left:
 			gtk_swatch_move_active(GTK_SWATCH(args->swatch_display),-1);
-			updateDiplays(args, widget);
+			updateDisplays(args, widget);
 			return TRUE;
 			break;
 
@@ -464,8 +524,6 @@ static gboolean on_key_up (GtkWidget *widget, GdkEventKey *event, gpointer data)
 			updateMainColor(args);
 			gtk_swatch_set_color_to_main(GTK_SWATCH(args->swatch_display));
 
-
-
 			if (dynv_get_bool_wd(args->params, "sampler.add_to_palette", true)){
 				Color c;
 				gtk_swatch_get_active_color(GTK_SWATCH(args->swatch_display), &c);
@@ -496,7 +554,17 @@ static gboolean on_key_up (GtkWidget *widget, GdkEventKey *event, gpointer data)
 			if (dynv_get_bool_wd(args->params, "sampler.rotate_swatch_after_sample", true)){
 				gtk_swatch_move_active(GTK_SWATCH(args->swatch_display),1);
 			}
-			updateDiplays(args, widget);
+			updateDisplays(args, widget);
+			return TRUE;
+			break;
+
+		case GDK_a:
+			on_swatch_menu_add_all_to_palette(NULL, args);
+			return TRUE;
+			break;
+
+		case GDK_e:
+			on_swatch_color_edit(NULL, args);
 			return TRUE;
 			break;
 
@@ -521,18 +589,20 @@ static void on_zoom_value_changed(GtkRange *slider, gpointer data){
 	gtk_zoomed_set_zoom(GTK_ZOOMED(args->zoomed_display), gtk_range_get_value(GTK_RANGE(slider)));
 }
 
-
 static void color_component_change_value(GtkWidget *widget, Color* c, ColorPickerArgs* args){
 	gtk_swatch_set_active_color(GTK_SWATCH(args->swatch_display), c);
-	updateDiplays(args, widget);
+	updateDisplays(args, widget);
+}
+
+static void color_component_input_clicked(GtkWidget *widget, int component_id, ColorPickerArgs* args){
+	dialog_color_component_input_show(GTK_WINDOW(gtk_widget_get_toplevel(args->main)), GTK_COLOR_COMPONENT(widget), component_id, dynv_get_dynv(args->params, "component_edit"));
 }
 
 static void ser_decimal_get(GtkColorComponentComp component, int component_id, Color* color, const char *text){
 	double v;
 	stringstream ss(text);
 	ss >> v;
-
-    switch (component){
+	switch (component){
 		case hsv:
 		case hsl:
 			if (component_id == 0){
@@ -573,21 +643,14 @@ static struct{
 };
 
 
-struct ColorCompItem{
-	GtkWidget *widget;
-	GtkColorComponentComp component;
-	int component_id;
-};
 
 static void color_component_copy(GtkWidget *widget, ColorPickerArgs* args){
-	Color color;
 	struct ColorCompItem *comp_item = (struct ColorCompItem*)g_object_get_data(G_OBJECT(gtk_widget_get_parent(widget)), "comp_item");
-	gtk_color_component_get_transformed_color(GTK_COLOR_COMPONENT(comp_item->widget), &color);
-
-	string text = serial[0].set(comp_item->component, comp_item->component_id, &color);
-
-    gtk_clipboard_set_text(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD), text.c_str(), text.length());
-    gtk_clipboard_store(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD));
+	const char *text = gtk_color_component_get_text(GTK_COLOR_COMPONENT(comp_item->widget), comp_item->component_id);
+	if (text){
+		gtk_clipboard_set_text(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD), text, strlen(text));
+		gtk_clipboard_store(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD));
+	}
 }
 
 static void color_component_paste(GtkWidget *widget, ColorPickerArgs* args){
@@ -603,10 +666,15 @@ static void color_component_paste(GtkWidget *widget, ColorPickerArgs* args){
 
 		gtk_color_component_get_color(GTK_COLOR_COMPONENT(comp_item->widget), &color);
 		gtk_swatch_set_active_color(GTK_SWATCH(args->swatch_display), &color);
-        updateDiplays(args, comp_item->widget);
+		updateDisplays(args, comp_item->widget);
 	}
 }
 
+static void color_component_edit(GtkWidget *widget, ColorPickerArgs* args){
+	struct ColorCompItem *comp_item = (struct ColorCompItem*)g_object_get_data(G_OBJECT(gtk_widget_get_parent(widget)), "comp_item");
+	dialog_color_component_input_show(GTK_WINDOW(gtk_widget_get_toplevel(args->main)), GTK_COLOR_COMPONENT(comp_item->widget), comp_item->component_id, dynv_get_dynv(args->params, "component_edit"));
+}
+
 static void destroy_comp_item(struct ColorCompItem *comp_item){
 	delete comp_item;
 }
@@ -628,14 +696,20 @@ static gboolean color_component_key_up_cb(GtkWidget *widget, GdkEventButton *eve
 
 		g_object_set_data_full(G_OBJECT(menu), "comp_item", comp_item, (GDestroyNotify)destroy_comp_item);
 
-		item = gtk_menu_item_new_with_image("Copy", gtk_image_new_from_stock(GTK_STOCK_COPY, GTK_ICON_SIZE_MENU));
+		item = gtk_menu_item_new_with_image(_("Copy"), gtk_image_new_from_stock(GTK_STOCK_COPY, GTK_ICON_SIZE_MENU));
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 		g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(color_component_copy), args);
 
-		item = gtk_menu_item_new_with_image("Paste", gtk_image_new_from_stock(GTK_STOCK_PASTE, GTK_ICON_SIZE_MENU));
+		item = gtk_menu_item_new_with_image(_("Paste"), gtk_image_new_from_stock(GTK_STOCK_PASTE, GTK_ICON_SIZE_MENU));
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 		g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(color_component_paste), args);
 
+		gtk_menu_shell_append(GTK_MENU_SHELL(menu), gtk_separator_menu_item_new());
+
+		item = gtk_menu_item_new_with_image(_("Edit"), gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU));
+		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+		g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(color_component_edit), args);
+
 		gtk_widget_show_all(GTK_WIDGET(menu));
 
 		if (event){
@@ -694,11 +768,11 @@ static GtkWidget* create_falloff_type_list (void){
 	GtkTreeIter iter1;
 
 	const char* falloff_types[][2] = {
-			{ "gpick-falloff-none", "None" },
-			{ "gpick-falloff-linear", "Linear" },
-			{ "gpick-falloff-quadratic", "Quadratic" },
-			{ "gpick-falloff-cubic", "Cubic" },
-			{ "gpick-falloff-exponential", "Exponential" },
+			{ "gpick-falloff-none", _("None")},
+			{ "gpick-falloff-linear", _("Linear")},
+			{ "gpick-falloff-quadratic", _("Quadratic")},
+			{ "gpick-falloff-cubic", _("Cubic")},
+			{ "gpick-falloff-exponential", _("Exponential")},
 			};
 
 	gint32 falloff_type_ids[]={
@@ -760,6 +834,7 @@ static int source_destroy(ColorPickerArgs *args){
 	dynv_set_bool(args->params, "expander.hsv", gtk_expander_get_expanded(GTK_EXPANDER(args->expanderHSV)));
 	dynv_set_bool(args->params, "expander.hsl", gtk_expander_get_expanded(GTK_EXPANDER(args->expanderHSL)));
 	dynv_set_bool(args->params, "expander.lab", gtk_expander_get_expanded(GTK_EXPANDER(args->expanderLAB)));
+	dynv_set_bool(args->params, "expander.lch", gtk_expander_get_expanded(GTK_EXPANDER(args->expanderLCH)));
 	dynv_set_bool(args->params, "expander.cmyk", gtk_expander_get_expanded(GTK_EXPANDER(args->expanderCMYK)));
 	dynv_set_bool(args->params, "expander.info", gtk_expander_get_expanded(GTK_EXPANDER(args->expanderInfo)));
 
@@ -788,15 +863,38 @@ static int source_get_color(ColorPickerArgs *args, ColorObject** color){
 	return 0;
 }
 
+static int source_set_nth_color(ColorPickerArgs *args, uint32_t color_n, ColorObject* color){
+	if (color_n < 0 || color_n > 6) return -1;
+	Color c;
+	color_object_get_color(color, &c);
+	gtk_swatch_set_color(GTK_SWATCH(args->swatch_display), color_n + 1, &c);
+
+	updateDisplays(args, 0);
+	return 0;
+}
+
+static int source_get_nth_color(ColorPickerArgs *args, uint32_t color_n, ColorObject** color){
+	if (color_n < 0 || color_n > 6) return -1;
+	Color c;
+
+	gtk_swatch_get_color(GTK_SWATCH(args->swatch_display), color_n + 1, &c);
+
+	struct ColorObject *color_object = color_list_new_color_object(args->gs->colors, &c);
+	string name = color_names_get(args->gs->color_names, &c, dynv_get_bool_wd(args->gs->params, "gpick.color_names.imprecision_postfix", true));
+	dynv_set_string(color_object->params, "name", name.c_str());
+
+	*color = color_object;
+
+	return 0;
+}
+
 
 static int source_set_color(ColorPickerArgs *args, ColorObject* color){
-
 	Color c;
 	color_object_get_color(color, &c);
 	gtk_swatch_set_active_color(GTK_SWATCH(args->swatch_display), &c);
 
-	updateDiplays(args, 0);
-
+	updateDisplays(args, 0);
 	return 0;
 }
 
@@ -807,12 +905,50 @@ static int source_activate(ColorPickerArgs *args){
 		args->timeout_source_id = 0;
 	}
 
-	float refresh_rate = dynv_get_float_wd(args->global_params, "refresh_rate", 30);
-	args->timeout_source_id = g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE, 1000/refresh_rate, (GSourceFunc)updateMainColorTimer, args, (GDestroyNotify)NULL);
+  struct{
+		GtkWidget *widget;
+		const char *setting;
+	}color_spaces[] = {
+		{args->expanderCMYK, "color_space.cmyk"},
+		{args->expanderHSL, "color_space.hsl"},
+		{args->expanderHSV, "color_space.hsv"},
+		{args->expanderLAB, "color_space.lab"},
+		{args->expanderLCH, "color_space.lch"},
+		{args->expanderRGB, "color_space.rgb"},
+		{0, 0},
+	};
+	for (int i = 0; color_spaces[i].setting; i++){
+		if (dynv_get_bool_wd(args->params, color_spaces[i].setting, true))
+			gtk_widget_show(color_spaces[i].widget);
+		else
+			gtk_widget_hide(color_spaces[i].widget);
+	}
+
+	bool out_of_gamut_mask = dynv_get_bool_wd(args->params, "out_of_gamut_mask", true);
+
+	gtk_color_component_set_out_of_gamut_mask(GTK_COLOR_COMPONENT(args->lab_control), out_of_gamut_mask);
+	gtk_color_component_set_lab_illuminant(GTK_COLOR_COMPONENT(args->lab_control), color_get_illuminant(dynv_get_string_wd(args->params, "lab.illuminant", "D50")));
+	gtk_color_component_set_lab_observer(GTK_COLOR_COMPONENT(args->lab_control), color_get_observer(dynv_get_string_wd(args->params, "lab.observer", "2")));
+	updateComponentText(args, GTK_COLOR_COMPONENT(args->lab_control), "lab");
+
+	gtk_color_component_set_out_of_gamut_mask(GTK_COLOR_COMPONENT(args->lch_control), out_of_gamut_mask);
+	gtk_color_component_set_lab_illuminant(GTK_COLOR_COMPONENT(args->lch_control), color_get_illuminant(dynv_get_string_wd(args->params, "lab.illuminant", "D50")));
+	gtk_color_component_set_lab_observer(GTK_COLOR_COMPONENT(args->lch_control), color_get_observer(dynv_get_string_wd(args->params, "lab.observer", "2")));
+	updateComponentText(args, GTK_COLOR_COMPONENT(args->lch_control), "lch");
+
+	transformation::Chain *chain = static_cast<transformation::Chain*>(dynv_get_pointer_wdc(args->gs->params, "TransformationChain", 0));
+	gtk_swatch_set_transformation_chain(GTK_SWATCH(args->swatch_display), chain);
+	gtk_color_set_transformation_chain(GTK_COLOR(args->color_code), chain);
+	gtk_color_set_transformation_chain(GTK_COLOR(args->contrastCheck), chain);
+
+	if (dynv_get_bool_wd(args->params, "zoomed_enabled", true)){
+		float refresh_rate = dynv_get_float_wd(args->global_params, "refresh_rate", 30);
+		args->timeout_source_id = g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE, 1000/refresh_rate, (GSourceFunc)updateMainColorTimer, args, (GDestroyNotify)NULL);
+	}
 
 	gtk_zoomed_set_size(GTK_ZOOMED(args->zoomed_display), dynv_get_int32_wd(args->params, "zoom_size", 150));
 
-	gtk_statusbar_push(GTK_STATUSBAR(args->statusbar), gtk_statusbar_get_context_id(GTK_STATUSBAR(args->statusbar), "focus_swatch"), "Click on swatch area to begin adding colors to palette");
+	gtk_statusbar_push(GTK_STATUSBAR(args->statusbar), gtk_statusbar_get_context_id(GTK_STATUSBAR(args->statusbar), "focus_swatch"), _("Click on swatch area to begin adding colors to palette"));
 
 	return 0;
 }
@@ -842,7 +978,7 @@ static int set_color_object_at(struct DragDrop* dd, struct ColorObject* colorobj
 	color_object_get_color(colorobject, &c);
 	gtk_swatch_set_color(GTK_SWATCH(dd->widget), color_index, &c);
 
-	updateDiplays((ColorPickerArgs*)dd->userdata, 0);
+	updateDisplays((ColorPickerArgs*)dd->userdata, 0);
 
 	return 0;
 }
@@ -876,7 +1012,7 @@ static int set_color_object_at_contrast(struct DragDrop* dd, struct ColorObject*
 	Color c;
 	color_object_get_color(colorobject, &c);
 	gtk_color_set_color(GTK_COLOR(args->contrastCheck), &c, "Sample");
-	updateDiplays((ColorPickerArgs*)dd->userdata, 0);
+	updateDisplays((ColorPickerArgs*)dd->userdata, 0);
 	return 0;
 }
 
@@ -885,6 +1021,29 @@ static void show_dialog_converter(GtkWidget *widget, ColorPickerArgs *args){
 	return;
 }
 
+static void on_zoomed_activate(GtkWidget *widget, ColorPickerArgs *args){
+	if (dynv_get_bool_wd(args->params, "zoomed_enabled", true)){
+		gtk_zoomed_set_fade(GTK_ZOOMED(args->zoomed_display), true);
+		dynv_set_bool(args->params, "zoomed_enabled", false);
+
+		if (args->timeout_source_id > 0){
+			g_source_remove(args->timeout_source_id);
+			args->timeout_source_id = 0;
+		}
+	}else{
+		gtk_zoomed_set_fade(GTK_ZOOMED(args->zoomed_display), false);
+		dynv_set_bool(args->params, "zoomed_enabled", true);
+
+		if (args->timeout_source_id > 0){
+			g_source_remove(args->timeout_source_id);
+			args->timeout_source_id = 0;
+		}
+		float refresh_rate = dynv_get_float_wd(args->global_params, "refresh_rate", 30);
+		args->timeout_source_id = g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE, 1000/refresh_rate, (GSourceFunc)updateMainColorTimer, args, (GDestroyNotify)NULL);
+	}
+	return;
+}
+
 static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struct dynvSystem *dynv_namespace){
 	ColorPickerArgs* args = new ColorPickerArgs;
 
@@ -897,6 +1056,8 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 	args->source.destroy = (int (*)(ColorSource *source))source_destroy;
 	args->source.get_color = (int (*)(ColorSource *source, ColorObject** color))source_get_color;
 	args->source.set_color = (int (*)(ColorSource *source, ColorObject* color))source_set_color;
+	args->source.get_nth_color = (int (*)(ColorSource *source, uint32_t color_n, ColorObject** color))source_get_nth_color;
+	args->source.set_nth_color = (int (*)(ColorSource *source, uint32_t color_n, ColorObject* color))source_set_nth_color;
 	args->source.activate = (int (*)(ColorSource *source))source_activate;
 	args->source.deactivate = (int (*)(ColorSource *source))source_deactivate;
 
@@ -926,6 +1087,7 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 			g_signal_connect(G_OBJECT(widget), "popup-menu", G_CALLBACK(swatch_popup_menu_cb), args);
 
 			gtk_swatch_set_active_index(GTK_SWATCH(widget), dynv_get_int32_wd(args->params, "swatch.active_color", 1));
+
 			args->swatch_display = widget;
 
 			gtk_drag_dest_set(widget, GtkDestDefaults(GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT), 0, 0, GDK_ACTION_COPY);
@@ -955,14 +1117,20 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 				}
 			}
 
+
+
 			args->color_code = gtk_color_new();
 			gtk_box_pack_start (GTK_BOX(vbox), args->color_code, false, true, 0);
 			g_signal_connect(G_OBJECT(args->color_code), "activated", G_CALLBACK(show_dialog_converter), args);
 
 
 			args->zoomed_display = gtk_zoomed_new();
+			if (!dynv_get_bool_wd(args->params, "zoomed_enabled", true)){
+				gtk_zoomed_set_fade(GTK_ZOOMED(args->zoomed_display), true);
+			}
 			gtk_zoomed_set_size(GTK_ZOOMED(args->zoomed_display), dynv_get_int32_wd(args->params, "zoom_size", 150));
 			gtk_box_pack_start (GTK_BOX(vbox), args->zoomed_display, false, false, 0);
+			g_signal_connect(G_OBJECT(args->zoomed_display), "activated", G_CALLBACK(on_zoomed_activate), args);
 
 
 		scrolled = gtk_scrolled_window_new(0, 0);
@@ -972,7 +1140,7 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 		vbox = gtk_vbox_new(false, 5);
 		gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled), vbox);
 
-			expander=gtk_expander_new("Settings");
+			expander=gtk_expander_new(_("Settings"));
 			gtk_expander_set_expanded(GTK_EXPANDER(expander), dynv_get_bool_wd(args->params, "expander.settings", false));
 			args->expanderSettings=expander;
 			gtk_box_pack_start (GTK_BOX(vbox), expander, FALSE, FALSE, 0);
@@ -982,24 +1150,24 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 
 			gtk_container_add(GTK_CONTAINER(expander), table);
 
-				gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new("Oversample:",0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
+				gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new(_("Oversample:"),0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
 				widget = gtk_hscale_new_with_range (0,16,1);
 				g_signal_connect (G_OBJECT (widget), "value-changed", G_CALLBACK (on_oversample_value_changed), args);
 				gtk_range_set_value(GTK_RANGE(widget), dynv_get_int32_wd(args->params, "sampler.oversample", 0));
 				gtk_table_attach(GTK_TABLE(table), widget,1,2,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,5,0);
 				table_y++;
 
-				gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new("Falloff:",0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
+				gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new(_("Falloff:"),0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
 				widget = create_falloff_type_list();
 				g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (on_oversample_falloff_changed), args);
 				gtk_combo_box_set_active(GTK_COMBO_BOX(widget), dynv_get_int32_wd(args->params, "sampler.falloff", NONE));
 				gtk_table_attach(GTK_TABLE(table), widget,1,2,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,5,0);
 				table_y++;
 
-				gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new("Zoom:",0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
-				widget = gtk_hscale_new_with_range (2, 15, 0.5);
+				gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new(_("Zoom:"),0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
+				widget = gtk_hscale_new_with_range (0, 100, 1);
 				g_signal_connect (G_OBJECT (widget), "value-changed", G_CALLBACK (on_zoom_value_changed), args);
-				gtk_range_set_value(GTK_RANGE(widget), dynv_get_float_wd(args->params, "zoom", 2));
+				gtk_range_set_value(GTK_RANGE(widget), dynv_get_float_wd(args->params, "zoom", 20));
 				gtk_table_attach(GTK_TABLE(table), widget,1,2,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,5,0);
 				table_y++;
 
@@ -1009,9 +1177,12 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 			gtk_box_pack_start(GTK_BOX(vbox), expander, FALSE, FALSE, 0);
 
 				widget = gtk_color_component_new(hsv);
+				const char *hsv_labels[] = {"H", _("Hue"), "S", _("Saturation"), "V", _("Value"), NULL};
+				gtk_color_component_set_label(GTK_COLOR_COMPONENT(widget), hsv_labels);
 				args->hsv_control = widget;
 				g_signal_connect(G_OBJECT(widget), "color-changed", G_CALLBACK(color_component_change_value), args);
 				g_signal_connect(G_OBJECT(widget), "button_release_event", G_CALLBACK(color_component_key_up_cb), args);
+				g_signal_connect(G_OBJECT(widget), "input-clicked", G_CALLBACK(color_component_input_clicked), args);
 				gtk_container_add(GTK_CONTAINER(expander), widget);
 
 			expander=gtk_expander_new("HSL");
@@ -1020,9 +1191,12 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 			gtk_box_pack_start(GTK_BOX(vbox), expander, FALSE, FALSE, 0);
 
 				widget = gtk_color_component_new(hsl);
+				const char *hsl_labels[] = {"H", _("Hue"), "S", _("Saturation"), "L", _("Lightness"), NULL};
+				gtk_color_component_set_label(GTK_COLOR_COMPONENT(widget), hsl_labels);
 				args->hsl_control = widget;
 				g_signal_connect(G_OBJECT(widget), "color-changed", G_CALLBACK(color_component_change_value), args);
 				g_signal_connect(G_OBJECT(widget), "button_release_event", G_CALLBACK(color_component_key_up_cb), args);
+				g_signal_connect(G_OBJECT(widget), "input-clicked", G_CALLBACK(color_component_input_clicked), args);
 				gtk_container_add(GTK_CONTAINER(expander), widget);
 
 			expander=gtk_expander_new("RGB");
@@ -1031,9 +1205,12 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 			gtk_box_pack_start (GTK_BOX(vbox), expander, FALSE, FALSE, 0);
 
 				widget = gtk_color_component_new(rgb);
+				const char *rgb_labels[] = {"R", _("Red"), "G", _("Green"), "B", _("Blue"), NULL};
+				gtk_color_component_set_label(GTK_COLOR_COMPONENT(widget), rgb_labels);
 				args->rgb_control = widget;
 				g_signal_connect(G_OBJECT(widget), "color-changed", G_CALLBACK(color_component_change_value), args);
 				g_signal_connect(G_OBJECT(widget), "button_release_event", G_CALLBACK(color_component_key_up_cb), args);
+				g_signal_connect(G_OBJECT(widget), "input-clicked", G_CALLBACK(color_component_input_clicked), args);
 				gtk_container_add(GTK_CONTAINER(expander), widget);
 
 			expander=gtk_expander_new("CMYK");
@@ -1042,9 +1219,12 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 			gtk_box_pack_start(GTK_BOX(vbox), expander, FALSE, FALSE, 0);
 
 				widget = gtk_color_component_new(cmyk);
+				const char *cmyk_labels[] = {"C", _("Cyan"), "M", _("Magenta"), "Y", _("Yellow"), "K", _("Key"), NULL};
+				gtk_color_component_set_label(GTK_COLOR_COMPONENT(widget), cmyk_labels);
 				args->cmyk_control = widget;
 				g_signal_connect(G_OBJECT(widget), "color-changed", G_CALLBACK(color_component_change_value), args);
 				g_signal_connect(G_OBJECT(widget), "button_release_event", G_CALLBACK(color_component_key_up_cb), args);
+				g_signal_connect(G_OBJECT(widget), "input-clicked", G_CALLBACK(color_component_input_clicked), args);
 				gtk_container_add(GTK_CONTAINER(expander), widget);
 
 			expander = gtk_expander_new("Lab");
@@ -1053,12 +1233,29 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 			gtk_box_pack_start (GTK_BOX(vbox), expander, FALSE, FALSE, 0);
 
 				widget = gtk_color_component_new(lab);
+				const char *lab_labels[] = {"L", _("Lightness"), "a", "a", "b", "b", NULL};
+				gtk_color_component_set_label(GTK_COLOR_COMPONENT(widget), lab_labels);
 				args->lab_control = widget;
 				g_signal_connect(G_OBJECT(widget), "color-changed", G_CALLBACK(color_component_change_value), args);
 				g_signal_connect(G_OBJECT(widget), "button_release_event", G_CALLBACK(color_component_key_up_cb), args);
+				g_signal_connect(G_OBJECT(widget), "input-clicked", G_CALLBACK(color_component_input_clicked), args);
 				gtk_container_add(GTK_CONTAINER(expander), widget);
 
-			expander=gtk_expander_new("Info");
+			expander = gtk_expander_new("LCH");
+			gtk_expander_set_expanded(GTK_EXPANDER(expander), dynv_get_bool_wd(args->params, "expander.lch", false));
+			args->expanderLCH = expander;
+			gtk_box_pack_start (GTK_BOX(vbox), expander, FALSE, FALSE, 0);
+
+				widget = gtk_color_component_new(lch);
+				const char *lch_labels[] = {"L", _("Lightness"), "C", "Chroma", "H", "Hue", NULL};
+				gtk_color_component_set_label(GTK_COLOR_COMPONENT(widget), lch_labels);
+				args->lch_control = widget;
+				g_signal_connect(G_OBJECT(widget), "color-changed", G_CALLBACK(color_component_change_value), args);
+				g_signal_connect(G_OBJECT(widget), "button_release_event", G_CALLBACK(color_component_key_up_cb), args);
+				g_signal_connect(G_OBJECT(widget), "input-clicked", G_CALLBACK(color_component_input_clicked), args);
+				gtk_container_add(GTK_CONTAINER(expander), widget);
+
+			expander=gtk_expander_new(_("Info"));
 			gtk_expander_set_expanded(GTK_EXPANDER(expander), dynv_get_bool_wd(args->params, "expander.info", false));
 			args->expanderInfo=expander;
 			gtk_box_pack_start (GTK_BOX(vbox), expander, FALSE, FALSE, 0);
@@ -1067,7 +1264,7 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 				table_y=0;
 				gtk_container_add(GTK_CONTAINER(expander), table);
 
-					gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new("Color name:",0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
+					gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new(_("Color name:"),0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
 					widget = gtk_entry_new();
 					gtk_table_attach(GTK_TABLE(table), widget,1,3,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,5,0);
 					gtk_editable_set_editable(GTK_EDITABLE(widget), FALSE);
@@ -1080,18 +1277,18 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 					dd.get_color_object = get_color_object_contrast;
 					dd.set_color_object_at = set_color_object_at_contrast;
 
-					gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new("Contrast:",0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
+					gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new(_("Contrast:"),0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
 					widget = gtk_color_new();
 
 					Color *c;
 					if ((c = const_cast<Color*>(dynv_get_color_wd(args->params, "contrast.color", 0)))){
-						gtk_color_set_color(GTK_COLOR(widget), c, "Sample");
+						gtk_color_set_color(GTK_COLOR(widget), c, _("Sample"));
 					}else{
 						Color c;
 						c.rgb.red = 1;
 						c.rgb.green = 1;
 						c.rgb.blue = 1;
-						gtk_color_set_color(GTK_COLOR(widget), &c, "Sample");
+						gtk_color_set_color(GTK_COLOR(widget), &c, _("Sample"));
 					}
 					gtk_color_set_rounded(GTK_COLOR(widget), true);
 					gtk_color_set_hcenter(GTK_COLOR(widget), true);
@@ -1110,7 +1307,7 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 					table_y++;
 
 
-	updateDiplays(args, 0);
+	updateDisplays(args, 0);
 
 
 	args->main = main_hbox;
@@ -1123,11 +1320,12 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 }
 
 int color_picker_source_register(ColorSourceManager *csm){
-    ColorSource *color_source = new ColorSource;
-	color_source_init(color_source, "color_picker", "Color picker");
+	ColorSource *color_source = new ColorSource;
+	color_source_init(color_source, "color_picker", _("Color picker"));
 	color_source->implement = (ColorSource* (*)(ColorSource *source, GlobalState *gs, struct dynvSystem *dynv_namespace))source_implement;
 	color_source->single_instance_only = true;
-    color_source_manager_add_source(csm, color_source);
+	color_source->default_accelerator = GDK_c;
+	color_source_manager_add_source(csm, color_source);
 	return 0;
 }
 
