@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2011, Albertas Vyšniauskas
+ * Copyright (c) 2009-2012, Albertas Vyšniauskas
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -27,9 +27,14 @@
 #include "ColorPicker.h"
 #include "Converter.h"
 #include "DynvHelpers.h"
+#include "ToolColorNaming.h"
 #include <gdk/gdkkeysyms.h>
 
+#include <string>
+#include <sstream>
+
 using namespace math;
+using namespace std;
 
 typedef struct FloatingPickerArgs{
 	GtkWidget* parent;
@@ -48,6 +53,24 @@ typedef struct FloatingPickerArgs{
 	bool click_mode;
 }FloatingPickerArgs;
 
+class PickerColorNameAssigner: public ToolColorNameAssigner {
+	protected:
+		stringstream m_stream;
+	public:
+		PickerColorNameAssigner(GlobalState *gs):ToolColorNameAssigner(gs){
+		}
+
+		void assign(struct ColorObject *color_object, Color *color){
+			ToolColorNameAssigner::assign(color_object, color);
+		}
+
+		virtual std::string getToolSpecificName(struct ColorObject *color_object, Color *color){
+			m_stream.str("");
+                        m_stream << color_names_get(m_gs->color_names, color, false);
+			return m_stream.str();
+		}
+};
+
 static void get_color_sample(FloatingPickerArgs *args, bool updateWidgets, Color* c){
 
 	GdkScreen *screen;
@@ -224,8 +247,8 @@ static gboolean button_release_cb(GtkWidget *widget, GdkEventButton *event, Floa
 
 			if (dynv_get_bool_wd(args->gs->params, "gpick.picker.sampler.add_on_release", false)){
 
-				string name = color_names_get(args->gs->color_names, &c, dynv_get_bool_wd(args->gs->params, "gpick.color_names.imprecision_postfix", true));
-				dynv_set_string(color_object->params, "name", name.c_str());
+                                PickerColorNameAssigner name_assigner(args->gs);
+                                name_assigner.assign(color_object, &c);
 				color_list_add_color_object(args->gs->colors, color_object, 1);
 
 			}
@@ -245,6 +268,7 @@ static gboolean button_release_cb(GtkWidget *widget, GdkEventButton *event, Floa
 static gboolean key_up_cb (GtkWidget *widget, GdkEventKey *event, FloatingPickerArgs *args){
 
 	GdkEventButton event2;
+	guint modifiers = gtk_accelerator_get_default_mod_mask();
 
 	switch(event->keyval){
 	case GDK_Escape:
@@ -255,6 +279,18 @@ static gboolean key_up_cb (GtkWidget *widget, GdkEventKey *event, FloatingPicker
 		button_release_cb(widget, &event2, args);
 		return TRUE;
 		break;
+	case GDK_m:
+		{
+			int x, y;
+			gdk_display_get_pointer(gdk_display_get_default(), NULL, &x, &y, NULL);
+			math::Vec2<int> position(x, y);
+			if ((event->state&modifiers)==GDK_CONTROL_MASK){
+				gtk_zoomed_set_mark(GTK_ZOOMED(args->zoomed), 1, position);
+			}else{
+				gtk_zoomed_set_mark(GTK_ZOOMED(args->zoomed), 0, position);
+			}
+		}
+		break;
 	default:
 		if (color_picker_key_up(args->color_source, event)){
 			args->release_mode = false;	//key pressed and color picked, disable copy on mouse button release
