@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2011, Albertas Vyšniauskas
+ * Copyright (c) 2009-2012, Albertas Vyšniauskas
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -20,6 +20,7 @@
 #include "DragDrop.h"
 
 #include "GlobalStateStruct.h"
+#include "ToolColorNaming.h"
 #include "uiUtilities.h"
 #include "ColorList.h"
 #include "MathUtil.h"
@@ -31,10 +32,12 @@
 #include "CopyPaste.h"
 #include "Converter.h"
 #include "DynvHelpers.h"
+#include "Internationalisation.h"
 
 #include "uiApp.h"
 
 #include <gdk/gdkkeysyms.h>
+#include <boost/lexical_cast.hpp>
 
 #include <math.h>
 #include <string.h>
@@ -83,36 +86,53 @@ typedef struct GenerateSchemeArgs{
 	struct ColorList *preview_color_list;
 }GenerateSchemeArgs;
 
-typedef struct SchemeType{
-	const char *name;
-	int32_t colors;
-	int32_t turn_types;
-	double turn[4];
-}SchemeType;
-
 const SchemeType scheme_types[]={
-	{"Complementary", 1, 1, {180}},
-	{"Analogous", 5, 1, {30}},
-	{"Triadic", 2, 1, {120}},
-	{"Split-Complementary", 2, 2, {150, 60}},
-	{"Rectangle (tetradic)", 3, 2, {60, 120}},
-	{"Square", 3, 1, {90}},
-	{"Neutral", 5, 1, {15}},
-	{"Clash", 2, 2, {90, 180}},
-	{"Five-Tone", 4, 4, {115, 40, 50, 40}},
-	{"Six-Tone", 5, 2, {30, 90}},
+	{N_("Complementary"), 1, 1, {180}},
+	{N_("Analogous"), 5, 1, {30}},
+	{N_("Triadic"), 2, 1, {120}},
+	{N_("Split-Complementary"), 2, 2, {150, 60}},
+	{N_("Rectangle (tetradic)"), 3, 2, {60, 120}},
+	{N_("Square"), 3, 1, {90}},
+	{N_("Neutral"), 5, 1, {15}},
+	{N_("Clash"), 2, 2, {90, 180}},
+	{N_("Five-Tone"), 4, 4, {115, 40, 50, 40}},
+	{N_("Six-Tone"), 5, 2, {30, 90}},
+};
+
+class GenerateSchemeColorNameAssigner: public ToolColorNameAssigner {
+	protected:
+		stringstream m_stream;
+		int32_t m_ident;
+		int32_t m_schemetype;
+	public:
+		GenerateSchemeColorNameAssigner(GlobalState *gs):ToolColorNameAssigner(gs){
+		}
+
+		void assign(struct ColorObject *color_object, Color *color, const int32_t ident, const int32_t schemetype){
+			m_ident = ident;
+			m_schemetype = schemetype;
+			ToolColorNameAssigner::assign(color_object, color);
+		}
+
+		virtual std::string getToolSpecificName(struct ColorObject *color_object, Color *color){
+			m_stream.str("");
+			m_stream << _("scheme") << " " << _(generate_scheme_get_scheme_type(m_schemetype)->name) << " #" << m_ident << "[" << color_names_get(m_gs->color_names, color, false) << "]";
+			return m_stream.str();
+		}
 };
 
 static int set_rgb_color(GenerateSchemeArgs *args, struct ColorObject* color, uint32_t color_index);
 static int set_rgb_color_by_widget(GenerateSchemeArgs *args, struct ColorObject* color, GtkWidget* color_widget);
 
+// XXX needs to have name assignment in here.
+
 static void calc(GenerateSchemeArgs *args, bool preview, bool save_settings){
 
 	int32_t type = gtk_combo_box_get_active(GTK_COMBO_BOX(args->gen_type));
 	int32_t wheel_type = gtk_combo_box_get_active(GTK_COMBO_BOX(args->wheel_type));
 
 	gtk_color_wheel_set_color_wheel_type(GTK_COLOR_WHEEL(args->color_wheel), &color_wheel_types_get()[wheel_type]);
-	gtk_color_wheel_set_n_colors(GTK_COLOR_WHEEL(args->color_wheel), scheme_types[type].colors + 1);
+	gtk_color_wheel_set_n_colors(GTK_COLOR_WHEEL(args->color_wheel), generate_scheme_get_scheme_type(type)->colors + 1);
 
 	//gfloat chaos = gtk_spin_button_get_value(GTK_SPIN_BUTTON(args->range_chaos));
 	//gboolean correction = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(args->toggle_brightness_correction));
@@ -151,7 +171,7 @@ static void calc(GenerateSchemeArgs *args, bool preview, bool save_settings){
 
 	Color hsv;
 
-	for (step_i = 0; step_i <= scheme_types[type].colors; ++step_i) {
+	for (step_i = 0; step_i <= generate_scheme_get_scheme_type(type)->colors; ++step_i) {
 
 		wheel->hue_to_hsl(wrap_float(hue + args->mod[step_i].hue), &hsl);
 		hsl.hsl.lightness = clamp_float(hsl.hsl.lightness + lightness, 0, 1);
@@ -177,7 +197,7 @@ static void calc(GenerateSchemeArgs *args, bool preview, bool save_settings){
 		gtk_color_wheel_set_saturation(GTK_COLOR_WHEEL(args->color_wheel), step_i, hsv.hsv.saturation);
 		gtk_color_wheel_set_value(GTK_COLOR_WHEEL(args->color_wheel), step_i, hsv.hsv.value);
 
-		hue_step = (scheme_types[type].turn[step_i%scheme_types[type].turn_types]) / (360.0)
+		hue_step = (generate_scheme_get_scheme_type(type)->turn[step_i % generate_scheme_get_scheme_type(type)->turn_types]) / (360.0)
 			+ chaos*(((random_get(args->gs->random)&0xFFFFFFFF)/(gdouble)0xFFFFFFFF)-0.5);
 
 		hue = wrap_float(hue + hue_step);
@@ -187,7 +207,7 @@ static void calc(GenerateSchemeArgs *args, bool preview, bool save_settings){
 
 	if (preview){
 
-		uint32_t total_colors = scheme_types[type].colors+1;
+		uint32_t total_colors = generate_scheme_get_scheme_type(type)->colors + 1;
 		if (total_colors > MAX_COLOR_WIDGETS) total_colors = MAX_COLOR_WIDGETS;
 
 		for (uint32_t i = args->colors_visible; i > total_colors; --i)
@@ -269,14 +289,14 @@ static void color_wheel_show_menu(GtkWidget* widget, GenerateSchemeArgs* args, G
 
 	menu = gtk_menu_new();
 
-	item = gtk_check_menu_item_new_with_mnemonic("_Locked");
+	item = gtk_check_menu_item_new_with_mnemonic(_("_Locked"));
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), args->wheel_locked);
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 	g_signal_connect(G_OBJECT(item), "toggled", G_CALLBACK(color_wheel_locked_toggled_cb), args);
 
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), gtk_separator_menu_item_new ());
 
-	item = gtk_menu_item_new_with_image("_Reset scheme", gtk_image_new_from_stock(GTK_STOCK_CANCEL, GTK_ICON_SIZE_MENU));
+	item = gtk_menu_item_new_with_image(_("_Reset scheme"), gtk_image_new_from_stock(GTK_STOCK_CANCEL, GTK_ICON_SIZE_MENU));
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 	g_signal_connect(G_OBJECT (item), "activate", G_CALLBACK(color_reset_scheme_cb), args);
 
@@ -356,34 +376,44 @@ static void on_color_edit(GtkWidget *widget,  gpointer item) {
 	color_object_release(color_object);
 }
 
+static string identify_color_widget(GtkWidget *widget, GenerateSchemeArgs *args)
+{
+	for (int i = 0; i < MAX_COLOR_WIDGETS; ++i){
+		if (args->colors[i] == widget){
+			return "result " + boost::lexical_cast<string>(i + 1);
+		}
+	}
+	return "unknown";
+}
 
-static void on_color_add_to_palette(GtkWidget *widget,  gpointer item) {
-	GenerateSchemeArgs* args=(GenerateSchemeArgs*)item;
+static void add_color_to_palette(GtkWidget *color_widget, GenerateSchemeColorNameAssigner &name_assigner, GenerateSchemeArgs *args)
+{
 	Color c;
-
-	gtk_color_get_color(GTK_COLOR(g_object_get_data(G_OBJECT(widget), "color_widget")), &c);
-
-	struct ColorObject *color_object=color_list_new_color_object(args->gs->colors, &c);
-	string name=color_names_get(args->gs->color_names, &c, dynv_get_bool_wd(args->gs->params, "gpick.color_names.imprecision_postfix", true));
-	dynv_set_string(color_object->params, "name", name.c_str());
+	struct ColorObject *color_object;
+	string widget_ident;
+        int32_t type = gtk_combo_box_get_active(GTK_COMBO_BOX(args->gen_type));
+	gtk_color_get_color(GTK_COLOR(color_widget), &c);
+	color_object = color_list_new_color_object(args->gs->colors, &c);
+	widget_ident = identify_color_widget(color_widget, args);
+	name_assigner.assign(color_object, &c, atoi(widget_ident.c_str()+7), type);
 	color_list_add_color_object(args->gs->colors, color_object, 1);
 	color_object_release(color_object);
 }
 
+static void on_color_add_to_palette(GtkWidget *widget,  gpointer item) {
+	GenerateSchemeArgs* args = (GenerateSchemeArgs*)item;
+	GtkWidget *color_widget = GTK_WIDGET(g_object_get_data(G_OBJECT(widget), "color_widget"));
+	GenerateSchemeColorNameAssigner name_assigner(args->gs);
+	add_color_to_palette(color_widget, name_assigner, args);
+}
+
 static void on_color_add_all_to_palette(GtkWidget *widget,  gpointer item) {
-	GenerateSchemeArgs* args=(GenerateSchemeArgs*)item;
-	Color c;
+	GenerateSchemeArgs* args = (GenerateSchemeArgs*)item;
+	GenerateSchemeColorNameAssigner name_assigner(args->gs);
 
 	for (int i = 0; i < args->colors_visible; ++i){
-		gtk_color_get_color(GTK_COLOR(args->colors[i]), &c);
-
-		struct ColorObject *color_object=color_list_new_color_object(args->gs->colors, &c);
-		string name=color_names_get(args->gs->color_names, &c, dynv_get_bool_wd(args->gs->params, "gpick.color_names.imprecision_postfix", true));
-		dynv_set_string(color_object->params, "name", name.c_str());
-		color_list_add_color_object(args->gs->colors, color_object, 1);
-		color_object_release(color_object);
+		add_color_to_palette(args->colors[i], name_assigner, args);
 	}
-
 }
 
 static gboolean color_focus_in_cb(GtkWidget *widget, GdkEventFocus *event, GenerateSchemeArgs *args){
@@ -401,15 +431,8 @@ static gboolean color_focus_in_cb(GtkWidget *widget, GdkEventFocus *event, Gener
 
 static void on_color_activate(GtkWidget *widget,  gpointer item) {
 	GenerateSchemeArgs* args=(GenerateSchemeArgs*)item;
-	Color c;
-
-	gtk_color_get_color(GTK_COLOR(widget), &c);
-
-	struct ColorObject *color_object=color_list_new_color_object(args->gs->colors, &c);
-	string name=color_names_get(args->gs->color_names, &c, dynv_get_bool_wd(args->gs->params, "gpick.color_names.imprecision_postfix", true));
-	dynv_set_string(color_object->params, "name", name.c_str());
-	color_list_add_color_object(args->gs->colors, color_object, 1);
-	color_object_release(color_object);
+	GenerateSchemeColorNameAssigner name_assigner(args->gs);
+	add_color_to_palette(widget, name_assigner, args);
 }
 
 static void color_show_menu(GtkWidget* widget, GenerateSchemeArgs* args, GdkEventButton *event ){
@@ -418,19 +441,19 @@ static void color_show_menu(GtkWidget* widget, GenerateSchemeArgs* args, GdkEven
 
 	menu = gtk_menu_new ();
 
-	item = gtk_menu_item_new_with_image ("_Add to palette", gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
+	item = gtk_menu_item_new_with_image (_("_Add to palette"), gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (on_color_add_to_palette), args);
 	g_object_set_data(G_OBJECT(item), "color_widget", widget);
 
-	item = gtk_menu_item_new_with_image ("_Add all to palette", gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
+	item = gtk_menu_item_new_with_image (_("A_dd all to palette"), gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (on_color_add_all_to_palette), args);
 	g_object_set_data(G_OBJECT(item), "color_widget", widget);
 
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), gtk_separator_menu_item_new ());
 
-	item = gtk_menu_item_new_with_mnemonic ("_Copy to clipboard");
+	item = gtk_menu_item_new_with_mnemonic (_("_Copy to clipboard"));
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
 	Color c;
@@ -443,12 +466,12 @@ static void color_show_menu(GtkWidget* widget, GenerateSchemeArgs* args, GdkEven
 
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), gtk_separator_menu_item_new ());
 
-	item = gtk_menu_item_new_with_image ("_Edit...", gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU));
+	item = gtk_menu_item_new_with_image (_("_Edit..."), gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU));
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (on_color_edit), args);
 	g_object_set_data(G_OBJECT(item), "color_widget", widget);
 
-	item = gtk_menu_item_new_with_image ("_Paste", gtk_image_new_from_stock(GTK_STOCK_PASTE, GTK_ICON_SIZE_MENU));
+	item = gtk_menu_item_new_with_image (_("_Paste"), gtk_image_new_from_stock(GTK_STOCK_PASTE, GTK_ICON_SIZE_MENU));
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (on_color_paste), args);
 	g_object_set_data(G_OBJECT(item), "color_widget", widget);
@@ -459,7 +482,7 @@ static void color_show_menu(GtkWidget* widget, GenerateSchemeArgs* args, GdkEven
 
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), gtk_separator_menu_item_new ());
 
-	item = gtk_menu_item_new_with_image("_Reset", gtk_image_new_from_stock(GTK_STOCK_CANCEL, GTK_ICON_SIZE_MENU));
+	item = gtk_menu_item_new_with_image(_("_Reset"), gtk_image_new_from_stock(GTK_STOCK_CANCEL, GTK_ICON_SIZE_MENU));
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 	g_signal_connect(G_OBJECT (item), "activate", G_CALLBACK(color_reset_cb), args);
 	g_object_set_data(G_OBJECT(item), "color_widget", widget);
@@ -557,16 +580,19 @@ static int source_destroy(GenerateSchemeArgs *args){
 }
 
 static int source_get_color(GenerateSchemeArgs *args, ColorObject** color){
+	GenerateSchemeColorNameAssigner name_assigner(args->gs);
 	Color c;
+	string widget_ident;
+        int32_t type = gtk_combo_box_get_active(GTK_COMBO_BOX(args->gen_type));
 	if (args->last_focused_color){
 		gtk_color_get_color(GTK_COLOR(args->last_focused_color), &c);
+		widget_ident = identify_color_widget(args->last_focused_color, args);
 	}else{
 		gtk_color_get_color(GTK_COLOR(args->colors[0]), &c);
+		widget_ident = identify_color_widget(args->colors[0], args);
 	}
 	*color = color_list_new_color_object(args->gs->colors, &c);
-
-	string name = color_names_get(args->gs->color_names, &c, dynv_get_bool_wd(args->gs->params, "gpick.color_names.imprecision_postfix", true));
-	dynv_set_string((*color)->params, "name", name.c_str());
+	name_assigner.assign(*color, &c, atoi(widget_ident.c_str() + 7), type);
 	return 0;
 }
 
@@ -622,6 +648,10 @@ static int set_rgb_color(GenerateSchemeArgs *args, struct ColorObject* color, ui
 	return 0;
 }
 
+static int source_set_nth_color(GenerateSchemeArgs *args, uint32_t color_n, ColorObject* color){
+	if (color_n < 0 || color_n > 6) return -1;
+	return set_rgb_color(args, color, color_n);
+}
 
 static int source_set_color(GenerateSchemeArgs *args, struct ColorObject* color){
 	if (args->last_focused_color){
@@ -632,6 +662,12 @@ static int source_set_color(GenerateSchemeArgs *args, struct ColorObject* color)
 }
 
 static int source_activate(GenerateSchemeArgs *args){
+
+	transformation::Chain *chain = static_cast<transformation::Chain*>(dynv_get_pointer_wdc(args->gs->params, "TransformationChain", 0));
+	for (uint32_t i = 0; i < MAX_COLOR_WIDGETS; ++i){
+		gtk_color_set_transformation_chain(GTK_COLOR(args->colors[i]), chain);
+	}
+
 	gtk_statusbar_push(GTK_STATUSBAR(args->statusbar), gtk_statusbar_get_context_id(GTK_STATUSBAR(args->statusbar), "empty"), "");
 	return 0;
 }
@@ -719,6 +755,7 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 	args->source.destroy = (int (*)(ColorSource *source))source_destroy;
 	args->source.get_color = (int (*)(ColorSource *source, ColorObject** color))source_get_color;
 	args->source.set_color = (int (*)(ColorSource *source, ColorObject* color))source_set_color;
+	args->source.set_nth_color = (int (*)(ColorSource *source, uint32_t color_n, ColorObject* color))source_set_nth_color;
 	args->source.deactivate = (int (*)(ColorSource *source))source_deactivate;
 	args->source.activate = (int (*)(ColorSource *source))source_activate;
 
@@ -749,7 +786,7 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 	gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 5);
 
 	args->color_previews = gtk_table_new(3, 2, false);
-	gtk_box_pack_start(GTK_BOX(vbox), args->color_previews, true, true, 5);
+	gtk_box_pack_start(GTK_BOX(vbox), args->color_previews, true, true, 0);
 
 	struct DragDrop dd;
 	dragdrop_init(&dd, gs);
@@ -758,9 +795,10 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 	dd.get_color_object = get_color_object;
 	dd.set_color_object_at = set_color_object_at;
 
-	for (int i = 0; i < MAX_COLOR_WIDGETS; ++i){
+	for (intptr_t i = 0; i < MAX_COLOR_WIDGETS; ++i){
 		widget = gtk_color_new();
 		gtk_color_set_rounded(GTK_COLOR(widget), true);
+		gtk_color_set_roundness(GTK_COLOR(widget), 5);
 		gtk_color_set_hcenter(GTK_COLOR(widget), true);
 
 		gtk_table_attach(GTK_TABLE(args->color_previews), widget, i % 2, (i % 2) + 1, i / 2, i / 2 + 1, GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL | GTK_EXPAND), 0, 0);
@@ -785,7 +823,7 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 
 
 	hbox2 = gtk_hbox_new(FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox2, false, false, 5);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox2, false, false, 0);
 
     args->color_wheel = gtk_color_wheel_new();
 	gtk_box_pack_start(GTK_BOX(hbox2), args->color_wheel, false, false, 0);
@@ -807,17 +845,17 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 
 	gint table_y;
 	table = gtk_table_new(5, 2, false);
-	gtk_box_pack_start(GTK_BOX(hbox2), table, true, true, 5);
+	gtk_box_pack_start(GTK_BOX(hbox2), table, true, true, 0);
 	table_y = 0;
 
-	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new("Hue:",0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
+	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new(_("Hue:"),0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
 	args->hue = gtk_hscale_new_with_range(0, 360, 1);
 	gtk_range_set_value(GTK_RANGE(args->hue), dynv_get_float_wd(args->params, "hue", 180));
 	g_signal_connect (G_OBJECT (args->hue), "value-changed", G_CALLBACK (update), args);
 	gtk_table_attach(GTK_TABLE(table), args->hue,1,2,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,5,0);
 	table_y++;
 
-	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new("Saturation:",0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
+	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new(_("Saturation:"),0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
 	args->saturation = gtk_hscale_new_with_range(0, 120, 1);
 	gtk_range_set_value(GTK_RANGE(args->saturation), dynv_get_float_wd(args->params, "saturation", 100));
 	g_signal_connect (G_OBJECT (args->saturation), "value-changed", G_CALLBACK (update), args);
@@ -825,7 +863,7 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 	gtk_table_attach(GTK_TABLE(table), args->saturation,1,2,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,5,0);
 	table_y++;
 
-	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new("Lightness:",0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
+	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new(_("Lightness:"),0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
 	args->lightness = gtk_hscale_new_with_range(-50, 80, 1);
 	gtk_range_set_value(GTK_RANGE(args->lightness), dynv_get_float_wd(args->params, "lightness", 0));
 	g_signal_connect (G_OBJECT (args->lightness), "value-changed", G_CALLBACK (update), args);
@@ -834,21 +872,21 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 	table_y++;
 
 	//table_y=0;
-	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new("Type:",0,0.5,0,0),0,1,table_y,table_y+1, GTK_FILL, GTK_SHRINK, 5, 5);
+	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new(_("Type:"),0,0.5,0,0),0,1,table_y,table_y+1, GTK_FILL, GTK_SHRINK, 5, 5);
 	args->gen_type = gtk_combo_box_new_text();
-	for (uint32_t i=0; i<sizeof(scheme_types)/sizeof(SchemeType); i++){
-		gtk_combo_box_append_text(GTK_COMBO_BOX(args->gen_type), scheme_types[i].name);
+	for (uint32_t i = 0; i < generate_scheme_get_n_scheme_types(); i++){
+		gtk_combo_box_append_text(GTK_COMBO_BOX(args->gen_type), _(generate_scheme_get_scheme_type(i)->name));
 	}
 	gtk_combo_box_set_active(GTK_COMBO_BOX(args->gen_type), dynv_get_int32_wd(args->params, "type", 0));
 	g_signal_connect (G_OBJECT (args->gen_type), "changed", G_CALLBACK(update), args);
 	gtk_table_attach(GTK_TABLE(table), args->gen_type,1,2,table_y,table_y+1, GTK_FILL, GTK_SHRINK,5,0);
     table_y++;
 
-	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new("Color wheel:",0,0.5,0,0),0,1,table_y,table_y+1, GTK_FILL, GTK_SHRINK, 5, 5);
+	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new(_("Color wheel:"),0,0.5,0,0),0,1,table_y,table_y+1, GTK_FILL, GTK_SHRINK, 5, 5);
 	args->wheel_type = gtk_combo_box_new_text();
 
 	for (uint32_t i = 0; i < color_wheel_types_get_n(); i++){
-		gtk_combo_box_append_text(GTK_COMBO_BOX(args->wheel_type), color_wheel_types_get()[i].name);
+		gtk_combo_box_append_text(GTK_COMBO_BOX(args->wheel_type), _(color_wheel_types_get()[i].name));
 	}
 	gtk_combo_box_set_active(GTK_COMBO_BOX(args->wheel_type), dynv_get_int32_wd(args->params, "wheel_type", 0));
 	g_signal_connect (G_OBJECT (args->wheel_type), "changed", G_CALLBACK(update), args);
@@ -877,10 +915,25 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 }
 
 int generate_scheme_source_register(ColorSourceManager *csm){
-    ColorSource *color_source = new ColorSource;
-	color_source_init(color_source, "generate_scheme", "Scheme generation");
+	ColorSource *color_source = new ColorSource;
+	color_source_init(color_source, "generate_scheme", _("Scheme generation"));
 	color_source->implement = (ColorSource* (*)(ColorSource *source, GlobalState *gs, struct dynvSystem *dynv_namespace))source_implement;
-    color_source_manager_add_source(csm, color_source);
+	color_source->default_accelerator = GDK_g;
+	color_source_manager_add_source(csm, color_source);
 	return 0;
 }
 
+const SchemeType* generate_scheme_get_scheme_type(uint32_t index)
+{
+	if (index >= 0 && index < generate_scheme_get_n_scheme_types())
+		return &scheme_types[index];
+	else
+		return 0;
+}
+
+uint32_t generate_scheme_get_n_scheme_types()
+{
+	return sizeof(scheme_types) / sizeof(SchemeType);
+}
+
+
