@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2011, Albertas Vyšniauskas
+ * Copyright (c) 2009-2012, Albertas Vyšniauskas
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -23,12 +23,15 @@
 #include "uiApp.h"
 #include "Converter.h"
 #include "DynvHelpers.h"
+#include "Internationalisation.h"
 
 #include "GlobalStateStruct.h"
+#include "ToolColorNaming.h"
 #include "uiUtilities.h"
 #include "ColorList.h"
 #include "MathUtil.h"
 
+#include <gdk/gdkkeysyms.h>
 #include "gtk/LayoutPreview.h"
 #include "layout/Layout.h"
 #include "layout/Style.h"
@@ -57,6 +60,26 @@ typedef struct LayoutPreviewArgs{
 	GlobalState *gs;
 }LayoutPreviewArgs;
 
+class LayoutPreviewColorNameAssigner: public ToolColorNameAssigner {
+	protected:
+		stringstream m_stream;
+		const char *m_ident;
+	public:
+		LayoutPreviewColorNameAssigner(GlobalState *gs):ToolColorNameAssigner(gs){
+		}
+
+		void assign(struct ColorObject *color_object, Color *color, const char *ident){
+			m_ident = ident;
+			ToolColorNameAssigner::assign(color_object, color);
+		}
+
+		virtual std::string getToolSpecificName(struct ColorObject *color_object, Color *color){
+			m_stream.str("");
+			m_stream << _("layout preview") << " " << m_ident << " [" << color_names_get(m_gs->color_names, color, false) << "]";
+			return m_stream.str();
+		}
+};
+
 typedef enum{
 	LAYOUTLIST_HUMAN_NAME = 0,
 	LAYOUTLIST_PTR,
@@ -127,7 +150,7 @@ static GtkWidget* style_list_new(LayoutPreviewArgs *args){
 	col = gtk_tree_view_column_new();
 	gtk_tree_view_column_set_sizing(col,GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 	gtk_tree_view_column_set_resizable(col, true);
-	gtk_tree_view_column_set_title(col, "Style item");
+	gtk_tree_view_column_set_title(col, _("Style item"));
 	renderer = gtk_cell_renderer_text_new();
 	gtk_tree_view_column_pack_start(col, renderer, true);
 	gtk_tree_view_column_add_attribute(col, renderer, "text", STYLELIST_HUMAN_NAME);
@@ -136,7 +159,7 @@ static GtkWidget* style_list_new(LayoutPreviewArgs *args){
 	col = gtk_tree_view_column_new();
 	gtk_tree_view_column_set_sizing(col,GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 	gtk_tree_view_column_set_resizable(col, true);
-	gtk_tree_view_column_set_title(col, "CSS selector");
+	gtk_tree_view_column_set_title(col, _("CSS selector"));
 	renderer = gtk_cell_renderer_text_new();
 	gtk_tree_view_column_pack_start(col, renderer, true);
 	gtk_tree_view_column_add_attribute(col, renderer, "text", STYLELIST_CSS_SELECTOR);
@@ -158,7 +181,7 @@ static void assign_css_selectors_cb(GtkWidget *widget, LayoutPreviewArgs* args)
 
 	GtkWidget *table;
 
-	GtkWidget *dialog = gtk_dialog_new_with_buttons("Assign CSS selectors", GTK_WINDOW(gtk_widget_get_toplevel(args->main)), GtkDialogFlags(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
+	GtkWidget *dialog = gtk_dialog_new_with_buttons(_("Assign CSS selectors"), GTK_WINDOW(gtk_widget_get_toplevel(args->main)), GtkDialogFlags(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
 			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 			GTK_STOCK_OK, GTK_RESPONSE_OK,
 			NULL);
@@ -246,15 +269,12 @@ static int source_destroy(LayoutPreviewArgs *args){
 }
 
 static int source_get_color(LayoutPreviewArgs *args, struct ColorObject** color){
-	Color c;
-	if (gtk_layout_preview_get_current_color(GTK_LAYOUT_PREVIEW(args->layout), &c)==0){
-
-		struct ColorObject *color_object = color_list_new_color_object(args->gs->colors, &c);
-		string name = color_names_get(args->gs->color_names, &c, dynv_get_bool_wd(args->gs->params, "gpick.color_names.imprecision_postfix", true));
-		dynv_set_string(color_object->params, "name", name.c_str());
-
+	Style *style = 0;
+	if (gtk_layout_preview_get_current_style(GTK_LAYOUT_PREVIEW(args->layout), &style) == 0){
+		struct ColorObject *color_object = color_list_new_color_object(args->gs->colors, &style->color);
+        LayoutPreviewColorNameAssigner name_assigner(args->gs);
+		name_assigner.assign(color_object, &style->color, style->ident_name.c_str());
 		*color = color_object;
-
 		return 0;
 	}
 	return -1;
@@ -341,29 +361,28 @@ static void paste_cb(GtkWidget *widget, LayoutPreviewArgs* args) {
 	}
 }
 
-static void add_to_palette_cb(GtkWidget *widget,  gpointer item) {
-	LayoutPreviewArgs* args=(LayoutPreviewArgs*)item;
+static void add_color_to_palette(Style *style, LayoutPreviewColorNameAssigner &name_assigner, LayoutPreviewArgs *args)
+{
+	struct ColorObject *color_object;
+	color_object = color_list_new_color_object(args->gs->colors, &style->color);
+	name_assigner.assign(color_object, &style->color, style->ident_name.c_str());
+	color_list_add_color_object(args->gs->colors, color_object, 1);
+	color_object_release(color_object);
+}
 
+static void add_to_palette_cb(GtkWidget *widget,  gpointer item) {
+	LayoutPreviewArgs* args = (LayoutPreviewArgs*)item;
+	LayoutPreviewColorNameAssigner name_assigner(args->gs);
 	Style* style = 0;
 	if (gtk_layout_preview_get_current_style(GTK_LAYOUT_PREVIEW(args->layout), &style) == 0){
-		struct ColorObject *color_object;
-		color_object = color_list_new_color_object(args->gs->colors, &style->color);
-		dynv_set_string(color_object->params, "name", style->ident_name.c_str());
-		color_list_add_color_object(args->gs->colors, color_object, 1);
-		color_object_release(color_object);
+		add_color_to_palette(style, name_assigner, args);
 	}
 }
 
 static void add_all_to_palette_cb(GtkWidget *widget, LayoutPreviewArgs *args) {
-
-	struct ColorObject *color_object;
-
+	LayoutPreviewColorNameAssigner name_assigner(args->gs);
 	for (list<Style*>::iterator i = args->layout_system->styles.begin(); i != args->layout_system->styles.end(); i++){
-
-		color_object = color_list_new_color_object(args->gs->colors, &(*i)->color);
-		dynv_set_string(color_object->params, "name", (*i)->ident_name.c_str());
-		color_list_add_color_object(args->gs->colors, color_object, 1);
-		color_object_release(color_object);
+		add_color_to_palette(*i, name_assigner, args);
 	}
 }
 
@@ -383,18 +402,18 @@ static gboolean button_press_cb (GtkWidget *widget, GdkEventButton *event, Layou
 
 		bool selection_avail = gtk_layout_preview_is_selected(GTK_LAYOUT_PREVIEW(args->layout));
 
-	    item = gtk_menu_item_new_with_image ("_Add to palette", gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
+	    item = gtk_menu_item_new_with_image(_("_Add to palette"), gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
 	    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	    g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (add_to_palette_cb), args);
 		if (!selection_avail) gtk_widget_set_sensitive(item, false);
 
-	    item = gtk_menu_item_new_with_image ("_Add all to palette", gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
+	    item = gtk_menu_item_new_with_image(_("A_dd all to palette"), gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
 	    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	    g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (add_all_to_palette_cb), args);
 
 	    gtk_menu_shell_append (GTK_MENU_SHELL (menu), gtk_separator_menu_item_new ());
 
-	    item = gtk_menu_item_new_with_mnemonic ("_Copy to clipboard");
+	    item = gtk_menu_item_new_with_mnemonic(_("_Copy to clipboard"));
 	    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
 		if (selection_avail){
@@ -408,12 +427,12 @@ static gboolean button_press_cb (GtkWidget *widget, GdkEventButton *event, Layou
 
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), gtk_separator_menu_item_new ());
 
-		item = gtk_menu_item_new_with_image ("_Edit...", gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU));
+		item = gtk_menu_item_new_with_image (_("_Edit..."), gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 		g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (edit_cb), args);
 		if (!selection_avail) gtk_widget_set_sensitive(item, false);
 
-		item = gtk_menu_item_new_with_image ("_Paste", gtk_image_new_from_stock(GTK_STOCK_PASTE, GTK_ICON_SIZE_MENU));
+		item = gtk_menu_item_new_with_image (_("_Paste"), gtk_image_new_from_stock(GTK_STOCK_PASTE, GTK_ICON_SIZE_MENU));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 		g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (paste_cb), args);
 		if (!selection_avail) gtk_widget_set_sensitive(item, false);
@@ -525,7 +544,7 @@ static void export_css_cb(GtkWidget *widget, LayoutPreviewArgs* args){
 
 		}else{
 			GtkWidget* message;
-			message=gtk_message_dialog_new(GTK_WINDOW(gtk_widget_get_toplevel(widget)), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "File could not be saved");
+			message=gtk_message_dialog_new(GTK_WINDOW(gtk_widget_get_toplevel(widget)), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("File could not be saved"));
 			gtk_dialog_run(GTK_DIALOG(message));
 			gtk_widget_destroy(message);
 		}
@@ -536,7 +555,7 @@ static void export_css_cb(GtkWidget *widget, LayoutPreviewArgs* args){
 	GtkWidget *dialog;
 	GtkFileFilter *filter;
 
-	dialog = gtk_file_chooser_dialog_new ("Export", GTK_WINDOW(gtk_widget_get_toplevel(widget)),
+	dialog = gtk_file_chooser_dialog_new(_("Export"), GTK_WINDOW(gtk_widget_get_toplevel(widget)),
 						  GTK_FILE_CHOOSER_ACTION_SAVE,
 						  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 						  GTK_STOCK_SAVE, GTK_RESPONSE_OK,
@@ -550,7 +569,7 @@ static void export_css_cb(GtkWidget *widget, LayoutPreviewArgs* args){
 	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), default_path);
 
 	filter = gtk_file_filter_new();
-	gtk_file_filter_set_name(filter, "Cascading Style Sheets *.css");
+	gtk_file_filter_set_name(filter, _("Cascading Style Sheets *.css"));
 	gtk_file_filter_add_pattern(filter, "*.css");
 	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);
 	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(dialog), filter);
@@ -573,7 +592,7 @@ static void export_css_cb(GtkWidget *widget, LayoutPreviewArgs* args){
 				finished = true;
 			}else{
 				GtkWidget* message;
-				message=gtk_message_dialog_new(GTK_WINDOW(dialog), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "File could not be saved");
+				message=gtk_message_dialog_new(GTK_WINDOW(dialog), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("File could not be saved"));
 				//gtk_window_set_title(GTK_WINDOW(dialog), "Open");
 				gtk_dialog_run(GTK_DIALOG(message));
 				gtk_widget_destroy(message);
@@ -599,6 +618,10 @@ static GtkWidget* attach_label(GtkWidget *widget, const char *label){
 }
 
 static int source_activate(LayoutPreviewArgs *args){
+
+	transformation::Chain *chain = static_cast<transformation::Chain*>(dynv_get_pointer_wdc(args->gs->params, "TransformationChain", 0));
+	gtk_layout_preview_set_transformation_chain(GTK_LAYOUT_PREVIEW(args->layout), chain);
+
 	gtk_statusbar_push(GTK_STATUSBAR(args->statusbar), gtk_statusbar_get_context_id(GTK_STATUSBAR(args->statusbar), "empty"), "");
 	return 0;
 }
@@ -638,16 +661,16 @@ static ColorSource* source_implement(ColorSource *source, GlobalState* gs, struc
 	table_y++;
 
 	tool = gtk_tool_item_new();
-    gtk_tool_item_set_expand(tool, true);
+	gtk_tool_item_set_expand(tool, true);
 	GtkWidget *layout_dropdown = layout_preview_dropdown_new(args, 0);
-    gtk_container_add(GTK_CONTAINER(tool), attach_label(layout_dropdown, "Layout:"));
+	gtk_container_add(GTK_CONTAINER(tool), attach_label(layout_dropdown, _("Layout:")));
 	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tool, -1);
 	g_signal_connect (G_OBJECT(layout_dropdown), "changed", G_CALLBACK(layout_changed_cb), args);
 
 	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), gtk_separator_tool_item_new(), -1);
 
-	tool = gtk_menu_tool_button_new(gtk_image_new_from_stock(GTK_STOCK_SAVE, GTK_ICON_SIZE_BUTTON), "Export CSS File");
-	gtk_tool_item_set_tooltip_text(tool, "Export CSS file");
+	tool = gtk_menu_tool_button_new(gtk_image_new_from_stock(GTK_STOCK_SAVE, GTK_ICON_SIZE_BUTTON), _("Export CSS File"));
+	gtk_tool_item_set_tooltip_text(tool, _("Export CSS file"));
 	g_signal_connect(G_OBJECT(tool), "clicked", G_CALLBACK(export_css_cb), args);
 	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tool, -1);
 
@@ -656,11 +679,11 @@ static ColorSource* source_implement(ColorSource *source, GlobalState* gs, struc
 
 	menu = gtk_menu_new ();
 
-	item = gtk_menu_item_new_with_image("_Export CSS File As...", gtk_image_new_from_stock(GTK_STOCK_SAVE_AS, GTK_ICON_SIZE_MENU));
+	item = gtk_menu_item_new_with_image(_("_Export CSS File As..."), gtk_image_new_from_stock(GTK_STOCK_SAVE_AS, GTK_ICON_SIZE_MENU));
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 	g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (export_css_as_cb), args);
 
-	item = gtk_menu_item_new_with_mnemonic("_Assign CSS Selectors...");
+	item = gtk_menu_item_new_with_mnemonic(_("_Assign CSS Selectors..."));
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 	g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (assign_css_selectors_cb), args);
 
@@ -683,7 +706,6 @@ static ColorSource* source_implement(ColorSource *source, GlobalState* gs, struc
 	dd.test_at = test_at;
 	dd.handler_map = dynv_system_get_handler_map(gs->colors->params);
 
-
 	gtk_drag_dest_set(args->layout, GtkDestDefaults(GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT), 0, 0, GDK_ACTION_COPY);
 	gtk_drag_source_set(args->layout, GDK_BUTTON1_MASK, 0, 0, GDK_ACTION_COPY);
 	dragdrop_widget_attach(args->layout, DragDropFlags(DRAGDROP_SOURCE | DRAGDROP_DESTINATION), &dd);
@@ -691,6 +713,7 @@ static ColorSource* source_implement(ColorSource *source, GlobalState* gs, struc
 	args->gs = gs;
 
 
+
 	// Restore settings and fill list
 	const char* layout_name = dynv_get_string_wd(args->params, "layout_name", "std_layout_menu_1");
 
@@ -732,10 +755,12 @@ static ColorSource* source_implement(ColorSource *source, GlobalState* gs, struc
 }
 
 int layout_preview_source_register(ColorSourceManager *csm){
-    ColorSource *color_source = new ColorSource;
-	color_source_init(color_source, "layout_preview", "Layout preview");
+	ColorSource *color_source = new ColorSource;
+	color_source_init(color_source, "layout_preview", _("Layout preview"));
+	color_source->needs_viewport = false;
 	color_source->implement = (ColorSource* (*)(ColorSource *source, GlobalState *gs, struct dynvSystem *dynv_namespace))source_implement;
-    color_source_manager_add_source(csm, color_source);
+	color_source->default_accelerator = GDK_l;
+	color_source_manager_add_source(csm, color_source);
 	return 0;
 }
 
