@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2011, Albertas Vyšniauskas
+ * Copyright (c) 2009-2012, Albertas Vyšniauskas
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -20,6 +20,7 @@
 #include "DragDrop.h"
 
 #include "GlobalStateStruct.h"
+#include "ToolColorNaming.h"
 #include "uiUtilities.h"
 #include "ColorList.h"
 #include "MathUtil.h"
@@ -31,10 +32,12 @@
 #include "CopyPaste.h"
 #include "Converter.h"
 #include "DynvHelpers.h"
+#include "Internationalisation.h"
 
 #include "uiApp.h"
 
 #include <gdk/gdkkeysyms.h>
+#include <boost/format.hpp>
 
 #include <math.h>
 #include <string.h>
@@ -50,6 +53,7 @@ using namespace std;
 #define COMPONENT_ID_HSL_LIGHTNESS	3
 #define COMPONENT_ID_LAB_LIGHTNESS	4
 
+
 typedef struct VariationType{
 	const char *name;
 	const char *symbol;
@@ -59,10 +63,10 @@ typedef struct VariationType{
 }VariationType;
 
 const VariationType variation_types[] = {
-	{"Hue", "H<span font='8' rise='8000'>HSL</span>", "hsl_hue", COMPONENT_ID_HSL_HUE, 1},
-	{"Saturation", "S<span font='8' rise='8000'>HSL</span>", "hsl_saturation", COMPONENT_ID_HSL_SATURATION, 1},
-	{"Lightness", "L<span font='8' rise='8000'>HSL</span>", "hsl_lightness", COMPONENT_ID_HSL_LIGHTNESS, 1},
-	{"Lightness (Lab)", "L<span font='8' rise='8000'>Lab</span>", "lab_lightness", COMPONENT_ID_LAB_LIGHTNESS, 1},
+	{N_("Hue"), "H<span font='8' rise='8000'>HSL</span>", "hsl_hue", COMPONENT_ID_HSL_HUE, 1},
+	{N_("Saturation"), "S<span font='8' rise='8000'>HSL</span>", "hsl_saturation", COMPONENT_ID_HSL_SATURATION, 1},
+	{N_("Lightness"), "L<span font='8' rise='8000'>HSL</span>", "hsl_lightness", COMPONENT_ID_HSL_LIGHTNESS, 1},
+	{N_("Lightness (Lab)"), "L<span font='8' rise='8000'>Lab</span>", "lab_lightness", COMPONENT_ID_LAB_LIGHTNESS, 1},
 };
 
 typedef struct VariationsArgs{
@@ -76,7 +80,7 @@ typedef struct VariationsArgs{
 	GtkWidget *color_previews;
 
 	GtkWidget *all_colors;
-    struct{
+	struct{
 		GtkWidget *color;
 		GtkWidget *var_colors[VAR_COLOR_WIDGETS + 1];
 		const VariationType *type;
@@ -88,6 +92,32 @@ typedef struct VariationsArgs{
 	GlobalState* gs;
 }VariationsArgs;
 
+class VariationsColorNameAssigner: public ToolColorNameAssigner {
+	protected:
+		stringstream m_stream;
+		const char *m_ident;
+	public:
+		VariationsColorNameAssigner(GlobalState *gs):ToolColorNameAssigner(gs){
+		}
+
+		void assign(struct ColorObject *color_object, Color *color, const char *ident){
+			m_ident = ident;
+			ToolColorNameAssigner::assign(color_object, color);
+		}
+
+		virtual std::string getToolSpecificName(struct ColorObject *color_object, Color *color){
+			m_stream.str("");
+			m_stream << color_names_get(m_gs->color_names, color, false) << " " << _("variations") << " " << m_ident;
+			return m_stream.str();
+		}
+};
+
+static boost::format format_ignore_arg_errors(const std::string &f_string) {
+	boost::format fmter(f_string);
+	fmter.exceptions(boost::io::all_error_bits ^ (boost::io::too_many_args_bit | boost::io::too_few_args_bit));
+	return fmter;
+}
+
 
 static int set_rgb_color(VariationsArgs *args, struct ColorObject* color, uint32_t color_index);
 static int set_rgb_color_by_widget(VariationsArgs *args, struct ColorObject* color, GtkWidget* color_widget);
@@ -101,10 +131,6 @@ static void calc(VariationsArgs *args, bool preview, bool save_settings){
 	}
 
 	Color color, hsl, lab, r, hsl_mod, lab_mod;
-	matrix3x3 adaptation_matrix, working_space_matrix, working_space_matrix_inverted;
-	vector3 d50, d65;
-	vector3_set(&d50, 96.442, 100.000,  82.821);
-	vector3_set(&d65, 95.047, 100.000, 108.883);
 
 	for (int i = 0; i < MAX_COLOR_LINES; ++i){
 		gtk_color_get_color(GTK_COLOR(args->color[i].color), &color);
@@ -117,10 +143,7 @@ static void calc(VariationsArgs *args, bool preview, bool save_settings){
 			break;
 		case COMPONENT_ID_LAB_LIGHTNESS:
 			{
-				color_get_chromatic_adaptation_matrix(&d50, &d65, &adaptation_matrix);
-				color_get_working_space_matrix(0.6400, 0.3300, 0.3000, 0.6000, 0.1500, 0.0600, &d65, &working_space_matrix);
-				matrix3x3_inverse(&working_space_matrix, &working_space_matrix_inverted);
-				color_rgb_to_lab(&color, &lab, &d50, &working_space_matrix);
+				color_rgb_to_lab_d50(&color, &lab);
 			}
 			break;
 		}
@@ -147,7 +170,7 @@ static void calc(VariationsArgs *args, bool preview, bool save_settings){
 			case COMPONENT_ID_LAB_LIGHTNESS:
 				color_copy(&lab, &lab_mod);
 				lab_mod.lab.L = clamp_float(lab.lab.L + (args->color[i].type->strength_mult * strength * (j - VAR_COLOR_WIDGETS / 2)) / 4.0, 0, 100);
-				color_lab_to_rgb(&lab_mod, &r, &d50, &working_space_matrix_inverted);
+				color_lab_to_rgb_d50(&lab_mod, &r);
 				color_rgb_normalize(&r);
 				break;
 			}
@@ -196,37 +219,62 @@ static void on_color_edit(GtkWidget *widget,  gpointer item) {
 	color_object_release(color_object);
 }
 
+static string identify_color_widget(GtkWidget *widget, VariationsArgs *args)
+{
+	if (args->all_colors == widget){
+		return _("all colors");
+	}else for (int i = 0; i < MAX_COLOR_LINES; ++i){
+		if (args->color[i].color == widget){
+			try{
+				return (format_ignore_arg_errors(_("primary %d")) % (i + 1)).str();
+			}catch(const boost::io::format_error &e){
+				return (format_ignore_arg_errors("primary %d") % (i + 1)).str();
+			}
+		}
+		for (int j = 0; j <= VAR_COLOR_WIDGETS; ++j){
+			if (args->color[i].var_colors[j] == widget){
+				if (j > VAR_COLOR_WIDGETS / 2)
+					j--;
+				try{
+					return (format_ignore_arg_errors(_("result %d line %d")) % (j + 1) % (i + 1)).str();
+				}catch(const boost::io::format_error &e){
+					return (format_ignore_arg_errors("result %d line %d") % (j + 1) % (i + 1)).str();
+				}
+			}
+		}
+	}
+	return "unknown";
+}
 
-static void on_color_add_to_palette(GtkWidget *widget,  gpointer item) {
-	VariationsArgs* args=(VariationsArgs*)item;
+static void add_color_to_palette(GtkWidget *color_widget, VariationsColorNameAssigner &name_assigner, VariationsArgs *args)
+{
 	Color c;
-
-	gtk_color_get_color(GTK_COLOR(g_object_get_data(G_OBJECT(widget), "color_widget")), &c);
-
-	struct ColorObject *color_object=color_list_new_color_object(args->gs->colors, &c);
-	string name = color_names_get(args->gs->color_names, &c, dynv_get_bool_wd(args->gs->params, "gpick.color_names.imprecision_postfix", true));
-	dynv_set_string(color_object->params, "name", name.c_str());
+	struct ColorObject *color_object;
+	string widget_ident;
+	gtk_color_get_color(GTK_COLOR(color_widget), &c);
+	color_object = color_list_new_color_object(args->gs->colors, &c);
+	widget_ident = identify_color_widget(color_widget, args);
+	name_assigner.assign(color_object, &c, widget_ident.c_str());
 	color_list_add_color_object(args->gs->colors, color_object, 1);
 	color_object_release(color_object);
 }
 
+static void on_color_add_to_palette(GtkWidget *widget,  gpointer item) {
+	VariationsArgs* args = (VariationsArgs*)item;
+    GtkWidget *color_widget = GTK_WIDGET(g_object_get_data(G_OBJECT(widget), "color_widget"));
+	VariationsColorNameAssigner name_assigner(args->gs);
+	add_color_to_palette(color_widget, name_assigner, args);
+}
+
 static void on_color_add_all_to_palette(GtkWidget *widget,  gpointer item) {
-	VariationsArgs* args=(VariationsArgs*)item;
-	Color c;
+	VariationsArgs* args = (VariationsArgs*)item;
+	VariationsColorNameAssigner name_assigner(args->gs);
 
 	for (int i = 0; i < MAX_COLOR_LINES; ++i){
 		for (int j = 0; j < VAR_COLOR_WIDGETS + 1; ++j){
-
-			gtk_color_get_color(GTK_COLOR(args->color[i].var_colors[j]), &c);
-
-			struct ColorObject *color_object = color_list_new_color_object(args->gs->colors, &c);
-			string name=color_names_get(args->gs->color_names, &c, dynv_get_bool_wd(args->gs->params, "gpick.color_names.imprecision_postfix", true));
-			dynv_set_string(color_object->params, "name", name.c_str());
-			color_list_add_color_object(args->gs->colors, color_object, 1);
-			color_object_release(color_object);
+			add_color_to_palette(args->color[i].var_colors[j], name_assigner, args);
 		}
 	}
-
 }
 
 static gboolean color_focus_in_cb(GtkWidget *widget, GdkEventFocus *event, VariationsArgs *args){
@@ -280,19 +328,19 @@ static void color_show_menu(GtkWidget* widget, VariationsArgs* args, GdkEventBut
 
 	menu = gtk_menu_new ();
 
-	item = gtk_menu_item_new_with_image ("_Add to palette", gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
+	item = gtk_menu_item_new_with_image(_("_Add to palette"), gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (on_color_add_to_palette), args);
 	g_object_set_data(G_OBJECT(item), "color_widget", widget);
 
-	item = gtk_menu_item_new_with_image ("_Add all to palette", gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
+	item = gtk_menu_item_new_with_image(_("A_dd all to palette"), gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (on_color_add_all_to_palette), args);
 	g_object_set_data(G_OBJECT(item), "color_widget", widget);
 
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), gtk_separator_menu_item_new ());
 
-	item = gtk_menu_item_new_with_mnemonic ("_Copy to clipboard");
+	item = gtk_menu_item_new_with_mnemonic(_("_Copy to clipboard"));
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
 	Color c;
@@ -319,7 +367,7 @@ static void color_show_menu(GtkWidget* widget, VariationsArgs* args, GdkEventBut
 		if (!all_colors){
 			GSList *group = 0;
 			for (uint32_t i = 0; i < sizeof(variation_types) / sizeof(VariationType); i++){
-				item = gtk_radio_menu_item_new_with_label(group, variation_types[i].name);
+				item = gtk_radio_menu_item_new_with_label(group, _(variation_types[i].name));
 				group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(item));
 				if (args->color[line_id].type == &variation_types[i]){
 					gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), true);
@@ -332,12 +380,12 @@ static void color_show_menu(GtkWidget* widget, VariationsArgs* args, GdkEventBut
 			gtk_menu_shell_append (GTK_MENU_SHELL (menu), gtk_separator_menu_item_new ());
 		}
 
-		item = gtk_menu_item_new_with_image ("_Edit...", gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU));
+		item = gtk_menu_item_new_with_image (_("_Edit..."), gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 		g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (on_color_edit), args);
 		g_object_set_data(G_OBJECT(item), "color_widget", widget);
 
-		item = gtk_menu_item_new_with_image ("_Paste", gtk_image_new_from_stock(GTK_STOCK_PASTE, GTK_ICON_SIZE_MENU));
+		item = gtk_menu_item_new_with_image (_("_Paste"), gtk_image_new_from_stock(GTK_STOCK_PASTE, GTK_ICON_SIZE_MENU));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 		g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (on_color_paste), args);
 		g_object_set_data(G_OBJECT(item), "color_widget", widget);
@@ -430,7 +478,7 @@ static int source_destroy(VariationsArgs *args){
 	char tmp[32];
 	for (gint i = 0; i < MAX_COLOR_LINES; ++i){
 		sprintf(tmp, "type%d", i);
-        dynv_set_string(args->params, tmp, args->color[i].type->unique_name);
+		dynv_set_string(args->params, tmp, args->color[i].type->unique_name);
 
 		sprintf(tmp, "color%d", i);
 		gtk_color_get_color(GTK_COLOR(args->color[i].color), &c);
@@ -448,16 +496,18 @@ static int source_destroy(VariationsArgs *args){
 }
 
 static int source_get_color(VariationsArgs *args, ColorObject** color){
+	VariationsColorNameAssigner name_assigner(args->gs);
 	Color c;
+	string widget_ident;
 	if (args->last_focused_color){
 		gtk_color_get_color(GTK_COLOR(args->last_focused_color), &c);
+		widget_ident = identify_color_widget(args->last_focused_color, args);
 	}else{
 		gtk_color_get_color(GTK_COLOR(args->color[0].color), &c);
+		widget_ident = identify_color_widget(args->color[0].color, args);
 	}
 	*color = color_list_new_color_object(args->gs->colors, &c);
-
-	string name = color_names_get(args->gs->color_names, &c, dynv_get_bool_wd(args->gs->params, "gpick.color_names.imprecision_postfix", true));
-	dynv_set_string((*color)->params, "name", name.c_str());
+	name_assigner.assign(*color, &c, widget_ident.c_str());
 	return 0;
 }
 
@@ -478,7 +528,7 @@ static int set_rgb_color_by_widget(VariationsArgs *args, struct ColorObject* col
 static int set_rgb_color(VariationsArgs *args, struct ColorObject* color, uint32_t color_index){
 	Color c;
 	color_object_get_color(color, &c);
-	if (color_index == -1){
+	if (color_index == (uint32_t)-1){
 		gtk_color_set_color(GTK_COLOR(args->all_colors), &c, "");
 		for (int i = 0; i < MAX_COLOR_LINES; ++i){
 			gtk_color_set_color(GTK_COLOR(args->color[i].color), &c, args->color[i].type->symbol);
@@ -500,6 +550,17 @@ static int source_set_color(VariationsArgs *args, struct ColorObject* color){
 }
 
 static int source_activate(VariationsArgs *args){
+
+	transformation::Chain *chain = static_cast<transformation::Chain*>(dynv_get_pointer_wdc(args->gs->params, "TransformationChain", 0));
+	gtk_color_set_transformation_chain(GTK_COLOR(args->all_colors), chain);
+
+	for (int i = 0; i < MAX_COLOR_LINES; ++i){
+		gtk_color_set_transformation_chain(GTK_COLOR(args->color[i].color), chain);
+		for (int j = 0; j < VAR_COLOR_WIDGETS + 1; ++j){
+			gtk_color_set_transformation_chain(GTK_COLOR(args->color[i].var_colors[j]), chain);
+		}
+	}
+
 	gtk_statusbar_push(GTK_STATUSBAR(args->statusbar), gtk_statusbar_get_context_id(GTK_STATUSBAR(args->statusbar), "empty"), "");
 	return 0;
 }
@@ -582,8 +643,8 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 	dragdrop_widget_attach(widget, DragDropFlags(DRAGDROP_SOURCE | DRAGDROP_DESTINATION), &dd);
 
 
-	for (int i = 0; i < MAX_COLOR_LINES; ++i){
-		for (int j = 0; j < VAR_COLOR_WIDGETS + 1; ++j){
+	for (intptr_t i = 0; i < MAX_COLOR_LINES; ++i){
+		for (intptr_t j = 0; j < VAR_COLOR_WIDGETS + 1; ++j){
 
 			widget = gtk_color_new();
 			gtk_color_set_rounded(GTK_COLOR(widget), true);
@@ -653,7 +714,7 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 	gtk_box_pack_start(GTK_BOX(hbox2), table, true, true, 0);
 	table_y = 0;
 
-	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new("Strength:",0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,0,0);
+	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new(_("Strength:"),0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,0,0);
 	args->strength = gtk_hscale_new_with_range(1, 100, 1);
 	gtk_range_set_value(GTK_RANGE(args->strength), dynv_get_float_wd(args->params, "strength", 30));
 	g_signal_connect(G_OBJECT(args->strength), "value-changed", G_CALLBACK (update), args);
@@ -680,10 +741,11 @@ static ColorSource* source_implement(ColorSource *source, GlobalState *gs, struc
 }
 
 int variations_source_register(ColorSourceManager *csm){
-    ColorSource *color_source = new ColorSource;
-	color_source_init(color_source, "variations", "Variations");
+	ColorSource *color_source = new ColorSource;
+	color_source_init(color_source, "variations", _("Variations"));
 	color_source->implement = (ColorSource* (*)(ColorSource *source, GlobalState *gs, struct dynvSystem *dynv_namespace))source_implement;
-    color_source_manager_add_source(csm, color_source);
+	color_source->default_accelerator = GDK_v;
+	color_source_manager_add_source(csm, color_source);
 	return 0;
 }
 
