@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2010, Albertas Vyšniauskas
+ * Copyright (c) 2009-2012, Albertas Vyšniauskas
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -21,6 +21,9 @@
 #include "../ColorWheelType.h"
 #include "../MathUtil.h"
 #include <math.h>
+#ifdef _MSC_VER
+#define M_PI 3.14159265359
+#endif
 #include <stdlib.h>
 
 #include <list>
@@ -328,7 +331,7 @@ static void draw_wheel(GtkColorWheelPrivate *ns, cairo_t *cr, double radius, dou
 
 				if ((dist >= inner_radius_sq) && (dist <= radius_sq)){
 
-					double angle = atan2(dx, dy) + M_PI;
+					double angle = atan2((double)dx, (double)dy) + M_PI;
 
 					wheel->hue_to_hsl(angle / (M_PI * 2), &c);
 					color_hsl_to_rgb(&c, &c);
