@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2011, Albertas Vyšniauskas
+ * Copyright (c) 2009-2012, Albertas Vyšniauskas
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -23,6 +23,7 @@
 #include <gtk/gtk.h>
 #include "../ColorObject.h"
 #include "../layout/System.h"
+#include "../transformation/Chain.h"
 
 G_BEGIN_DECLS
 
@@ -71,6 +72,8 @@ bool gtk_layout_preview_is_editable(GtkLayoutPreview* widget);
 
 int gtk_layout_preview_get_current_style(GtkLayoutPreview* widget, layout::Style** style);
 
+void gtk_layout_preview_set_transformation_chain(GtkLayoutPreview* widget, transformation::Chain *chain);
+
 G_END_DECLS
 
 
