@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2010, Albertas Vyšniauskas
+ * Copyright (c) 2009-2012, Albertas Vyšniauskas
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -51,17 +51,23 @@ typedef struct GtkZoomedClass
 {
 	GtkDrawingAreaClass parent_class;
 	void  (* color_changed)(GtkWidget* widget, Color* c, gpointer userdata);
+	void  (*activated)(GtkWidget* widget, gpointer userdata);
 }GtkZoomedClass;
 
 
 GtkWidget* gtk_zoomed_new ();
 
-void gtk_zoomed_set_zoom (GtkZoomed* zoomed, gfloat zoom);
-gfloat gtk_zoomed_get_zoom (GtkZoomed* zoomed);
+void gtk_zoomed_set_zoom(GtkZoomed* zoomed, gfloat zoom);
+gfloat gtk_zoomed_get_zoom(GtkZoomed* zoomed);
+
+void gtk_zoomed_set_fade(GtkZoomed* zoomed, bool fade);
 
 int32_t gtk_zoomed_get_size(GtkZoomed *zoomed);
 void gtk_zoomed_set_size(GtkZoomed *zoomed, int32_t width_height);
 
+void gtk_zoomed_set_mark(GtkZoomed *zoomed, int index, math::Vec2<int>& position);
+void gtk_zoomed_clear_mark(GtkZoomed *zoomed, int index);
+
 void gtk_zoomed_update(GtkZoomed* zoomed, math::Vec2<int>& pointer, math::Vec2<int>& screen_size, math::Vec2<int>& offset, GdkPixbuf* pixbuf);
 void gtk_zoomed_get_screen_rect(GtkZoomed* zoomed, math::Vec2<int>& pointer, math::Vec2<int>& screen_size, math::Rect2<int> *rect);
 
