@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2011, Albertas Vyšniauskas
+ * Copyright (c) 2009-2012, Albertas Vyšniauskas
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -20,7 +20,9 @@
 #include "../uiUtilities.h"
 #include "../uiListPalette.h"
 #include "../GlobalStateStruct.h"
+#include "../ToolColorNaming.h"
 #include "../DynvHelpers.h"
+#include "../Internationalisation.h"
 
 #include <string.h>
 #include <iostream>
@@ -69,8 +71,8 @@ typedef struct PaletteFromImageArgs{
 	string filename;
 	uint32_t n_colors;
 
-    string previuos_filename;
-    Node *previuos_node;
+	string previous_filename;
+	Node *previous_node;
 
 	struct ColorList *color_list;
 	struct ColorList *preview_color_list;
@@ -79,6 +81,29 @@ typedef struct PaletteFromImageArgs{
 	GlobalState* gs;
 }PaletteFromImageArgs;
 
+class PaletteColorNameAssigner: public ToolColorNameAssigner {
+	protected:
+		stringstream m_stream;
+		const char *m_filename;
+		int m_index;
+	public:
+		PaletteColorNameAssigner(GlobalState *gs):ToolColorNameAssigner(gs){
+			m_index = 0;
+		}
+
+		void assign(struct ColorObject *color_object, Color *color, const char *filename, const int index){
+			m_filename = filename;
+			m_index = index;
+			ToolColorNameAssigner::assign(color_object, color);
+		}
+
+		virtual std::string getToolSpecificName(struct ColorObject *color_object, Color *color){
+			m_stream.str("");
+			m_stream <<  m_filename << " #" << m_index;
+			return m_stream.str();
+		}
+};
+
 /**
  * Allocate and initialize a new node with specified parent
  * @param[in] parent Parent node
@@ -225,19 +250,18 @@ static bool node_prune_threshold(Node *node, PruneData *prune_data){
 }
 
 static void node_reduce(Node *node, uint32_t colors){
-    PruneData prune_data;
-    prune_data.n_colors = node_count_leafs(node);
-    prune_data.n_colors_target = colors;
-    prune_data.threshold = 0;
+	PruneData prune_data;
+	prune_data.n_colors = node_count_leafs(node);
+	prune_data.n_colors_target = colors;
+	prune_data.threshold = 0;
 
-    while (prune_data.n_colors > colors){
-	    prune_data.min_distance = node->distance;
+	while (prune_data.n_colors > colors){
+		prune_data.min_distance = node->distance;
 
-	    if (node_prune_threshold(node, &prune_data)) break;
+		if (node_prune_threshold(node, &prune_data)) break;
 
-	    prune_data.threshold = prune_data.min_distance;
+		prune_data.threshold = prune_data.min_distance;
 	}
-
 }
 
 static void node_update(Node *node, Color *color, Cube *cube, uint32_t max_depth){
@@ -306,17 +330,17 @@ static void leaf_cb(Node *node, void *userdata){
 
 static Node* process_image(PaletteFromImageArgs *args, const char *filename, Node* node){
 
-	if (args->previuos_filename == filename){
-		if (args->previuos_node)
-			return node_copy(args->previuos_node, 0);
+	if (args->previous_filename == filename){
+		if (args->previous_node)
+			return node_copy(args->previous_node, 0);
 		else
 			return 0;
 	}
 
-	args->previuos_filename = filename;
-	if (args->previuos_node){
-		node_delete(args->previuos_node);
-		args->previuos_node = 0;
+	args->previous_filename = filename;
+	if (args->previous_node){
+		node_delete(args->previous_node);
+		args->previous_node = 0;
 	}
 
 	GError *error = NULL;
@@ -344,7 +368,7 @@ static Node* process_image(PaletteFromImageArgs *args, const char *filename, Nod
 
 	Color color;
 
-	args->previuos_node = node_new(0);
+	args->previous_node = node_new(0);
 
 	for (int y = 0; y < height; y++){
 		ptr = image_data + rowstride * y;
@@ -354,7 +378,7 @@ static Node* process_image(PaletteFromImageArgs *args, const char *filename, Nod
 			color.xyz.y = ptr[1] / 255.0;
 			color.xyz.z = ptr[2] / 255.0;
 
-			node_update(args->previuos_node, &color, &cube, 5);
+			node_update(args->previous_node, &color, &cube, 5);
 
 			ptr += channels;
 		}
@@ -362,9 +386,9 @@ static Node* process_image(PaletteFromImageArgs *args, const char *filename, Nod
 
 	g_object_unref(pixbuf);
 
-	node_reduce(args->previuos_node, 200);
+	node_reduce(args->previous_node, 200);
 
-	return node_copy(args->previuos_node, 0);
+	return node_copy(args->previous_node, 0);
 }
 
 
@@ -398,7 +422,10 @@ static void save_settings(PaletteFromImageArgs *args){
 static void calc(PaletteFromImageArgs *args, bool preview, int limit){
 
 	Node *root_node = 0;
-    if (!args->filename.empty())
+	int index = 0;
+	gchar *name = g_path_get_basename(args->filename.c_str());
+	PaletteColorNameAssigner name_assigner(args->gs);
+	if (!args->filename.empty())
 		root_node = process_image(args, args->filename.c_str(), root_node);
 
 	struct ColorList *color_list;
@@ -418,10 +445,10 @@ static void calc(PaletteFromImageArgs *args, bool preview, int limit){
 
 	for (list<Color>::iterator i = tmp_list.begin(); i != tmp_list.end(); i++){
 		struct ColorObject *color_object = color_list_new_color_object(color_list, &(*i));
-		string name = color_names_get(args->gs->color_names, &(*i), dynv_get_bool_wd(args->gs->params, "gpick.color_names.imprecision_postfix", true));
-		dynv_set_string(color_object->params, "name", name.c_str());
+		name_assigner.assign(color_object, &(*i), name, index);
 		color_list_add_color_object(color_list, color_object, 1);
 		color_object_release(color_object);
+		index++;
 	}
 
 }
@@ -440,7 +467,7 @@ static gchar* format_threshold_value_cb(GtkScale *scale, gdouble value){
 
 static void destroy_cb(GtkWidget* widget, PaletteFromImageArgs *args){
 
-	if (args->previuos_node) node_delete(args->previuos_node);
+	if (args->previous_node) node_delete(args->previous_node);
 
 	color_list_destroy(args->preview_color_list);
 	dynv_system_release(args->params);
@@ -477,14 +504,14 @@ void tools_palette_from_image_show(GtkWindow* parent, GlobalState* gs){
 
 	PaletteFromImageArgs *args = new PaletteFromImageArgs;
 
-    args->previuos_filename = "";
+    args->previous_filename = "";
 	args->gs = gs;
 	args->params = dynv_get_dynv(args->gs->params, "gpick.tools.palette_from_image");
-	args->previuos_node = 0;
+	args->previous_node = 0;
 
 	GtkWidget *table, *table_m, *widget;
 
-	GtkWidget *dialog = gtk_dialog_new_with_buttons("Palette from image", parent, GtkDialogFlags(GTK_DIALOG_DESTROY_WITH_PARENT), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, GTK_STOCK_ADD, GTK_RESPONSE_APPLY, NULL);
+	GtkWidget *dialog = gtk_dialog_new_with_buttons(_("Palette from image"), parent, GtkDialogFlags(GTK_DIALOG_DESTROY_WITH_PARENT), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, GTK_STOCK_ADD, GTK_RESPONSE_APPLY, NULL);
 
 	gtk_window_set_default_size(GTK_WINDOW(dialog), dynv_get_int32_wd(args->params, "window.width", -1),
 		dynv_get_int32_wd(args->params, "window.height", -1));
@@ -497,7 +524,7 @@ void tools_palette_from_image_show(GtkWindow* parent, GlobalState* gs){
 
 	table_m = gtk_table_new(3, 1, FALSE);
 	table_m_y = 0;
-	frame = gtk_frame_new("Image");
+	frame = gtk_frame_new(_("Image"));
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
 	gtk_table_attach(GTK_TABLE(table_m), frame, 0, 1, table_m_y, table_m_y+1, GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL), 5, 5);
 	table_m_y++;
@@ -506,7 +533,7 @@ void tools_palette_from_image_show(GtkWindow* parent, GlobalState* gs){
 	table_y=0;
 	gtk_container_add(GTK_CONTAINER(frame), table);
 
-	args->file_browser = widget = gtk_file_chooser_button_new("Image file", GTK_FILE_CHOOSER_ACTION_OPEN);
+	args->file_browser = widget = gtk_file_chooser_button_new(_("Image file"), GTK_FILE_CHOOSER_ACTION_OPEN);
 	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(widget), dynv_get_string_wd(args->params, "current_folder", ""));
 	gtk_table_attach(GTK_TABLE(table), widget, 0, 3, table_y, table_y+1, GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,3,3);
 	g_signal_connect(G_OBJECT(args->file_browser), "file-set", G_CALLBACK(update), args);
@@ -518,7 +545,7 @@ void tools_palette_from_image_show(GtkWindow* parent, GlobalState* gs){
 	GtkFileFilter *all_image_filter;
 
 	filter = gtk_file_filter_new();
-	gtk_file_filter_set_name(filter, "All files");
+	gtk_file_filter_set_name(filter, _("All files"));
 	gtk_file_filter_add_pattern(filter, "*");
 	g_object_set_data_full(G_OBJECT(filter), "name", (void*)"all_files", GDestroyNotify(NULL));
 	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(widget), filter);
@@ -526,7 +553,7 @@ void tools_palette_from_image_show(GtkWindow* parent, GlobalState* gs){
 
 
 	all_image_filter = gtk_file_filter_new();
-	gtk_file_filter_set_name(all_image_filter, "All images");
+	gtk_file_filter_set_name(all_image_filter, _("All images"));
 	g_object_set_data_full(G_OBJECT(all_image_filter), "name", (void*)"all_images", GDestroyNotify(NULL));
 	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(widget), all_image_filter);
 	if (g_strcmp0("all_images", selected_filter) == 0) gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(widget), all_image_filter);
@@ -559,7 +586,7 @@ void tools_palette_from_image_show(GtkWindow* parent, GlobalState* gs){
 	if (formats) g_slist_free(formats);
 
 
-	frame = gtk_frame_new("Options");
+	frame = gtk_frame_new(_("Options"));
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
 	gtk_table_attach(GTK_TABLE(table_m), frame, 0, 1, table_m_y, table_m_y+1, GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL), 5, 5);
 	table_m_y++;
@@ -569,7 +596,7 @@ void tools_palette_from_image_show(GtkWindow* parent, GlobalState* gs){
 
 
 
-	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new("Colors:",0,0,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
+	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new(_("Colors:"),0,0,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
 	args->range_colors = widget = gtk_spin_button_new_with_range (1, 100, 1);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(args->range_colors), dynv_get_int32_wd(args->params, "colors", 3));
 	gtk_table_attach(GTK_TABLE(table), widget,1,3,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,3,3);
@@ -578,7 +605,7 @@ void tools_palette_from_image_show(GtkWindow* parent, GlobalState* gs){
 
 
     /*
-	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new("Threshold:",0,0,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
+	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new(_("Threshold:"),0,0,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
 	args->merge_threshold = widget = gtk_hscale_new_with_range(0, 100, 0.1);
 	gtk_range_set_value(GTK_RANGE(widget), dynv_get_float_wd(args->params, "merge_threshold", 20));
 	g_signal_connect(G_OBJECT(widget), "value-changed", G_CALLBACK(update), args);
@@ -589,7 +616,7 @@ void tools_palette_from_image_show(GtkWindow* parent, GlobalState* gs){
     */
 
 	struct ColorList* preview_color_list = NULL;
-	gtk_table_attach(GTK_TABLE(table_m), args->preview_expander = palette_list_preview_new(gs, dynv_get_bool_wd(args->params, "show_preview", true), gs->colors, &preview_color_list), 0, 1, table_m_y, table_m_y+1 , GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL | GTK_EXPAND), 5, 5);
+	gtk_table_attach(GTK_TABLE(table_m), args->preview_expander = palette_list_preview_new(gs, true, dynv_get_bool_wd(args->params, "show_preview", true), gs->colors, &preview_color_list), 0, 1, table_m_y, table_m_y+1 , GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL | GTK_EXPAND), 5, 5);
 	table_m_y++;
 
 	args->preview_color_list = preview_color_list;
