@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2011, Albertas Vyšniauskas
+ * Copyright (c) 2009-2012, Albertas Vyšniauskas
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -22,14 +22,19 @@
 #include "MathUtil.h"
 #include "DynvHelpers.h"
 #include "GlobalStateStruct.h"
+#include "ToolColorNaming.h"
+#include "Internationalisation.h"
 
+#ifndef _MSC_VER
 #include <stdbool.h>
+#endif
 #include <sstream>
 using namespace std;
 
 typedef struct DialogMixArgs{
 	GtkWidget *mix_type;
 	GtkWidget *mix_steps;
+	GtkWidget *toggle_endpoints;
 
 	struct ColorList *selected_color_list;
 	struct ColorList *preview_color_list;
@@ -38,14 +43,77 @@ typedef struct DialogMixArgs{
 	GlobalState* gs;
 }DialogMixArgs;
 
+class MixColorNameAssigner: public ToolColorNameAssigner {
+	protected:
+		stringstream m_stream;
+		const char *m_color_start;
+		const char *m_color_end;
+		int m_start_percent;
+		int m_end_percent;
+                int m_is_node;
+	public:
+		MixColorNameAssigner(GlobalState *gs):ToolColorNameAssigner(gs){
+		}
+
+		void assign(struct ColorObject *color_object, Color *color, const char *start_color_name, const char *end_color_name, int start_percent, int end_percent, bool is_node){
+			m_color_start = start_color_name;
+			m_color_end = end_color_name;
+			m_start_percent = start_percent;
+			m_end_percent = end_percent;
+			m_is_node = is_node;
+			ToolColorNameAssigner::assign(color_object, color);
+		}
+
+		void assign(struct ColorObject *color_object, Color *color, const char *item_name){
+			m_color_start = item_name;
+			ToolColorNameAssigner::assign(color_object, color);
+		}
+
+		virtual std::string getToolSpecificName(struct ColorObject *color_object, Color *color){
+			m_stream.str("");
+			if (m_is_node){
+				if (m_end_percent == 100){
+					m_stream << m_color_end << " " << _("mix node");
+				}else{
+					m_stream << m_color_start << " " << _("mix node");
+				}
+			}else{
+				m_stream << m_color_start << " " << m_start_percent << " " << _("mix") << " " << m_end_percent << " " << m_color_end;
+			}
+			return m_stream.str();
+		}
+};
+
+
+#define STORE_COLOR() struct ColorObject *color_object=color_list_new_color_object(color_list, &r); \
+    float mixfactor = step_i/(float)(steps-1); \
+    name_assigner.assign(color_object, &r, name_a, name_b, (int)((1.0 - mixfactor)*100), (int)(mixfactor*100), with_endpoints && (step_i == 0 || step_i == (max_step - 1))); \
+	color_list_add_color_object(color_list, color_object, 1); \
+	color_object_release(color_object)
+
+#define STORE_LINEARCOLOR() color_linear_get_rgb(&r, &r); \
+	STORE_COLOR()
+
+
+
 static void calc( DialogMixArgs *args, bool preview, int limit){
 
 	gint steps=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(args->mix_steps));
 	gint type=gtk_combo_box_get_active(GTK_COMBO_BOX(args->mix_type));
+	bool with_endpoints=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(args->toggle_endpoints));
+	gint start_step = 0;
+	gint max_step = steps;
+	MixColorNameAssigner name_assigner(args->gs);
 
 	if (!preview){
 		dynv_set_int32(args->params, "type", type);
 		dynv_set_int32(args->params, "steps", steps);
+		dynv_set_bool(args->params, "includeendpoints", with_endpoints);
+	}
+
+	if (with_endpoints == false){
+		start_step = 1;
+		max_step = steps - 1;
 	}
 
 	Color r;
@@ -56,13 +124,6 @@ static void calc( DialogMixArgs *args, bool preview, int limit){
 	s.setf(ios::fixed,ios::floatfield);
 
 	Color a,b;
-	matrix3x3 adaptation_matrix, working_space_matrix, working_space_matrix_inverted;
-	vector3 d50, d65;
-	vector3_set(&d50, 96.442, 100.000,  82.821);
-	vector3_set(&d65, 95.047, 100.000, 108.883);
-	color_get_chromatic_adaptation_matrix(&d50, &d65, &adaptation_matrix);
-	color_get_working_space_matrix(0.6400, 0.3300, 0.3000, 0.6000, 0.1500, 0.0600, &d65, &working_space_matrix);
-    matrix3x3_inverse(&working_space_matrix, &working_space_matrix_inverted);
 
 	struct ColorList *color_list;
 	if (preview)
@@ -74,6 +135,9 @@ static void calc( DialogMixArgs *args, bool preview, int limit){
 	for (ColorList::iter i=args->selected_color_list->colors.begin(); i!=args->selected_color_list->colors.end(); ++i){
 
 		color_object_get_color(*i, &a);
+		if (type == 0)
+			color_rgb_get_linear(&a, &a);
+
 		const char* name_a = dynv_get_string_wd((*i)->params, "name", 0);
 		j=i;
 		++j;
@@ -85,22 +149,15 @@ static void calc( DialogMixArgs *args, bool preview, int limit){
 			}
 
 			color_object_get_color(*j, &b);
+			if (type == 0)
+				color_rgb_get_linear(&b, &b);
 			const char* name_b = dynv_get_string_wd((*j)->params, "name", 0);
 
 			switch (type) {
 			case 0:
-				for (step_i = 0; step_i < steps; ++step_i) {
-					r.rgb.red = mix_float(a.rgb.red, b.rgb.red, step_i/(float)(steps-1));
-					r.rgb.green = mix_float(a.rgb.green, b.rgb.green, step_i/(float)(steps-1));
-					r.rgb.blue = mix_float(a.rgb.blue, b.rgb.blue, step_i/(float)(steps-1));
-
-					s.str("");
-					s<<name_a<<" "<<(step_i/float(steps-1))*100<< " mix " <<100-(step_i/float(steps-1))*100<<" "<< name_b;
-
-					struct ColorObject *color_object=color_list_new_color_object(color_list, &r);
-					dynv_set_string(color_object->params, "name", s.str().c_str());
-					color_list_add_color_object(color_list, color_object, 1);
-					color_object_release(color_object);
+				for (step_i = start_step; step_i < max_step; ++step_i) {
+					MIX_COMPONENTS(r.rgb, a.rgb, b.rgb, red, green, blue);
+					STORE_LINEARCOLOR();
 				}
 				break;
 
@@ -110,19 +167,10 @@ static void calc( DialogMixArgs *args, bool preview, int limit){
 					color_rgb_to_hsv(&a, &a_hsv);
 					color_rgb_to_hsv(&b, &b_hsv);
 
-					for (step_i = 0; step_i < steps; ++step_i) {
-						r_hsv.hsv.hue = mix_float(a_hsv.hsv.hue, b_hsv.hsv.hue, step_i/(float)(steps-1));
-						r_hsv.hsv.saturation = mix_float(a_hsv.hsv.saturation, b_hsv.hsv.saturation, step_i/(float)(steps-1));
-						r_hsv.hsv.value = mix_float(a_hsv.hsv.value, b_hsv.hsv.value, step_i/(float)(steps-1));
-
+					for (step_i = start_step; step_i < max_step; ++step_i) {
+						MIX_COMPONENTS(r_hsv.hsv, a_hsv.hsv, b_hsv.hsv, hue, saturation, value);
 						color_hsv_to_rgb(&r_hsv, &r);
-
-						s.str("");
-						s<<name_a<<" "<<(step_i/float(steps-1))*100<< " mix " <<100-(step_i/float(steps-1))*100<<" "<< name_b;
-
-						struct ColorObject *color_object=color_list_new_color_object(color_list, &r);
-						dynv_set_string(color_object->params, "name", s.str().c_str());
-						color_list_add_color_object(color_list, color_object, 1);
+						STORE_COLOR();
 					}
 				}
 				break;
@@ -140,20 +188,13 @@ static void calc( DialogMixArgs *args, bool preview, int limit){
 						if (b_hsv.hsv.hue-a_hsv.hsv.hue>0.5)
 							b_hsv.hsv.hue-=1;
 					}
-					for (step_i = 0; step_i < steps; ++step_i) {
-						r_hsv.hsv.hue = mix_float(a_hsv.hsv.hue, b_hsv.hsv.hue, step_i/(float)(steps-1));
-						r_hsv.hsv.saturation = mix_float(a_hsv.hsv.saturation, b_hsv.hsv.saturation, step_i/(float)(steps-1));
-						r_hsv.hsv.value = mix_float(a_hsv.hsv.value, b_hsv.hsv.value, step_i/(float)(steps-1));
+					for (step_i = start_step; step_i < max_step; ++step_i) {
+						MIX_COMPONENTS(r_hsv.hsv, a_hsv.hsv, b_hsv.hsv, hue, saturation, value);
 
 						if (r_hsv.hsv.hue<0) r_hsv.hsv.hue+=1;
+
 						color_hsv_to_rgb(&r_hsv, &r);
-
-						s.str("");
-						s<<name_a<<" "<<(step_i/float(steps-1))*100<< " mix " <<100-(step_i/float(steps-1))*100<<" "<< name_b;
-
-						struct ColorObject *color_object=color_list_new_color_object(color_list, &r);
-						dynv_set_string(color_object->params, "name", s.str().c_str());
-						color_list_add_color_object(color_list, color_object, 1);
+						STORE_COLOR();
 					}
 				}
 				break;
@@ -161,23 +202,15 @@ static void calc( DialogMixArgs *args, bool preview, int limit){
 			case 3:
 				{
 					Color a_lab, b_lab, r_lab;
-					color_rgb_to_lab(&a, &a_lab, &d50, &working_space_matrix);
-					color_rgb_to_lab(&b, &b_lab, &d50, &working_space_matrix);
+					color_rgb_to_lab_d50(&a, &a_lab);
+					color_rgb_to_lab_d50(&b, &b_lab);
 
-					for (step_i = 0; step_i < steps; ++step_i) {
-						r_lab.lab.L = mix_float(a_lab.lab.L, b_lab.lab.L, step_i/(float)(steps-1));
-						r_lab.lab.a = mix_float(a_lab.lab.a, b_lab.lab.a, step_i/(float)(steps-1));
-						r_lab.lab.b = mix_float(a_lab.lab.b, b_lab.lab.b, step_i/(float)(steps-1));
+					for (step_i = start_step; step_i < max_step; ++step_i) {
+						MIX_COMPONENTS(r_lab.lab, a_lab.lab, b_lab.lab, L, a, b);
 
-						color_lab_to_rgb(&r_lab, &r, &d50, &working_space_matrix_inverted);
+						color_lab_to_rgb_d50(&r_lab, &r);
 						color_rgb_normalize(&r);
-
-						s.str("");
-						s<<name_a<<" "<<(step_i/float(steps-1))*100<< " mix " <<100-(step_i/float(steps-1))*100<<" "<< name_b;
-
-						struct ColorObject *color_object=color_list_new_color_object(color_list, &r);
-						dynv_set_string(color_object->params, "name", s.str().c_str());
-						color_list_add_color_object(color_list, color_object, 1);
+						STORE_COLOR();
 					}
 				}
 				break;
@@ -199,7 +232,7 @@ void dialog_mix_show(GtkWindow* parent, struct ColorList *selected_color_list, G
 	GtkWidget *table;
 	GtkWidget *mix_type, *mix_steps;
 
-	GtkWidget *dialog = gtk_dialog_new_with_buttons("Mix colors", parent, GtkDialogFlags(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
+	GtkWidget *dialog = gtk_dialog_new_with_buttons(_("Mix colors"), parent, GtkDialogFlags(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
 			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 			GTK_STOCK_OK, GTK_RESPONSE_OK,
 			NULL);
@@ -213,12 +246,12 @@ void dialog_mix_show(GtkWindow* parent, struct ColorList *selected_color_list, G
 	table = gtk_table_new(3, 2, FALSE);
 	table_y=0;
 
-	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new("Type:",0,0,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
+	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new(_("Type:"),0,0,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
 	mix_type = gtk_combo_box_new_text();
-	gtk_combo_box_append_text(GTK_COMBO_BOX(mix_type), "RGB");
-	gtk_combo_box_append_text(GTK_COMBO_BOX(mix_type), "HSV");
-	gtk_combo_box_append_text(GTK_COMBO_BOX(mix_type), "HSV shortest hue distance");
-	gtk_combo_box_append_text(GTK_COMBO_BOX(mix_type), "LAB");
+	gtk_combo_box_append_text(GTK_COMBO_BOX(mix_type), _("RGB"));
+	gtk_combo_box_append_text(GTK_COMBO_BOX(mix_type), _("HSV"));
+	gtk_combo_box_append_text(GTK_COMBO_BOX(mix_type), _("HSV shortest hue distance"));
+	gtk_combo_box_append_text(GTK_COMBO_BOX(mix_type), _("LAB"));
 	gtk_combo_box_set_active(GTK_COMBO_BOX(mix_type), dynv_get_int32_wd(args->params, "type", 0));
 	gtk_table_attach(GTK_TABLE(table), mix_type,1,2,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,5,0);
 	table_y++;
@@ -226,7 +259,7 @@ void dialog_mix_show(GtkWindow* parent, struct ColorList *selected_color_list, G
 	g_signal_connect (G_OBJECT (mix_type), "changed", G_CALLBACK (update), args);
 
 
-	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new("Steps:",0,0,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
+	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new(_("Steps:"),0,0,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
 	mix_steps = gtk_spin_button_new_with_range (3,255,1);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(mix_steps), dynv_get_int32_wd(args->params, "steps", 3));
 	gtk_table_attach(GTK_TABLE(table), mix_steps,1,2,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,5,0);
@@ -234,9 +267,15 @@ void dialog_mix_show(GtkWindow* parent, struct ColorList *selected_color_list, G
 	args->mix_steps = mix_steps;
 	g_signal_connect (G_OBJECT (mix_steps), "value-changed", G_CALLBACK (update), args);
 
+	args->toggle_endpoints = gtk_check_button_new_with_mnemonic (_("_Include Endpoints"));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(args->toggle_endpoints), dynv_get_bool_wd(args->params, "includeendpoints", true));
+	gtk_table_attach(GTK_TABLE(table), args->toggle_endpoints,1,4,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,5,0);
+	g_signal_connect (G_OBJECT(args->toggle_endpoints), "toggled", G_CALLBACK (update), args);
+	table_y++;
+
 	GtkWidget* preview_expander;
 	struct ColorList* preview_color_list=NULL;
-	gtk_table_attach(GTK_TABLE(table), preview_expander=palette_list_preview_new(gs, dynv_get_bool_wd(args->params, "show_preview", true), gs->colors, &preview_color_list), 0, 2, table_y, table_y+1 , GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL | GTK_EXPAND), 5, 5);
+	gtk_table_attach(GTK_TABLE(table), preview_expander=palette_list_preview_new(gs, true, dynv_get_bool_wd(args->params, "show_preview", true), gs->colors, &preview_color_list), 0, 2, table_y, table_y+1 , GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL | GTK_EXPAND), 5, 5);
 	table_y++;
 
 
