@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2011, Albertas Vyšniauskas
+ * Copyright (c) 2009-2012, Albertas Vyšniauskas
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -18,16 +18,36 @@
 
 #include "uiDialogOptions.h"
 #include "uiUtilities.h"
+#include "ToolColorNaming.h"
 #include "GlobalStateStruct.h"
+#include "Internationalisation.h"
 
 #include "DynvHelpers.h"
 
+#include <string>
+
+using namespace std;
+
+static const struct{
+	const char *label;
+	const char *setting;
+}available_color_spaces[] = {
+	{"CMYK", "picker.color_space.cmyk"},
+	{"HSL", "picker.color_space.hsl"},
+	{"HSV", "picker.color_space.hsv"},
+	{"LAB", "picker.color_space.lab"},
+	{"LCH", "picker.color_space.lch"},
+	{"RGB", "picker.color_space.rgb"},
+	{0, 0},
+};
+
 typedef struct DialogOptionsArgs{
 	GtkWidget *minimize_to_tray;
 	GtkWidget *close_to_tray;
 	GtkWidget *start_in_tray;
 	GtkWidget *refresh_rate;
 	GtkWidget *single_instance;
+	GtkWidget *save_restore_palette;
 	GtkWidget *add_on_release;
 	GtkWidget *add_to_palette;
 	GtkWidget *copy_to_clipboard;
@@ -35,6 +55,12 @@ typedef struct DialogOptionsArgs{
 	GtkWidget *copy_on_release;
 	GtkWidget *zoom_size;
 	GtkWidget *imprecision_postfix;
+	GtkWidget *tool_color_naming[3];
+	GtkWidget *color_spaces[6];
+	GtkWidget *out_of_gamut_mask;
+
+	GtkWidget *lab_illuminant;
+	GtkWidget *lab_observer;
 
 	struct dynvSystem *params;
 	GlobalState* gs;
@@ -47,6 +73,7 @@ static void calc( DialogOptionsArgs *args, bool preview, int limit){
 	dynv_set_bool(args->params, "main.close_to_tray", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(args->close_to_tray)));
 	dynv_set_bool(args->params, "main.start_in_tray", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(args->start_in_tray)));
 	dynv_set_bool(args->params, "main.single_instance", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(args->single_instance)));
+	dynv_set_bool(args->params, "main.save_restore_palette", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(args->save_restore_palette)));
 
 	dynv_set_float(args->params, "picker.refresh_rate", gtk_spin_button_get_value(GTK_SPIN_BUTTON(args->refresh_rate)));
 	dynv_set_int32(args->params, "picker.zoom_size", gtk_spin_button_get_value(GTK_SPIN_BUTTON(args->zoom_size)));
@@ -55,8 +82,25 @@ static void calc( DialogOptionsArgs *args, bool preview, int limit){
 	dynv_set_bool(args->params, "picker.sampler.add_to_palette", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(args->add_to_palette)));
 	dynv_set_bool(args->params, "picker.sampler.copy_to_clipboard", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(args->copy_to_clipboard)));
 	dynv_set_bool(args->params, "picker.sampler.rotate_swatch_after_sample", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(args->rotate_swatch)));
+	dynv_set_bool(args->params, "picker.out_of_gamut_mask", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(args->out_of_gamut_mask)));
 
 	dynv_set_bool(args->params, "color_names.imprecision_postfix", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(args->imprecision_postfix)));
+	const ToolColorNamingOption *color_naming_options = tool_color_naming_get_options();
+	int i = 0;
+	while (color_naming_options[i].name){
+		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(args->tool_color_naming[i]))){
+			dynv_set_string(args->params, "color_names.tool_color_naming", color_naming_options[i].name);
+			break;
+		}
+		i++;
+	}
+
+	for (int i = 0; available_color_spaces[i].label; i++){
+		dynv_set_bool(args->params, available_color_spaces[i].setting, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(args->color_spaces[i])));
+	}
+
+	dynv_set_string(args->params, "picker.lab.illuminant", gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(args->lab_illuminant)));
+	dynv_set_string(args->params, "picker.lab.observer", gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(args->lab_observer)));
 }
 
 
@@ -70,7 +114,7 @@ void dialog_options_show(GtkWindow* parent, GlobalState* gs) {
 	GtkWidget *table, *table_m, *widget;
 
 
-	GtkWidget *dialog = gtk_dialog_new_with_buttons("Options", parent, GtkDialogFlags(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
+	GtkWidget *dialog = gtk_dialog_new_with_buttons(_("Options"), parent, GtkDialogFlags(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
 			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 			GTK_STOCK_OK, GTK_RESPONSE_OK,
 			NULL);
@@ -87,7 +131,7 @@ void dialog_options_show(GtkWindow* parent, GlobalState* gs) {
 
 	table_m = gtk_table_new(3, 1, FALSE);
 	table_m_y = 0;
-	frame = gtk_frame_new("System");
+	frame = gtk_frame_new(_("System"));
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
 	gtk_table_attach(GTK_TABLE(table_m), frame, 0, 1, table_m_y, table_m_y+1, GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL), 5, 5);
 	table_m_y++;
@@ -96,13 +140,18 @@ void dialog_options_show(GtkWindow* parent, GlobalState* gs) {
 	table_y=0;
 	gtk_container_add(GTK_CONTAINER(frame), table);
 
-	args->single_instance = widget = gtk_check_button_new_with_mnemonic ("_Single instance");
+	args->single_instance = widget = gtk_check_button_new_with_mnemonic (_("_Single instance"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), dynv_get_bool_wd(args->params, "main.single_instance", false));
 	gtk_table_attach(GTK_TABLE(table), widget,0,3,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,3,3);
 	table_y++;
 
+	args->save_restore_palette = widget = gtk_check_button_new_with_mnemonic (_("Save/_Restore palette"));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), dynv_get_bool_wd(args->params, "main.save_restore_palette", true));
+	gtk_table_attach(GTK_TABLE(table), widget,0,3,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,3,3);
+	table_y++;
 
-	frame = gtk_frame_new("System tray");
+
+	frame = gtk_frame_new(_("System tray"));
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
 	gtk_table_attach(GTK_TABLE(table_m), frame, 0, 1, table_m_y, table_m_y+1, GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL), 5, 5);
 	table_m_y++;
@@ -110,29 +159,29 @@ void dialog_options_show(GtkWindow* parent, GlobalState* gs) {
 	table_y=0;
 	gtk_container_add(GTK_CONTAINER(frame), table);
 
-	args->minimize_to_tray = widget = gtk_check_button_new_with_mnemonic ("_Minimize to system tray");
+	args->minimize_to_tray = widget = gtk_check_button_new_with_mnemonic (_("_Minimize to system tray"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), dynv_get_bool_wd(args->params, "main.minimize_to_tray", false));
 	gtk_table_attach(GTK_TABLE(table), widget,0,3,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,3,3);
 	table_y++;
 
-	args->close_to_tray = widget = gtk_check_button_new_with_mnemonic ("_Close to system tray");
+	args->close_to_tray = widget = gtk_check_button_new_with_mnemonic (_("_Close to system tray"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), dynv_get_bool_wd(args->params, "main.close_to_tray", false));
 	gtk_table_attach(GTK_TABLE(table), widget,0,3,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,3,3);
 	table_y++;
 
-	args->start_in_tray = widget = gtk_check_button_new_with_mnemonic ("_Start in system tray");
+	args->start_in_tray = widget = gtk_check_button_new_with_mnemonic (_("_Start in system tray"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), dynv_get_bool_wd(args->params, "main.start_in_tray", false));
 	gtk_table_attach(GTK_TABLE(table), widget,0,3,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,3,3);
 	table_y++;
 
 
-	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table_m, gtk_label_new_with_mnemonic("_Main"));
+	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table_m, gtk_label_new_with_mnemonic(_("_Main")));
 
 
 
-	table_m = gtk_table_new(3, 1, FALSE);
+	table_m = gtk_table_new(3, 2, FALSE);
 	table_m_y = 0;
-	frame = gtk_frame_new("Display");
+	frame = gtk_frame_new(_("Display"));
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
 	gtk_table_attach(GTK_TABLE(table_m), frame, 0, 1, table_m_y, table_m_y+1, GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL), 5, 5);
 	table_m_y++;
@@ -141,7 +190,7 @@ void dialog_options_show(GtkWindow* parent, GlobalState* gs) {
 	table_y=0;
 	gtk_container_add(GTK_CONTAINER(frame), table);
 
-	gtk_table_attach(GTK_TABLE(table), gtk_label_mnemonic_aligned_new("_Refresh rate:",0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,3,3);
+	gtk_table_attach(GTK_TABLE(table), gtk_label_mnemonic_aligned_new(_("_Refresh rate:"),0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,3,3);
 	args->refresh_rate = widget = gtk_spin_button_new_with_range(1, 60, 1);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(args->refresh_rate), dynv_get_float_wd(args->params, "picker.refresh_rate", 30));
 	gtk_table_attach(GTK_TABLE(table), widget,1,2,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,5,5);
@@ -149,13 +198,13 @@ void dialog_options_show(GtkWindow* parent, GlobalState* gs) {
 	table_y++;
 
 
-	gtk_table_attach(GTK_TABLE(table), gtk_label_mnemonic_aligned_new("_Magnified area size:",0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,3,3);
+	gtk_table_attach(GTK_TABLE(table), gtk_label_mnemonic_aligned_new(_("_Magnified area size:"),0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,3,3);
 	args->zoom_size = widget = gtk_spin_button_new_with_range(75, 300, 15);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(args->zoom_size), dynv_get_int32_wd(args->params, "picker.zoom_size", 150));
 	gtk_table_attach(GTK_TABLE(table), widget,1,3,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,5,5);
 	table_y++;
 
-	frame = gtk_frame_new("Floating picker click behaviour");
+	frame = gtk_frame_new(_("Floating picker click behaviour"));
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
 	gtk_table_attach(GTK_TABLE(table_m), frame, 0, 1, table_m_y, table_m_y+1, GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL), 5, 5);
 	table_m_y++;
@@ -163,18 +212,18 @@ void dialog_options_show(GtkWindow* parent, GlobalState* gs) {
 	table_y=0;
 	gtk_container_add(GTK_CONTAINER(frame), table);
 
-	args->add_on_release = widget = gtk_check_button_new_with_mnemonic("_Add to palette");
+	args->add_on_release = widget = gtk_check_button_new_with_mnemonic(_("_Add to palette"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), dynv_get_bool_wd(args->params, "picker.sampler.add_on_release", false));
 	gtk_table_attach(GTK_TABLE(table), widget,0,3,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,3,3);
 	table_y++;
 
-	args->copy_on_release = widget = gtk_check_button_new_with_mnemonic("_Copy to clipboard");
+	args->copy_on_release = widget = gtk_check_button_new_with_mnemonic(_("_Copy to clipboard"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), dynv_get_bool_wd(args->params, "picker.sampler.copy_on_release", false));
 	gtk_table_attach(GTK_TABLE(table), widget,0,3,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,3,3);
 	table_y++;
 
 
-	frame = gtk_frame_new("'Spacebar' button behaviour");
+	frame = gtk_frame_new(_("'Spacebar' button behaviour"));
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
 	gtk_table_attach(GTK_TABLE(table_m), frame, 0, 1, table_m_y, table_m_y+1, GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL), 5, 5);
 	table_m_y++;
@@ -182,29 +231,114 @@ void dialog_options_show(GtkWindow* parent, GlobalState* gs) {
 	table_y=0;
 	gtk_container_add(GTK_CONTAINER(frame), table);
 
-	args->add_to_palette = widget = gtk_check_button_new_with_mnemonic("_Add to palette");
+	args->add_to_palette = widget = gtk_check_button_new_with_mnemonic(_("_Add to palette"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), dynv_get_bool_wd(args->params, "picker.sampler.add_to_palette", false));
 	gtk_table_attach(GTK_TABLE(table), widget,1,2,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,3,3);
 	table_y++;
 
-	args->copy_to_clipboard = widget = gtk_check_button_new_with_mnemonic("_Copy to clipboard");
+	args->copy_to_clipboard = widget = gtk_check_button_new_with_mnemonic(_("_Copy to clipboard"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), dynv_get_bool_wd(args->params, "picker.sampler.copy_to_clipboard", false));
 	gtk_table_attach(GTK_TABLE(table), widget,1,2,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,3,3);
 	table_y++;
 
-	args->rotate_swatch = widget = gtk_check_button_new_with_mnemonic("_Rotate swatch");
+	args->rotate_swatch = widget = gtk_check_button_new_with_mnemonic(_("_Rotate swatch"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), dynv_get_bool_wd(args->params, "picker.sampler.rotate_swatch_after_sample", false));
 	gtk_table_attach(GTK_TABLE(table), widget,1,2,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,3,3);
 	table_y++;
 
+	table_m_y = 0;
 
-	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table_m, gtk_label_new_with_mnemonic("_Picker"));
+	frame = gtk_frame_new(_("Enabled color spaces"));
+	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
+	gtk_table_attach(GTK_TABLE(table_m), frame, 1, 2, table_m_y, table_m_y+1, GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL), 5, 5);
+	table_m_y++;
+	table = gtk_table_new(5, 3, FALSE);
+	table_y=0;
+	gtk_container_add(GTK_CONTAINER(frame), table);
 
+	for (int i = 0; available_color_spaces[i].label; i++){
+		args->color_spaces[i] = widget = gtk_check_button_new_with_label(available_color_spaces[i].label);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), dynv_get_bool_wd(args->params, available_color_spaces[i].setting, true));
+		gtk_table_attach(GTK_TABLE(table), widget, 1, 2, table_y, table_y+1, GtkAttachOptions(GTK_FILL | GTK_EXPAND), GTK_FILL, 3, 3);
+		table_y++;
+	}
+
+	frame = gtk_frame_new(_("Lab settings"));
+	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
+	gtk_table_attach(GTK_TABLE(table_m), frame, 1, 2, table_m_y, table_m_y+1, GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL), 5, 5);
+	table_m_y++;
+	table = gtk_table_new(5, 3, FALSE);
+	table_y=0;
+	gtk_container_add(GTK_CONTAINER(frame), table);
+
+	{
+		int selected;
+		const char *option;
+
+		gtk_table_attach(GTK_TABLE(table), gtk_label_mnemonic_aligned_new(_("_Illuminant:"),0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,3,3);
+		args->lab_illuminant = widget = gtk_combo_box_text_new();
+		const char *illuminants[] = {
+			"A",
+			"C",
+			"D50",
+			"D55",
+			"D65",
+			"D75",
+			"F2",
+			"F7",
+			"F11",
+			0,
+		};
+		selected = 0;
+		option = dynv_get_string_wd(args->params, "picker.lab.illuminant", "D50");
+		for (int i = 0; illuminants[i]; i++){
+			if (string(illuminants[i]).compare(option) == 0) selected = i;
+			gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widget), illuminants[i]);
+		}
+		gtk_combo_box_set_active(GTK_COMBO_BOX(widget), selected);
+		gtk_table_attach(GTK_TABLE(table), widget,1,3,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,5,5);
+		table_y++;
+
+		gtk_table_attach(GTK_TABLE(table), gtk_label_mnemonic_aligned_new(_("_Observer:"),0,0.5,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,3,3);
+		args->lab_observer = widget = gtk_combo_box_text_new();
+		const char *observers[] = {
+			"2",
+			"10",
+			0,
+		};
+		selected = 0;
+		option = dynv_get_string_wd(args->params, "picker.lab.observer", "2");
+		for (int i = 0; observers[i]; i++){
+			if (string(observers[i]).compare(option) == 0) selected = i;
+			gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widget), observers[i]);
+		}
+		gtk_combo_box_set_active(GTK_COMBO_BOX(widget), selected);
+		gtk_table_attach(GTK_TABLE(table), widget,1,3,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,5,5);
+		table_y++;
+
+	}
+
+	frame = gtk_frame_new(_("Other settings"));
+	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
+	gtk_table_attach(GTK_TABLE(table_m), frame, 1, 2, table_m_y, table_m_y+1, GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL), 5, 5);
+	table_m_y++;
+	table = gtk_table_new(5, 3, FALSE);
+	table_y=0;
+	gtk_container_add(GTK_CONTAINER(frame), table);
+
+
+		args->out_of_gamut_mask = widget = gtk_check_button_new_with_mnemonic(_("_Mask out of gamut colors"));
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), dynv_get_bool_wd(args->params, "picker.out_of_gamut_mask", true));
+		gtk_table_attach(GTK_TABLE(table), widget, 1, 2, table_y, table_y+1, GtkAttachOptions(GTK_FILL | GTK_EXPAND), GTK_FILL, 3, 3);
+		table_y++;
+
+
+	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table_m, gtk_label_new_with_mnemonic(_("_Picker")));
 
 
 	table_m = gtk_table_new(3, 1, FALSE);
 	table_m_y = 0;
-	frame = gtk_frame_new("Color name generation");
+	frame = gtk_frame_new(_("Color name generation"));
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
 	gtk_table_attach(GTK_TABLE(table_m), frame, 0, 1, table_m_y, table_m_y+1, GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL), 5, 5);
 	table_m_y++;
@@ -213,12 +347,38 @@ void dialog_options_show(GtkWindow* parent, GlobalState* gs) {
 	table_y=0;
 	gtk_container_add(GTK_CONTAINER(frame), table);
 
-	args->imprecision_postfix = widget = gtk_check_button_new_with_mnemonic("_Imprecision postix");
+	args->imprecision_postfix = widget = gtk_check_button_new_with_mnemonic(_("_Imprecision postfix"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), dynv_get_bool_wd(args->params, "color_names.imprecision_postfix", true));
 	gtk_table_attach(GTK_TABLE(table), widget,1,2,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,3,3);
 	table_y++;
 
-	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table_m, gtk_label_new_with_mnemonic("_Color names"));
+
+	frame = gtk_frame_new(_("Tool color naming"));
+	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
+	gtk_table_attach(GTK_TABLE(table_m), frame, 0, 1, table_m_y, table_m_y+1, GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL), 5, 5);
+	table_m_y++;
+
+	table = gtk_table_new(5, 3, FALSE);
+	table_y=0;
+	gtk_container_add(GTK_CONTAINER(frame), table);
+
+	GSList *group = NULL;
+	ToolColorNamingType color_naming_type = tool_color_naming_name_to_type(dynv_get_string_wd(args->params, "color_names.tool_color_naming", "tool_specific"));
+	const ToolColorNamingOption *color_naming_options = tool_color_naming_get_options();
+	int i = 0;
+	while (color_naming_options[i].name){
+		args->tool_color_naming[i] = widget = gtk_radio_button_new_with_mnemonic(group, _(color_naming_options[i].label));
+		group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(widget));
+
+		if (color_naming_type == color_naming_options[i].type)
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), true);
+
+		gtk_table_attach(GTK_TABLE(table), widget,1,2,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,3,3);
+		table_y++;
+		i++;
+	}
+
+	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table_m, gtk_label_new_with_mnemonic(_("_Color names")));
 
 
 	gtk_widget_show_all(notebook);
