@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2011, Albertas Vyšniauskas
+ * Copyright (c) 2009-2012, Albertas Vyšniauskas
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -17,18 +17,23 @@
  */
 
 #include "uiExport.h"
+#include "FileFormat.h"
 #include "uiUtilities.h"
 #include "uiListPalette.h"
 #include "Endian.h"
 #include "DynvHelpers.h"
 #include "GlobalStateStruct.h"
+#include "Internationalisation.h"
 
 #include <string.h>
+#include <math.h>
 
 #include <fstream>
 #include <string>
 #include <iostream>
 #include <sstream>
+#include <boost/math/special_functions/round.hpp>
+
 using namespace std;
 
 
@@ -37,9 +42,10 @@ static int32_t palette_export_gpl_color(struct ColorObject* color_object, void*
 	color_object_get_color(color_object, &color);
 	const char* name = dynv_get_string_wd(color_object->params, "name", "");
 
-	(*(ofstream*)userdata) << int32_t(color.rgb.red*255) << "\t"
-						<< int32_t(color.rgb.green*255) << "\t"
-						<< int32_t(color.rgb.blue*255) << "\t" << name << endl;
+	(*(ofstream*)userdata)
+		<< boost::math::iround(color.rgb.red*255) << "\t"
+		<< boost::math::iround(color.rgb.green*255) << "\t"
+		<< boost::math::iround(color.rgb.blue*255) << "\t" << name << endl;
 	return 0;
 }
 
@@ -68,7 +74,15 @@ static vector<struct ColorObject*> get_ordered_list(struct ColorList *color_list
 		}
 }
 
-static int32_t palette_export_gpl(struct ColorList *color_list, const gchar* filename, gboolean selected){
+int32_t palette_import_gpa(struct ColorList *color_list, const gchar* filename){
+	return palette_file_load (filename, color_list);
+}
+
+int32_t palette_export_gpa(struct ColorList *color_list, const gchar* filename, gboolean selected){
+	return palette_file_save (filename, color_list);
+}
+
+int32_t palette_export_gpl(struct ColorList *color_list, const gchar* filename, gboolean selected){
 	ofstream f(filename, ios::out | ios::trunc);
 	if (f.is_open()){
 
@@ -81,7 +95,7 @@ static int32_t palette_export_gpl(struct ColorList *color_list, const gchar* fil
 
 		g_free(name);
 
-    vector<struct ColorObject*> ordered = get_ordered_list(color_list);
+		vector<struct ColorObject*> ordered = get_ordered_list(color_list);
 		for (vector<struct ColorObject*>::iterator i=ordered.begin(); i!=ordered.end(); ++i){
 			palette_export_gpl_color(*i, &f);
 		}
@@ -107,7 +121,7 @@ static void strip_leading_trailing_chars(string& x, string& stripchars){
    x = x.substr(start, end - start + 1);
 }
 
-static int32_t palette_import_gpl(struct ColorList *color_list, const gchar* filename){
+int32_t palette_import_gpl(struct ColorList *color_list, const gchar* filename){
 	ifstream f(filename, ios::in);
 	if (f.is_open()){
 		int r = 0;
@@ -116,19 +130,16 @@ static int32_t palette_import_gpl(struct ColorList *color_list, const gchar* fil
 		if (f.good() && line=="GIMP Palette"){
 			do{
 				getline(f, line);
-			}while (f.good() && line!="#");
+			}while (f.good() && ((line.size() < 1) || (line[0] > '9') || (line[0] < '0')));
 
-			if (line=="#"){
-				int r, g, b;
-				Color c;
-				struct ColorObject* color_object;
-				string strip_chars = " \t";
-
-				for(;;){
-					getline(f, line);
-					if (!f.good()) break;
-					if (line[0]=='#') continue;
+			int r, g, b;
+			Color c;
+			struct ColorObject* color_object;
+			string strip_chars = " \t";
 
+			for(;;){
+				if (!f.good()) break;
+				if (line[0]!='#'){
 					stringstream ss(line);
 
 					ss >> r >> g >> b;
@@ -145,8 +156,9 @@ static int32_t palette_import_gpl(struct ColorList *color_list, const gchar* fil
 					color_list_add_color_object(color_list, color_object, TRUE);
 					color_object_release(color_object);
 				}
+				getline(f, line);
+			}
 
-			}else r = -1;
 		}else r = -1;
 
 		f.close();
@@ -237,7 +249,7 @@ static int32_t palette_export_ase_color(struct ColorObject* color_object, void*
 	return 0;
 }
 
-static int32_t palette_export_ase(struct ColorList *color_list, const gchar* filename, gboolean selected){
+int32_t palette_export_ase(struct ColorList *color_list, const gchar* filename, gboolean selected){
 	ofstream f(filename, ios::out | ios::trunc | ios::binary);
 	if (f.is_open()){
 		f << "ASEF";	//magic header
@@ -260,7 +272,7 @@ static int32_t palette_export_ase(struct ColorList *color_list, const gchar* fil
 }
 
 
-static int32_t palette_import_ase(struct ColorList *color_list, const gchar* filename){
+int32_t palette_import_ase(struct ColorList *color_list, const gchar* filename){
 	ifstream f(filename, ios::binary);
 	if (f.is_open()){
 		char magic[4];
@@ -282,15 +294,6 @@ static int32_t palette_import_ase(struct ColorList *color_list, const gchar* fil
 		uint32_t block_size;
 		int color_supported;
 
-	    matrix3x3 adaptation_matrix, working_space_matrix;
-	    vector3 d50, d65;
-	    vector3_set(&d50, 96.442, 100.000,  82.821);    // D50 illuminant tristimulus values
-	    vector3_set(&d65, 95.047, 100.000, 108.883);    // D65 illuminant tristimulus values
-	    color_get_chromatic_adaptation_matrix(&d50, &d65, &adaptation_matrix);
-	    // constants used below are sRGB working space red, green and blue primaries for D65 reference white
-	    color_get_working_space_matrix(0.6400, 0.3300, 0.3000, 0.6000, 0.1500, 0.0600, &d65, &working_space_matrix);
-	    matrix3x3_inverse(&working_space_matrix, &working_space_matrix);
-
 		for (uint32_t i=0; i<blocks; ++i){
 			f.read((char*)&block_type, 2);
 			block_type=UINT16_FROM_BE(block_type);
@@ -313,11 +316,11 @@ static int32_t palette_import_ase(struct ColorList *color_list, const gchar* fil
 
 					Color c;
 
-					char colorspace[4];
-					f.read(colorspace, 4);
+					char color_space[4];
+					f.read(color_space, 4);
 					color_supported = 0;
 
-					if (memcmp(colorspace, "RGB ", 4)==0){
+					if (memcmp(color_space, "RGB ", 4)==0){
 						FloatInt rgb[3];
 						f.read((char*)&rgb[0], 4);
 						f.read((char*)&rgb[1], 4);
@@ -333,7 +336,7 @@ static int32_t palette_import_ase(struct ColorList *color_list, const gchar* fil
 
 						color_supported = 1;
 
-					}else if (memcmp(colorspace, "CMYK", 4)==0){
+					}else if (memcmp(color_space, "CMYK", 4)==0){
 						Color c2;
 						FloatInt cmyk[4];
 
@@ -356,7 +359,7 @@ static int32_t palette_import_ase(struct ColorList *color_list, const gchar* fil
 
 						color_supported = 1;
 
-					}else if (memcmp(colorspace, "Gray", 4)==0){
+					}else if (memcmp(color_space, "Gray", 4)==0){
 						FloatInt gray;
 						f.read((char*)&gray, 4);
 						gray.i = UINT32_FROM_BE(gray.i);
@@ -364,8 +367,8 @@ static int32_t palette_import_ase(struct ColorList *color_list, const gchar* fil
 
 						color_supported = 1;
 
-					}else if (memcmp(colorspace, "LAB ", 4)==0){
-						Color c2, c3;
+					}else if (memcmp(color_space, "LAB ", 4)==0){
+						Color c2;
 						FloatInt lab[3];
 						f.read((char*)&lab[0], 4);
 						f.read((char*)&lab[1], 4);
@@ -379,13 +382,11 @@ static int32_t palette_import_ase(struct ColorList *color_list, const gchar* fil
 						c2.lab.a = lab[1].f;
 						c2.lab.b = lab[2].f;
 
-					    color_lab_to_xyz(&c2, &c3, &d50);
-						color_xyz_chromatic_adaptation(&c3, &c3, &adaptation_matrix);
-					    color_xyz_to_rgb(&c3, &c, &working_space_matrix);
+						color_lab_to_rgb_d50(&c2, &c);
 
-					    c.rgb.red = clamp_float(c.rgb.red, 0, 1);
-					    c.rgb.green = clamp_float(c.rgb.green, 0, 1);
-					    c.rgb.blue = clamp_float(c.rgb.blue, 0, 1);
+						c.rgb.red = clamp_float(c.rgb.red, 0, 1);
+						c.rgb.green = clamp_float(c.rgb.green, 0, 1);
+						c.rgb.blue = clamp_float(c.rgb.blue, 0, 1);
 
 						color_supported = 1;
 
@@ -453,7 +454,7 @@ int dialog_export_show(GtkWindow* parent, struct ColorList *selected_color_list,
 	GtkWidget *dialog;
 	GtkFileFilter *filter;
 
-	dialog = gtk_file_chooser_dialog_new ("Export",	parent,
+	dialog = gtk_file_chooser_dialog_new (_("Export"),	parent,
 						  GTK_FILE_CHOOSER_ACTION_SAVE,
 						  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 						  GTK_STOCK_SAVE, GTK_RESPONSE_OK,
@@ -469,10 +470,11 @@ int dialog_export_show(GtkWindow* parent, struct ColorList *selected_color_list,
 		int32_t (*export_function)(struct ColorList *color_list, const gchar* filename, gboolean selected);
 	};
 	struct export_formats formats[] = {
-		{ "GIMP/Inkscape Palette *.gpl", "*.gpl", palette_export_gpl },
-		{ "Alias/WaveFront Material *.mtl", "*.mtl", palette_export_mtl },
-		{ "Adobe Swatch Exchange *.ase", "*.ase", palette_export_ase },
-		{ "Text file *.txt", "*.txt", palette_export_txt },
+		{ _("Gpick Palette (*.gpa)"), "*.gpa", palette_export_gpa },
+		{ _("GIMP/Inkscape Palette (*.gpl)"), "*.gpl", palette_export_gpl },
+		{ _("Alias/WaveFront Material (*.mtl)"), "*.mtl", palette_export_mtl },
+		{ _("Adobe Swatch Exchange (*.ase)"), "*.ase", palette_export_ase },
+		{ _("Text file (*.txt)"), "*.txt", palette_export_txt },
 	};
 
 
@@ -517,8 +519,8 @@ int dialog_export_show(GtkWindow* parent, struct ColorList *selected_color_list,
 						saved=TRUE;
 					}else{
 						GtkWidget* message;
-						message=gtk_message_dialog_new(GTK_WINDOW(dialog), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "File could not be exported");
-						gtk_window_set_title(GTK_WINDOW(dialog), "Export");
+						message=gtk_message_dialog_new(GTK_WINDOW(dialog), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("File could not be exported"));
+						gtk_window_set_title(GTK_WINDOW(dialog), _("Export"));
 						gtk_dialog_run(GTK_DIALOG(message));
 						gtk_widget_destroy(message);
 					}
@@ -534,14 +536,41 @@ int dialog_export_show(GtkWindow* parent, struct ColorList *selected_color_list,
 	return 0;
 }
 
+enum FileType{
+	GPA,
+	GPL,
+	ASE,
+	UNKNOWN,
+};
+
+static FileType get_file_type_from_ext(const char *filename){
+	const struct{
+		FileType type;
+		const char *extension;
+	}extensions[] = {
+		{GPA, ".gpa"},
+		{GPL, ".gpl"},
+		{ASE, ".ase"},
+		{UNKNOWN, 0},
+	};
+	const char *ext = g_strrstr(filename, ".");
+	if (ext){
+		for (int i = 0; extensions[i].type != UNKNOWN; ++i){
+			if (g_ascii_strcasecmp(ext, extensions[i].extension) == 0){
+				return extensions[i].type;
+			}
+		}
+	}
+	return UNKNOWN;
+}
 
 int dialog_import_show(GtkWindow* parent, struct ColorList *selected_color_list, GlobalState *gs){
 
 	GtkWidget *dialog;
-
+	GtkWidget* message;
 	GtkFileFilter *filter;
 
-	dialog = gtk_file_chooser_dialog_new ("Import",	parent,
+	dialog = gtk_file_chooser_dialog_new (_("Import"),	parent,
 						  GTK_FILE_CHOOSER_ACTION_OPEN,
 						  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 						  GTK_STOCK_OPEN, GTK_RESPONSE_OK,
@@ -549,30 +578,46 @@ int dialog_import_show(GtkWindow* parent, struct ColorList *selected_color_list,
 
 	gtk_dialog_set_alternative_button_order(GTK_DIALOG(dialog), GTK_RESPONSE_OK, GTK_RESPONSE_CANCEL, -1);
 
-	struct import_formats{
+	const struct import_formats{
+		FileType type;
 		const gchar* name;
 		const gchar* pattern;
 		int32_t (*import_function)(struct ColorList *color_list, const gchar* filename);
+	}formats[] = {
+		{GPA, _("Gpick Palette (*.gpa)"), "*.gpa", palette_import_gpa },
+		{GPL, _("GIMP/Inkscape Palette (*.gpl)"), "*.gpl", palette_import_gpl },
+		{ASE, _("Adobe Swatch Exchange (*.ase)"), "*.ase", palette_import_ase },
+		{UNKNOWN, NULL, NULL, NULL},
 	};
-	struct import_formats formats[] = {
-		{ "GIMP/Inkscape Palette *.gpl", "*.gpl", palette_import_gpl },
-		{ "Adobe Swatch Exchange *.ase", "*.ase", palette_import_ase },
-
-		/*{ "Alias/WaveFront Material *.mtl", "*.mtl", palette_export_mtl },
-
-		{ "Text file *.txt", "*.txt", palette_export_txt },*/
-	};
-
-
 
 	const char* default_path = dynv_get_string_wd(gs->params, "gpick.import.path", "");
 	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), default_path);
+	const char* selected_filter = dynv_get_string_wd(gs->params, "gpick.import.filter", "all_supported");
 
-	const char* selected_filter = dynv_get_string_wd(gs->params, "gpick.import.filter", "*.gpl");
+	filter = gtk_file_filter_new();
+	gtk_file_filter_set_name(filter, _("All files"));
+	g_object_set_data(G_OBJECT(filter), "identification", (gpointer)"all");
+	gtk_file_filter_add_pattern(filter, "*");
+	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);
+	if (g_strcmp0(selected_filter, "all") == 0){
+		gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(dialog), filter);
+	}
 
-	for (gint i = 0; i != sizeof(formats) / sizeof(struct import_formats); ++i) {
+	filter = gtk_file_filter_new();
+	gtk_file_filter_set_name(filter, _("All supported formats"));
+	g_object_set_data(G_OBJECT(filter), "identification", (gpointer)"all_supported");
+	for (gint i = 0; formats[i].name; ++i) {
+		gtk_file_filter_add_pattern(filter, formats[i].pattern);
+	}
+	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);
+	if (g_strcmp0(selected_filter, "all_supported") == 0){
+		gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(dialog), filter);
+	}
+
+	for (gint i = 0; formats[i].name; ++i) {
 		filter = gtk_file_filter_new();
 		gtk_file_filter_set_name(filter, formats[i].name);
+		g_object_set_data(G_OBJECT(filter), "identification", (gpointer)formats[i].pattern);
 		gtk_file_filter_add_pattern(filter, formats[i].pattern);
 		gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);
 
@@ -581,7 +626,7 @@ int dialog_import_show(GtkWindow* parent, struct ColorList *selected_color_list,
 		}
 	}
 
-	gboolean finished=FALSE;
+	gboolean finished = false;
 
 	while (!finished){
 		if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) {
@@ -593,21 +638,40 @@ int dialog_import_show(GtkWindow* parent, struct ColorList *selected_color_list,
 			dynv_set_string(gs->params, "gpick.import.path", path);
 			g_free(path);
 
-			const gchar *format_name = gtk_file_filter_get_name(gtk_file_chooser_get_filter(GTK_FILE_CHOOSER(dialog)));
-			for (gint i = 0; i != sizeof(formats) / sizeof(struct import_formats); ++i) {
-				if (g_strcmp0(formats[i].name, format_name)==0){
-					struct ColorList *color_list_arg;
-					color_list_arg = gs->colors;
-					if (formats[i].import_function(color_list_arg, filename)==0){
-						finished=TRUE;
-					}else{
-						GtkWidget* message;
-						message=gtk_message_dialog_new(GTK_WINDOW(dialog), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "File could not be imported");
-						gtk_window_set_title(GTK_WINDOW(dialog), "Import");
-						gtk_dialog_run(GTK_DIALOG(message));
-						gtk_widget_destroy(message);
+      FileType type = get_file_type_from_ext(filename);
+			if (type == UNKNOWN){
+				const gchar *format_name = gtk_file_filter_get_name(gtk_file_chooser_get_filter(GTK_FILE_CHOOSER(dialog)));
+				for (int i = 0; formats[i].name; ++i) {
+					if (g_strcmp0(formats[i].name, format_name) == 0){
+						type = formats[i].type;
+						break;
+					}
+				}
+			}
+
+			if (type == UNKNOWN){
+				message = gtk_message_dialog_new(GTK_WINDOW(dialog), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("File format is not supported"));
+				gtk_window_set_title(GTK_WINDOW(message), _("Import"));
+				gtk_dialog_run(GTK_DIALOG(message));
+				gtk_widget_destroy(message);
+			}else{
+				for (int i = 0; formats[i].name; ++i) {
+					if (formats[i].type == type){
+						struct ColorList *color_list_arg;
+						color_list_arg = gs->colors;
+						if (formats[i].import_function(color_list_arg, filename) == 0){
+							finished = true;
+						}else{
+							message = gtk_message_dialog_new(GTK_WINDOW(dialog), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("File could not be imported"));
+							gtk_window_set_title(GTK_WINDOW(message), _("Import"));
+							gtk_dialog_run(GTK_DIALOG(message));
+							gtk_widget_destroy(message);
+						}
+
+						const char *identification = (const char*)g_object_get_data(G_OBJECT(gtk_file_chooser_get_filter(GTK_FILE_CHOOSER(dialog))), "identification");
+						dynv_set_string(gs->params, "gpick.import.filter", identification);
+						break;
 					}
-					dynv_set_string(gs->params, "gpick.import.filter", formats[i].pattern);
 				}
 			}
 
